@@ -35,10 +35,14 @@ from glance.common import wsgi
 import glance.db
 from glance import image_cache
 from glance import notifier
+from glance.openstack.common import gettextutils
 import glance.openstack.common.log as logging
 import glance.registry.client.v1.api as registry
 
 LOG = logging.getLogger(__name__)
+_LI = gettextutils._LI
+_LE = gettextutils._LE
+_LW = gettextutils._LW
 
 PATTERNS = {
     ('v1', 'GET'): re.compile(r'^/v1/images/([^\/]+)$'),
@@ -54,7 +58,7 @@ class CacheFilter(wsgi.Middleware):
         self.cache = image_cache.ImageCache()
         self.serializer = images.ImageSerializer()
         self.policy = policy.Enforcer()
-        LOG.info(_("Initialized image cache middleware"))
+        LOG.info(_LI("Initialized image cache middleware"))
         super(CacheFilter, self).__init__(app)
 
     def _verify_metadata(self, image_meta):
@@ -63,7 +67,7 @@ class CacheFilter(wsgi.Middleware):
         """
         # NOTE: admins can see image metadata in the v1 API, but shouldn't
         # be able to download the actual image data.
-        if image_meta['deleted']:
+        if image_meta['status'] == 'deleted' and image_meta['deleted']:
             raise exception.NotFound()
 
         if not image_meta['size']:
@@ -91,12 +95,44 @@ class CacheFilter(wsgi.Middleware):
             else:
                 return (version, method, image_id)
 
-    def _enforce(self, req, action):
+    def _enforce(self, req, action, target=None):
         """Authorize an action against our policies"""
+        if target is None:
+            target = {}
         try:
-            self.policy.enforce(req.context, action, {})
+            self.policy.enforce(req.context, action, target)
         except exception.Forbidden as e:
-            raise webob.exc.HTTPForbidden(explanation=unicode(e), request=req)
+            raise webob.exc.HTTPForbidden(explanation=e.msg, request=req)
+
+    def _get_v1_image_metadata(self, request, image_id):
+        """
+        Retrieves image metadata using registry for v1 api and creates
+        dictionary-like mash-up of image core and custom properties.
+        """
+        try:
+            image_metadata = registry.get_image_metadata(request.context,
+                                                         image_id)
+            return utils.create_mashup_dict(image_metadata)
+        except exception.NotFound as e:
+            LOG.debug("No metadata found for image '%s'" % image_id)
+            raise webob.exc.HTTPNotFound(explanation=e.msg, request=request)
+
+    def _get_v2_image_metadata(self, request, image_id):
+        """
+        Retrieves image and for v2 api and creates adapter like object
+        to access image core or custom properties on request.
+        """
+        db_api = glance.db.get_api()
+        image_repo = glance.db.ImageRepo(request.context, db_api)
+        try:
+            image = image_repo.get(image_id)
+            # Storing image object in request as it is required in
+            # _process_v2_request call.
+            request.environ['api.cache.image'] = image
+
+            return policy.ImageTarget(image)
+        except exception.NotFound as e:
+            raise webob.exc.HTTPNotFound(explanation=e.msg, request=request)
 
     def process_request(self, request):
         """
@@ -112,53 +148,56 @@ class CacheFilter(wsgi.Middleware):
             # Trying to unpack None raises this exception
             return None
 
-        self._stash_request_info(request, image_id, method)
+        self._stash_request_info(request, image_id, method, version)
 
         if request.method != 'GET' or not self.cache.is_cached(image_id):
             return None
-
+        method = getattr(self, '_get_%s_image_metadata' % version)
+        image_metadata = method(request, image_id)
         try:
-            self._enforce(request, 'download_image')
-        except webob.exc.HTTPForbidden:
+            self._enforce(request, 'download_image', target=image_metadata)
+        except exception.Forbidden:
             return None
 
-        LOG.debug(_("Cache hit for image '%s'"), image_id)
+        LOG.debug("Cache hit for image '%s'", image_id)
         image_iterator = self.get_from_cache(image_id)
         method = getattr(self, '_process_%s_request' % version)
 
         try:
-            return method(request, image_id, image_iterator)
+            return method(request, image_id, image_iterator, image_metadata)
         except exception.NotFound:
-            msg = _("Image cache contained image file for image '%s', "
-                    "however the registry did not contain metadata for "
-                    "that image!") % image_id
+            msg = _LE("Image cache contained image file for image '%s', "
+                      "however the registry did not contain metadata for "
+                      "that image!") % image_id
             LOG.error(msg)
             self.cache.delete_cached_image(image_id)
 
     @staticmethod
-    def _stash_request_info(request, image_id, method):
+    def _stash_request_info(request, image_id, method, version):
         """
-        Preserve the image id and request method for later retrieval
+        Preserve the image id, version and request method for later retrieval
         """
         request.environ['api.cache.image_id'] = image_id
         request.environ['api.cache.method'] = method
+        request.environ['api.cache.version'] = version
 
     @staticmethod
     def _fetch_request_info(request):
         """
-        Preserve the cached image id for consumption by the
+        Preserve the cached image id, version for consumption by the
         process_response method of this middleware
         """
         try:
             image_id = request.environ['api.cache.image_id']
             method = request.environ['api.cache.method']
+            version = request.environ['api.cache.version']
         except KeyError:
             return None
         else:
-            return (image_id, method)
+            return (image_id, method, version)
 
-    def _process_v1_request(self, request, image_id, image_iterator):
-        image_meta = registry.get_image_metadata(request.context, image_id)
+    def _process_v1_request(self, request, image_id, image_iterator,
+                            image_meta):
         # Don't display location
         if 'location' in image_meta:
             del image_meta['location']
@@ -172,16 +211,14 @@ class CacheFilter(wsgi.Middleware):
         }
         return self.serializer.show(response, raw_response)
 
-    def _process_v2_request(self, request, image_id, image_iterator):
+    def _process_v2_request(self, request, image_id, image_iterator,
+                            image_meta):
         # We do some contortions to get the image_metadata so
         # that we can provide it to 'size_checked_iter' which
         # will generate a notification.
         # TODO(mclaren): Make notification happen more
         # naturally once caching is part of the domain model.
-        db_api = glance.db.get_api()
-        image_repo = glance.db.ImageRepo(request.context, db_api)
-        image = image_repo.get(image_id)
-        image_meta = glance.notifier.format_image_notification(image)
+        image = request.environ['api.cache.image']
         self._verify_metadata(image_meta)
         response = webob.Response(request=request)
         response.app_iter = size_checked_iter(response, image_meta,
@@ -213,7 +250,8 @@ class CacheFilter(wsgi.Middleware):
             return resp
 
         try:
-            (image_id, method) = self._fetch_request_info(resp.request)
+            (image_id, method, version) = self._fetch_request_info(
+                resp.request)
         except TypeError:
             return resp
 
@@ -227,34 +265,38 @@ class CacheFilter(wsgi.Middleware):
         try:
             process_response_method = getattr(self, method_str)
         except AttributeError:
-            LOG.error(_('could not find %s') % method_str)
+            LOG.error(_LE('could not find %s') % method_str)
             # Nothing to do here, move along
             return resp
         else:
-            return process_response_method(resp, image_id)
+            return process_response_method(resp, image_id, version=version)
 
-    def _process_DELETE_response(self, resp, image_id):
+    def _process_DELETE_response(self, resp, image_id, version=None):
         if self.cache.is_cached(image_id):
-            LOG.debug(_("Removing image %s from cache"), image_id)
+            LOG.debug("Removing image %s from cache", image_id)
             self.cache.delete_cached_image(image_id)
         return resp
 
-    def _process_GET_response(self, resp, image_id):
+    def _process_GET_response(self, resp, image_id, version=None):
         image_checksum = resp.headers.get('Content-MD5')
-
         if not image_checksum:
             # API V1 stores the checksum in a different header:
             image_checksum = resp.headers.get('x-image-meta-checksum')
 
         if not image_checksum:
-            LOG.error(_("Checksum header is missing."))
+            LOG.error(_LE("Checksum header is missing."))
 
+        # fetch image_meta on the basis of version
+        image_metadata = None
+        if version:
+            method = getattr(self, '_get_%s_image_metadata' % version)
+            image_metadata = method(resp.request, image_id)
         # NOTE(zhiyan): image_cache return a generator object and set to
         # response.app_iter, it will be called by eventlet.wsgi later.
         # So we need enforce policy firstly but do it by application
         # since eventlet.wsgi could not catch webob.exc.HTTPForbidden and
         # return 403 error to client then.
-        self._enforce(resp.request, 'download_image')
+        self._enforce(resp.request, 'download_image', target=image_metadata)
 
         resp.app_iter = self.cache.get_caching_iter(image_id, image_checksum,
                                                     resp.app_iter)
