@@ -80,7 +80,7 @@ class Enforcer(object):
             rule_type = "default "
 
         text_rules = dict((k, str(v)) for k, v in rules.items())
-        msg = (_('Loaded %(rule_type)spolicy rules: %(text_rules)s') %
+        msg = ('Loaded %(rule_type)spolicy rules: %(text_rules)s' %
                {'rule_type': rule_type, 'text_rules': text_rules})
         LOG.debug(msg)
 
@@ -103,7 +103,7 @@ class Enforcer(object):
         """
         mtime = os.path.getmtime(self.policy_path)
         if not self.policy_file_contents or mtime != self.policy_file_mtime:
-            LOG.debug(_("Loading policy from %s") % self.policy_path)
+            LOG.debug("Loading policy from %s" % self.policy_path)
             with open(self.policy_path) as fap:
                 raw_contents = fap.read()
                 rules_dict = jsonutils.loads(raw_contents)
@@ -135,7 +135,7 @@ class Enforcer(object):
 
            :param context: Glance request context
            :param action: String representing the action to be checked
-           :param object: Dictionary representing the object of the action.
+           :param target: Dictionary representing the object of the action.
            :raises: `glance.common.exception.Forbidden`
            :returns: A non-False value if access is allowed.
         """
@@ -147,7 +147,7 @@ class Enforcer(object):
 
            :param context: Glance request context
            :param action: String representing the action to be checked
-           :param object: Dictionary representing the object of the action.
+           :param target: Dictionary representing the object of the action.
            :returns: A non-False value if access is allowed.
         """
         return self._check(context, action, target)
@@ -182,9 +182,9 @@ class ImageRepoProxy(glance.domain.proxy.Repo):
         self.policy.enforce(self.context, 'get_images', {})
         return super(ImageRepoProxy, self).list(*args, **kwargs)
 
-    def save(self, image, from_state=None):
+    def save(self, image):
         self.policy.enforce(self.context, 'modify_image', {})
-        return super(ImageRepoProxy, self).save(image, from_state=from_state)
+        return super(ImageRepoProxy, self).save(image)
 
     def add(self, image):
         self.policy.enforce(self.context, 'add_image', {})
@@ -230,7 +230,9 @@ class ImageProxy(glance.domain.proxy.Image):
         return self.image.delete()
 
     def get_data(self, *args, **kwargs):
-        self.policy.enforce(self.context, 'download_image', {})
+        target = ImageTarget(self.image)
+        self.policy.enforce(self.context, 'download_image',
+                            target=target)
         return self.image.get_data(*args, **kwargs)
 
     def set_data(self, *args, **kwargs):
@@ -283,9 +285,9 @@ class ImageMemberRepoProxy(glance.domain.proxy.Repo):
         self.policy.enforce(self.context, 'get_member', {})
         return self.member_repo.get(member_id)
 
-    def save(self, member, from_state=None):
+    def save(self, member):
         self.policy.enforce(self.context, 'modify_member', {})
-        self.member_repo.save(member, from_state=from_state)
+        self.member_repo.save(member)
 
     def list(self, *args, **kwargs):
         self.policy.enforce(self.context, 'get_members', {})
@@ -353,17 +355,14 @@ class TaskProxy(glance.domain.proxy.Task):
         self.policy = policy
         super(TaskProxy, self).__init__(task)
 
-    def run(self, executor):
-        self.base.run(executor)
 
+class TaskStubProxy(glance.domain.proxy.TaskStub):
 
-class TaskDetailsProxy(glance.domain.proxy.TaskDetails):
-
-    def __init__(self, task_details, context, policy):
-        self.task_details = task_details
+    def __init__(self, task_stub, context, policy):
+        self.task_stub = task_stub
         self.context = context
         self.policy = policy
-        super(TaskDetailsProxy, self).__init__(task_details)
+        super(TaskStubProxy, self).__init__(task_stub)
 
 
 class TaskRepoProxy(glance.domain.proxy.TaskRepo):
@@ -376,25 +375,36 @@ class TaskRepoProxy(glance.domain.proxy.TaskRepo):
         super(TaskRepoProxy,
               self).__init__(task_repo,
                              task_proxy_class=TaskProxy,
-                             task_proxy_kwargs=proxy_kwargs,
-                             task_details_proxy_class=TaskDetailsProxy,
-                             task_details_proxy_kwargs=proxy_kwargs)
+                             task_proxy_kwargs=proxy_kwargs)
 
-    def get_task_and_details(self, task_id):
+    def get(self, task_id):
         self.policy.enforce(self.context, 'get_task', {})
-        return super(TaskRepoProxy, self).get_task_and_details(task_id)
+        return super(TaskRepoProxy, self).get(task_id)
 
-    def list_tasks(self, *args, **kwargs):
-        self.policy.enforce(self.context, 'get_tasks', {})
-        return super(TaskRepoProxy, self).list_tasks(*args, **kwargs)
-
-    def add(self, task, task_details=None):
+    def add(self, task):
         self.policy.enforce(self.context, 'add_task', {})
-        super(TaskRepoProxy, self).add(task, task_details)
+        super(TaskRepoProxy, self).add(task)
 
-    def save(self, task, task_details=None):
+    def save(self, task):
         self.policy.enforce(self.context, 'modify_task', {})
-        super(TaskRepoProxy, self).save(task, task_details)
+        super(TaskRepoProxy, self).save(task)
+
+
+class TaskStubRepoProxy(glance.domain.proxy.TaskStubRepo):
+
+    def __init__(self, task_stub_repo, context, task_policy):
+        self.context = context
+        self.policy = task_policy
+        self.task_stub_repo = task_stub_repo
+        proxy_kwargs = {'context': self.context, 'policy': self.policy}
+        super(TaskStubRepoProxy,
+              self).__init__(task_stub_repo,
+                             task_stub_proxy_class=TaskStubProxy,
+                             task_stub_proxy_kwargs=proxy_kwargs)
+
+    def list(self, *args, **kwargs):
+        self.policy.enforce(self.context, 'get_tasks', {})
+        return super(TaskStubRepoProxy, self).list(*args, **kwargs)
 
 
 class TaskFactoryProxy(glance.domain.proxy.TaskFactory):
@@ -407,6 +417,244 @@ class TaskFactoryProxy(glance.domain.proxy.TaskFactory):
         super(TaskFactoryProxy, self).__init__(
             task_factory,
             task_proxy_class=TaskProxy,
-            task_proxy_kwargs=proxy_kwargs,
-            task_details_proxy_class=TaskDetailsProxy,
-            task_details_proxy_kwargs=proxy_kwargs)
+            task_proxy_kwargs=proxy_kwargs)
+
+
+class ImageTarget(object):
+
+    def __init__(self, image):
+        """
+        Initialize the object
+
+        :param image: Image object
+        """
+        self.image = image
+
+    def __getitem__(self, key):
+        """
+        Returns the value of 'key' from the image if image has that attribute
+        else tries to retrieve value from the extra_properties of image.
+
+        :param key: value to retrieve
+        """
+        # Need to change the key 'id' to 'image_id' as Image object has
+        # attribute as 'image_id' in case of V2.
+        if key == 'id':
+            key = 'image_id'
+
+        if hasattr(self.image, key):
+            return getattr(self.image, key)
+        else:
+            return self.image.extra_properties[key]
+
+
+#Metadef Namespace classes
+class MetadefNamespaceProxy(glance.domain.proxy.MetadefNamespace):
+
+    def __init__(self, namespace, context, policy):
+        self.namespace_input = namespace
+        self.context = context
+        self.policy = policy
+        super(MetadefNamespaceProxy, self).__init__(namespace)
+
+
+class MetadefNamespaceRepoProxy(glance.domain.proxy.MetadefNamespaceRepo):
+
+    def __init__(self, namespace_repo, context, namespace_policy):
+        self.context = context
+        self.policy = namespace_policy
+        self.namespace_repo = namespace_repo
+        proxy_kwargs = {'context': self.context, 'policy': self.policy}
+        super(MetadefNamespaceRepoProxy,
+              self).__init__(namespace_repo,
+                             namespace_proxy_class=MetadefNamespaceProxy,
+                             namespace_proxy_kwargs=proxy_kwargs)
+
+    def get(self, namespace):
+        self.policy.enforce(self.context, 'get_metadef_namespace', {})
+        return super(MetadefNamespaceRepoProxy, self).get(namespace)
+
+    def list(self, *args, **kwargs):
+        self.policy.enforce(self.context, 'get_metadef_namespaces', {})
+        return super(MetadefNamespaceRepoProxy, self).list(*args, **kwargs)
+
+    def save(self, namespace):
+        self.policy.enforce(self.context, 'modify_metadef_namespace', {})
+        return super(MetadefNamespaceRepoProxy, self).save(namespace)
+
+    def add(self, namespace):
+        self.policy.enforce(self.context, 'add_metadef_namespace', {})
+        return super(MetadefNamespaceRepoProxy, self).add(namespace)
+
+
+class MetadefNamespaceFactoryProxy(
+        glance.domain.proxy.MetadefNamespaceFactory):
+
+    def __init__(self, meta_namespace_factory, context, policy):
+        self.meta_namespace_factory = meta_namespace_factory
+        self.context = context
+        self.policy = policy
+        proxy_kwargs = {'context': self.context, 'policy': self.policy}
+        super(MetadefNamespaceFactoryProxy, self).__init__(
+            meta_namespace_factory,
+            meta_namespace_proxy_class=MetadefNamespaceProxy,
+            meta_namespace_proxy_kwargs=proxy_kwargs)
+
+
+#Metadef Object classes
+class MetadefObjectProxy(glance.domain.proxy.MetadefObject):
+
+    def __init__(self, meta_object, context, policy):
+        self.meta_object = meta_object
+        self.context = context
+        self.policy = policy
+        super(MetadefObjectProxy, self).__init__(meta_object)
+
+
+class MetadefObjectRepoProxy(glance.domain.proxy.MetadefObjectRepo):
+
+    def __init__(self, object_repo, context, object_policy):
+        self.context = context
+        self.policy = object_policy
+        self.object_repo = object_repo
+        proxy_kwargs = {'context': self.context, 'policy': self.policy}
+        super(MetadefObjectRepoProxy,
+              self).__init__(object_repo,
+                             object_proxy_class=MetadefObjectProxy,
+                             object_proxy_kwargs=proxy_kwargs)
+
+    def get(self, namespace, object_name):
+        self.policy.enforce(self.context, 'get_metadef_object', {})
+        return super(MetadefObjectRepoProxy, self).get(namespace, object_name)
+
+    def list(self, *args, **kwargs):
+        self.policy.enforce(self.context, 'get_metadef_objects', {})
+        return super(MetadefObjectRepoProxy, self).list(*args, **kwargs)
+
+    def save(self, meta_object):
+        self.policy.enforce(self.context, 'modify_metadef_object', {})
+        return super(MetadefObjectRepoProxy, self).save(meta_object)
+
+    def add(self, meta_object):
+        self.policy.enforce(self.context, 'add_metadef_object', {})
+        return super(MetadefObjectRepoProxy, self).add(meta_object)
+
+
+class MetadefObjectFactoryProxy(glance.domain.proxy.MetadefObjectFactory):
+
+    def __init__(self, meta_object_factory, context, policy):
+        self.meta_object_factory = meta_object_factory
+        self.context = context
+        self.policy = policy
+        proxy_kwargs = {'context': self.context, 'policy': self.policy}
+        super(MetadefObjectFactoryProxy, self).__init__(
+            meta_object_factory,
+            meta_object_proxy_class=MetadefObjectProxy,
+            meta_object_proxy_kwargs=proxy_kwargs)
+
+
+#Metadef ResourceType classes
+class MetadefResourceTypeProxy(glance.domain.proxy.MetadefResourceType):
+
+    def __init__(self, meta_resource_type, context, policy):
+        self.meta_resource_type = meta_resource_type
+        self.context = context
+        self.policy = policy
+        super(MetadefResourceTypeProxy, self).__init__(meta_resource_type)
+
+
+class MetadefResourceTypeRepoProxy(
+        glance.domain.proxy.MetadefResourceTypeRepo):
+
+    def __init__(self, resource_type_repo, context, resource_type_policy):
+        self.context = context
+        self.policy = resource_type_policy
+        self.resource_type_repo = resource_type_repo
+        proxy_kwargs = {'context': self.context, 'policy': self.policy}
+        super(MetadefResourceTypeRepoProxy, self).__init__(
+            resource_type_repo,
+            resource_type_proxy_class=MetadefResourceTypeProxy,
+            resource_type_proxy_kwargs=proxy_kwargs)
+
+    def list(self, *args, **kwargs):
+        self.policy.enforce(self.context, 'list_metadef_resource_types', {})
+        return super(MetadefResourceTypeRepoProxy, self).list(*args, **kwargs)
+
+    def get(self, *args, **kwargs):
+        self.policy.enforce(self.context, 'get_metadef_resource_type', {})
+        return super(MetadefResourceTypeRepoProxy, self).get(*args, **kwargs)
+
+    def add(self, resource_type):
+        self.policy.enforce(self.context,
+                            'add_metadef_resource_type_association', {})
+        return super(MetadefResourceTypeRepoProxy, self).add(resource_type)
+
+
+class MetadefResourceTypeFactoryProxy(
+        glance.domain.proxy.MetadefResourceTypeFactory):
+
+    def __init__(self, resource_type_factory, context, policy):
+        self.resource_type_factory = resource_type_factory
+        self.context = context
+        self.policy = policy
+        proxy_kwargs = {'context': self.context, 'policy': self.policy}
+        super(MetadefResourceTypeFactoryProxy, self).__init__(
+            resource_type_factory,
+            resource_type_proxy_class=MetadefResourceTypeProxy,
+            resource_type_proxy_kwargs=proxy_kwargs)
+
+
+#Metadef namespace properties classes
+class MetadefPropertyProxy(glance.domain.proxy.MetadefProperty):
+
+    def __init__(self, namespace_property, context, policy):
+        self.namespace_property = namespace_property
+        self.context = context
+        self.policy = policy
+        super(MetadefPropertyProxy, self).__init__(namespace_property)
+
+
+class MetadefPropertyRepoProxy(glance.domain.proxy.MetadefPropertyRepo):
+
+    def __init__(self, property_repo, context, object_policy):
+        self.context = context
+        self.policy = object_policy
+        self.property_repo = property_repo
+        proxy_kwargs = {'context': self.context, 'policy': self.policy}
+        super(MetadefPropertyRepoProxy, self).__init__(
+            property_repo,
+            property_proxy_class=MetadefPropertyProxy,
+            property_proxy_kwargs=proxy_kwargs)
+
+    def get(self, namespace, property_name):
+        self.policy.enforce(self.context, 'get_metadef_property', {})
+        return super(MetadefPropertyRepoProxy, self).get(namespace,
+                                                         property_name)
+
+    def list(self, *args, **kwargs):
+        self.policy.enforce(self.context, 'get_metadef_properties', {})
+        return super(MetadefPropertyRepoProxy, self).list(
+            *args, **kwargs)
+
+    def save(self, namespace_property):
+        self.policy.enforce(self.context, 'modify_metadef_property', {})
+        return super(MetadefPropertyRepoProxy, self).save(
+            namespace_property)
+
+    def add(self, namespace_property):
+        self.policy.enforce(self.context, 'add_metadef_property', {})
+        return super(MetadefPropertyRepoProxy, self).add(
+            namespace_property)
+
+
+class MetadefPropertyFactoryProxy(glance.domain.proxy.MetadefPropertyFactory):
+
+    def __init__(self, namespace_property_factory, context, policy):
+        self.namespace_property_factory = namespace_property_factory
+        self.context = context
+        self.policy = policy
+        proxy_kwargs = {'context': self.context, 'policy': self.policy}
+        super(MetadefPropertyFactoryProxy, self).__init__(
+            namespace_property_factory,
+            property_proxy_class=MetadefPropertyProxy,
+            property_proxy_kwargs=proxy_kwargs)
