@@ -20,10 +20,14 @@
 import copy
 
 import eventlet
+import glance_store as store
+import glance_store.location
 from oslo.config import cfg
+import six.moves.urllib.parse as urlparse
 from webob.exc import HTTPBadRequest
 from webob.exc import HTTPConflict
 from webob.exc import HTTPForbidden
+from webob.exc import HTTPMethodNotAllowed
 from webob.exc import HTTPNotFound
 from webob.exc import HTTPRequestEntityTooLarge
 from webob import Response
@@ -38,18 +42,17 @@ from glance.common import exception
 from glance.common import property_utils
 from glance.common import utils
 from glance.common import wsgi
+from glance.i18n import _LE
 from glance import notifier
+from glance.openstack.common import excutils
+from glance.openstack.common import gettextutils
 import glance.openstack.common.log as logging
 from glance.openstack.common import strutils
 import glance.registry.client.v1.api as registry
-from glance.store import get_from_backend
-from glance.store import get_known_schemes
-from glance.store import get_size_from_backend
-from glance.store import get_store_from_location
-from glance.store import get_store_from_scheme
-from glance.store import validate_external_location
 
 LOG = logging.getLogger(__name__)
+_LI = gettextutils._LI
+_LW = gettextutils._LW
 SUPPORTED_PARAMS = glance.api.v1.SUPPORTED_PARAMS
 SUPPORTED_FILTERS = glance.api.v1.SUPPORTED_FILTERS
 ACTIVE_IMMUTABLE = glance.api.v1.ACTIVE_IMMUTABLE
@@ -142,10 +145,12 @@ class Controller(controller.BaseController):
         else:
             self.prop_enforcer = None
 
-    def _enforce(self, req, action):
+    def _enforce(self, req, action, target=None):
         """Authorize an action against our policies"""
+        if target is None:
+            target = {}
         try:
-            self.policy.enforce(req.context, action, {})
+            self.policy.enforce(req.context, action, target)
         except exception.Forbidden:
             raise HTTPForbidden()
 
@@ -190,7 +195,7 @@ class Controller(controller.BaseController):
             for key in create_props:
                 if (self.prop_enforcer.check_property_rules(
                         key, 'create', req.context) is False):
-                    msg = _("Property '%s' is protected") % key
+                    msg = "Property '%s' is protected" % key
                     LOG.debug(msg)
                     raise HTTPForbidden(explanation=msg,
                                         request=req,
@@ -234,7 +239,7 @@ class Controller(controller.BaseController):
                         key, 'update', req.context) is False and
                         image_meta['properties'][key] !=
                         orig_meta['properties'][key]) or not has_read):
-                    msg = _("Property '%s' is protected") % key
+                    msg = "Property '%s' is protected" % key
                     LOG.debug(msg)
                     raise HTTPForbidden(explanation=msg,
                                         request=req,
@@ -268,7 +273,7 @@ class Controller(controller.BaseController):
                         orig_meta['properties'][key]
                 elif (self.prop_enforcer.check_property_rules(
                         key, 'delete', req.context) is False):
-                    msg = _("Property '%s' is protected") % key
+                    msg = "Property '%s' is protected" % key
                     LOG.debug(msg)
                     raise HTTPForbidden(explanation=msg,
                                         request=req,
@@ -302,7 +307,7 @@ class Controller(controller.BaseController):
         try:
             images = registry.get_images_list(req.context, **params)
         except exception.Invalid as e:
-            raise HTTPBadRequest(explanation="%s" % e)
+            raise HTTPBadRequest(explanation=e.msg, request=req)
 
         return dict(images=images)
 
@@ -330,6 +335,12 @@ class Controller(controller.BaseController):
                  'properties': {'distro': 'Ubuntu 10.04 LTS', ...}}, ...
             ]}
         """
+        if req.method == 'HEAD':
+            msg = (_("This operation is currently not permitted on "
+                     "Glance images details."))
+            raise HTTPMethodNotAllowed(explanation=msg,
+                                       headers={'Allow': 'GET'},
+                                       body_template='${explanation}')
         self._enforce(req, 'get_images')
         params = self._get_query_params(req)
         try:
@@ -343,7 +354,7 @@ class Controller(controller.BaseController):
                 redact_loc(image, copy_dict=False)
                 self._enforce_read_protected_props(image, req)
         except exception.Invalid as e:
-            raise HTTPBadRequest(explanation="%s" % e)
+            raise HTTPBadRequest(explanation=e.msg, request=req)
         return dict(images=images)
 
     def _get_query_params(self, req):
@@ -404,19 +415,26 @@ class Controller(controller.BaseController):
     @staticmethod
     def _validate_source(source, req):
         """
-        To validate if external sources (as specified via the location
-        or copy-from headers) are supported. Otherwise we reject
-        with 400 "Bad Request".
+        External sources (as specified via the location or copy-from headers)
+        are supported only over non-local store types, i.e. S3, Swift, HTTP.
+        Note the absence of 'file://' for security reasons, see LP bug #942118.
+        'swift+config://' is also absent for security reasons, see LP bug
+        #1334196.
+        If the above constraint is violated, we reject with 400 "Bad Request".
         """
         if source:
-            if validate_external_location(source):
-                return source
-            else:
-                msg = _("External source are not supported: '%s'") % source
-                LOG.debug(msg)
-                raise HTTPBadRequest(explanation=msg,
-                                     request=req,
-                                     content_type="text/plain")
+            pieces = urlparse.urlparse(source)
+            schemes = [scheme for scheme in store.get_known_schemes()
+                       if scheme != 'file' and scheme != 'swift+config']
+            for scheme in schemes:
+                if pieces.scheme == scheme:
+                    return source
+            msg = ("External sourcing not supported for "
+                   "store '%s'" % pieces.scheme)
+            LOG.debug(msg)
+            raise HTTPBadRequest(explanation=msg,
+                                 request=req,
+                                 content_type="text/plain")
 
     @staticmethod
     def _copy_from(req):
@@ -431,9 +449,16 @@ class Controller(controller.BaseController):
         return Controller._validate_source(source, req)
 
     @staticmethod
-    def _get_from_store(context, where):
+    def _get_from_store(context, where, dest=None):
         try:
-            image_data, image_size = get_from_backend(context, where)
+            loc = glance_store.location.get_location_from_uri(where)
+            src_store = store.get_store_from_uri(where)
+
+            if dest is not None:
+                src_store.READ_CHUNKSIZE = dest.WRITE_CHUNKSIZE
+
+            image_data, image_size = src_store.get(loc, context=context)
+
         except exception.NotFound as e:
             raise HTTPNotFound(explanation=e.msg)
         image_size = int(image_size) if image_size else None
@@ -449,9 +474,20 @@ class Controller(controller.BaseController):
 
         :raises HTTPNotFound if image is not available to user
         """
+
         self._enforce(req, 'get_image')
-        self._enforce(req, 'download_image')
-        image_meta = self.get_active_image_meta_or_404(req, id)
+
+        try:
+            image_meta = self.get_active_image_meta_or_404(req, id)
+        except HTTPNotFound:
+            # provision for backward-compatibility breaking issue
+            # catch the 404 exception and raise it after enforcing
+            # the policy
+            with excutils.save_and_reraise_exception():
+                self._enforce(req, 'download_image')
+        else:
+            target = utils.create_mashup_dict(image_meta)
+            self._enforce(req, 'download_image', target=target)
 
         self._enforce_read_protected_props(image_meta, req)
 
@@ -462,7 +498,6 @@ class Controller(controller.BaseController):
                                                         image_meta['location'])
             image_iterator = utils.cooperative_iter(image_iterator)
             image_meta['size'] = size or image_meta['size']
-
         image_meta = redact_loc(image_meta)
         return {
             'image_iterator': image_iterator,
@@ -483,9 +518,9 @@ class Controller(controller.BaseController):
         :raises HTTPBadRequest if image metadata is not valid
         """
         location = self._external_source(image_meta, req)
-        store = image_meta.get('store')
-        if store and store not in get_known_schemes():
-            msg = _("Required store %s is invalid") % store
+        scheme = image_meta.get('store')
+        if scheme and scheme not in store.get_known_schemes():
+            msg = "Required store %s is invalid" % scheme
             LOG.debug(msg)
             raise HTTPBadRequest(explanation=msg,
                                  content_type='text/plain')
@@ -495,16 +530,16 @@ class Controller(controller.BaseController):
 
         if location:
             try:
-                store = get_store_from_location(location)
-            except exception.BadStoreUri:
-                msg = _("Invalid location %s") % location
+                backend = store.get_store_from_location(location)
+            except store.BadStoreUri:
+                msg = "Invalid location %s" % location
                 LOG.debug(msg)
                 raise HTTPBadRequest(explanation=msg,
                                      request=req,
                                      content_type="text/plain")
             # check the store exists before we hit the registry, but we
             # don't actually care what it is at this point
-            self.get_store_or_400(req, store)
+            self.get_store_or_400(req, backend)
 
             # retrieve the image size from remote store (if not provided)
             image_meta['size'] = self._get_size(req.context, image_meta,
@@ -520,21 +555,22 @@ class Controller(controller.BaseController):
             self.notifier.info("image.create", redact_loc(image_meta))
             return image_meta
         except exception.Duplicate:
-            msg = (_("An image with identifier %s already exists") %
+            msg = ("An image with identifier %s already exists" %
                    image_meta['id'])
             LOG.debug(msg)
             raise HTTPConflict(explanation=msg,
                                request=req,
                                content_type="text/plain")
         except exception.Invalid as e:
-            msg = _("Failed to reserve image. Got error: %(e)s") % {'e': e}
+            msg = (_("Failed to reserve image. Got error: %s") %
+                   utils.exception_to_str(e))
             for line in msg.split('\n'):
                 LOG.debug(line)
             raise HTTPBadRequest(explanation=msg,
                                  request=req,
                                  content_type="text/plain")
         except exception.Forbidden:
-            msg = _("Forbidden to reserve image.")
+            msg = "Forbidden to reserve image."
             LOG.debug(msg)
             raise HTTPForbidden(explanation=msg,
                                 request=req,
@@ -554,15 +590,23 @@ class Controller(controller.BaseController):
         :retval The location where the image was stored
         """
 
+        scheme = req.headers.get('x-image-meta-store',
+                                 CONF.glance_store.default_store)
+
+        store = self.get_store_or_400(req, scheme)
+
         copy_from = self._copy_from(req)
         if copy_from:
             try:
                 image_data, image_size = self._get_from_store(req.context,
-                                                              copy_from)
-            except Exception as e:
+                                                              copy_from,
+                                                              dest=store)
+            except Exception:
                 upload_utils.safe_kill(req, image_meta['id'], 'queued')
-                msg = _("Copy from external source failed: %s") % e
-                LOG.debug(msg)
+                msg = (_LE("Copy from external source '%(scheme)s' failed for "
+                           "image: %(image)s") %
+                       {'scheme': scheme, 'image': image_meta['id']})
+                LOG.exception(msg)
                 return
             image_meta['size'] = image_size or image_meta['size']
         else:
@@ -570,51 +614,43 @@ class Controller(controller.BaseController):
                 req.get_content_type(('application/octet-stream',))
             except exception.InvalidContentType:
                 upload_utils.safe_kill(req, image_meta['id'], 'queued')
-                msg = _("Content-Type must be application/octet-stream")
+                msg = "Content-Type must be application/octet-stream"
                 LOG.debug(msg)
                 raise HTTPBadRequest(explanation=msg)
 
             image_data = req.body_file
 
-        scheme = req.headers.get('x-image-meta-store', CONF.default_store)
-
-        store = self.get_store_or_400(req, scheme)
-
         image_id = image_meta['id']
-        LOG.debug(_("Setting image %s to status 'saving'"), image_id)
+        LOG.debug("Setting image %s to status 'saving'", image_id)
         registry.update_image_metadata(req.context, image_id,
                                        {'status': 'saving'})
 
-        LOG.debug(_("Uploading image data for image %(image_id)s "
-                    "to %(scheme)s store"), {'image_id': image_id,
-                                             'scheme': scheme})
+        LOG.debug("Uploading image data for image %(image_id)s "
+                  "to %(scheme)s store", {'image_id': image_id,
+                                          'scheme': scheme})
 
         self.notifier.info("image.prepare", redact_loc(image_meta))
 
-        image_meta, location, loc_meta = upload_utils.upload_data_to_store(
+        image_meta, location_data = upload_utils.upload_data_to_store(
             req, image_meta, image_data, store, self.notifier)
 
         self.notifier.info('image.upload', redact_loc(image_meta))
 
-        return location, loc_meta
+        return location_data
 
-    def _activate(self, req, image_id, location, location_metadata=None,
-                  from_state=None):
+    def _activate(self, req, image_id, location_data, from_state=None):
         """
         Sets the image status to `active` and the image's location
         attribute.
 
         :param req: The WSGI/Webob Request object
         :param image_id: Opaque image identifier
-        :param location: Location of where Glance stored this image
-        :param location_metadata: a dictionary of storage specific information
+        :param location_data: Location of where Glance stored this image
         """
         image_meta = {}
-        image_meta['location'] = location
+        image_meta['location'] = location_data['url']
         image_meta['status'] = 'active'
-        if location_metadata:
-            image_meta['location_data'] = [{'url': location,
-                                            'metadata': location_metadata}]
+        image_meta['location_data'] = [location_data]
 
         try:
             s = from_state
@@ -626,17 +662,16 @@ class Controller(controller.BaseController):
             self.notifier.info("image.update", redact_loc(image_meta_data))
             return image_meta_data
         except exception.Duplicate:
-            # Delete image data since it has been supersceded by another
-            # upload.
-            LOG.debug(_("duplicate operation - deleting image data for %(id)s "
-                      "(location:%(location)s)") %
-                      {'id': image_id, 'location': image_meta['location']})
-            upload_utils.initiate_deletion(req, image_meta['location'],
-                                           image_id, CONF.delayed_delete)
-            # Then propagate the exception.
-            raise
+            with excutils.save_and_reraise_exception():
+                # Delete image data since it has been supersceded by another
+                # upload and re-raise.
+                LOG.debug("duplicate operation - deleting image data for "
+                          " %(id)s (location:%(location)s)" %
+                          {'id': image_id, 'location': image_meta['location']})
+                upload_utils.initiate_deletion(req, location_data, image_id)
         except exception.Invalid as e:
-            msg = _("Failed to activate image. Got error: %(e)s") % {'e': e}
+            msg = ("Failed to activate image. Got error: %s" %
+                   utils.exception_to_str(e))
             LOG.debug(msg)
             raise HTTPBadRequest(explanation=msg,
                                  request=req,
@@ -653,24 +688,40 @@ class Controller(controller.BaseController):
 
         :retval Mapping of updated image data
         """
+        location_data = self._upload(req, image_meta)
         image_id = image_meta['id']
-        # This is necessary because of a bug in Webob 1.0.2 - 1.0.7
-        # See: https://bitbucket.org/ianb/webob/
-        # issue/12/fix-for-issue-6-broke-chunked-transfer
-        req.is_body_readable = True
-        location, location_metadata = self._upload(req, image_meta)
-        return self._activate(req,
-                              image_id,
-                              location,
-                              location_metadata,
-                              from_state='saving') if location else None
+        LOG.info(_LI("Uploaded data of image %s from request "
+                     "payload successfully.") % image_id)
+
+        if location_data:
+            try:
+                image_meta = self._activate(req,
+                                            image_id,
+                                            location_data,
+                                            from_state='saving')
+            except Exception as e:
+                with excutils.save_and_reraise_exception():
+                    if not isinstance(e, exception.Duplicate):
+                        # NOTE(zhiyan): Delete image data since it has already
+                        # been added to store by above _upload() call.
+                        LOG.warn(_LW("Failed to activate image %s in "
+                                     "registry. About to delete image "
+                                     "bits from store and update status "
+                                     "to 'killed'.") % image_id)
+                        upload_utils.initiate_deletion(req, location_data,
+                                                       image_id)
+                        upload_utils.safe_kill(req, image_id, 'saving')
+        else:
+            image_meta = None
+
+        return image_meta
 
     def _get_size(self, context, image_meta, location):
         # retrieve the image size from remote store (if not provided)
         try:
             return (image_meta.get('size', 0) or
-                    get_size_from_backend(context, location))
-        except (exception.NotFound, exception.BadStoreUri) as e:
+                    store.get_size_from_backend(location, context=context))
+        except (exception.NotFound, store.BadStoreUri) as e:
             LOG.debug(e)
             raise HTTPBadRequest(explanation=e.msg, content_type="text/plain")
 
@@ -679,7 +730,7 @@ class Controller(controller.BaseController):
         location = image_meta.get('location')
         sources = filter(lambda x: x, (copy_from, location, image_data))
         if len(sources) >= 2:
-            msg = _("It's invalid to provide multiple image sources.")
+            msg = "It's invalid to provide multiple image sources."
             LOG.debug(msg)
             raise HTTPBadRequest(explanation=msg,
                                  request=req,
@@ -690,7 +741,7 @@ class Controller(controller.BaseController):
                                                              image_meta)
             image_meta = self._upload_and_activate(req, image_meta)
         elif copy_from:
-            msg = _('Triggering asynchronous copy from external source')
+            msg = _LI('Triggering asynchronous copy from external source')
             LOG.info(msg)
             self.pool.spawn_n(self._upload_and_activate, req, image_meta)
         else:
@@ -698,23 +749,31 @@ class Controller(controller.BaseController):
                 self._validate_image_for_activation(req, image_id, image_meta)
                 image_size_meta = image_meta.get('size')
                 if image_size_meta:
-                    image_size_store = get_size_from_backend(req.context,
-                                                             location)
+                    try:
+                        image_size_store = store.get_size_from_backend(
+                            location, req.context)
+                    except (store.BadStoreUri, store.UnknownScheme) as e:
+                        LOG.debug(utils.exception_to_str(e))
+                        raise HTTPBadRequest(explanation=e.msg,
+                                             request=req,
+                                             content_type="text/plain")
                     # NOTE(zhiyan): A returned size of zero usually means
                     # the driver encountered an error. In this case the
                     # size provided by the client will be used as-is.
                     if (image_size_store and
                             image_size_store != image_size_meta):
-                        msg = _("Provided image size must match the stored "
-                                "image size. (provided size: %(ps)d, "
-                                "stored size: %(ss)d)") % {
-                                    "ps": image_size_meta,
-                                    "ss": image_size_store}
+                        msg = ("Provided image size must match the stored "
+                               "image size. (provided size: %(ps)d, "
+                               "stored size: %(ss)d)" % {
+                                   "ps": image_size_meta,
+                                   "ss": image_size_store})
                         LOG.debug(msg)
                         raise HTTPConflict(explanation=msg,
                                            request=req,
                                            content_type="text/plain")
-                image_meta = self._activate(req, image_id, location)
+                location_data = {'url': location, 'metadata': {},
+                                 'status': 'active'}
+                image_meta = self._activate(req, image_id, location_data)
         return image_meta
 
     def _validate_image_for_activation(self, req, id, values):
@@ -872,8 +931,8 @@ class Controller(controller.BaseController):
             try:
                 self.update_store_acls(req, id, orig_or_updated_loc,
                                        public=is_public)
-            except exception.BadStoreUri:
-                msg = _("Invalid location %s") % location
+            except store.BadStoreUri:
+                msg = "Invalid location %s" % location
                 LOG.debug(msg)
                 raise HTTPBadRequest(explanation=msg,
                                      request=req,
@@ -920,28 +979,30 @@ class Controller(controller.BaseController):
                                                  image_data)
 
         except exception.Invalid as e:
-            msg = (_("Failed to update image metadata. Got error: %(e)s") %
-                   {'e': e})
+            msg = ("Failed to update image metadata. Got error: %s" %
+                   utils.exception_to_str(e))
             LOG.debug(msg)
             raise HTTPBadRequest(explanation=msg,
                                  request=req,
                                  content_type="text/plain")
         except exception.NotFound as e:
-            msg = _("Failed to find image to update: %(e)s") % {'e': e}
+            msg = (_("Failed to find image to update: %s") %
+                   utils.exception_to_str(e))
             for line in msg.split('\n'):
                 LOG.info(line)
             raise HTTPNotFound(explanation=msg,
                                request=req,
                                content_type="text/plain")
         except exception.Forbidden as e:
-            msg = _("Forbidden to update image: %(e)s") % {'e': e}
+            msg = (_("Forbidden to update image: %s") %
+                   utils.exception_to_str(e))
             for line in msg.split('\n'):
                 LOG.info(line)
             raise HTTPForbidden(explanation=msg,
                                 request=req,
                                 content_type="text/plain")
         except (exception.Conflict, exception.Duplicate) as e:
-            LOG.info(unicode(e))
+            LOG.info(utils.exception_to_str(e))
             raise HTTPConflict(body='Image operation conflicts',
                                request=req,
                                content_type='text/plain')
@@ -973,19 +1034,19 @@ class Controller(controller.BaseController):
 
         image = self.get_image_meta_or_404(req, id)
         if image['protected']:
-            msg = _("Image is protected")
+            msg = "Image is protected"
             LOG.debug(msg)
             raise HTTPForbidden(explanation=msg,
                                 request=req,
                                 content_type="text/plain")
 
         if image['status'] == 'pending_delete':
-            msg = (_("Forbidden to delete a %s image.") % image['status'])
+            msg = "Forbidden to delete a %s image." % image['status']
             LOG.debug(msg)
             raise HTTPForbidden(explanation=msg, request=req,
                                 content_type="text/plain")
         elif image['status'] == 'deleted':
-            msg = _("Image %s not found.") % id
+            msg = "Image %s not found." % id
             LOG.debug(msg)
             raise HTTPNotFound(explanation=msg, request=req,
                                content_type="text/plain")
@@ -1010,27 +1071,39 @@ class Controller(controller.BaseController):
                 # to delete the image if the backend doesn't yet store it.
                 # See https://bugs.launchpad.net/glance/+bug/747799
                 if image['location']:
-                    upload_utils.initiate_deletion(req, image['location'], id,
-                                                   CONF.delayed_delete)
-            except Exception as e:
-                registry.update_image_metadata(req.context, id,
-                                               {'status': ori_status})
-                raise e
+                    for loc_data in image['location_data']:
+                        if loc_data['status'] == 'active':
+                            upload_utils.initiate_deletion(req, loc_data, id)
+            except Exception:
+                with excutils.save_and_reraise_exception():
+                    registry.update_image_metadata(req.context, id,
+                                                   {'status': ori_status})
+
             registry.delete_image_metadata(req.context, id)
         except exception.NotFound as e:
-            msg = _("Failed to find image to delete: %(e)s") % {'e': e}
+            msg = (_("Failed to find image to delete: %s") %
+                   utils.exception_to_str(e))
             for line in msg.split('\n'):
                 LOG.info(line)
             raise HTTPNotFound(explanation=msg,
                                request=req,
                                content_type="text/plain")
         except exception.Forbidden as e:
-            msg = _("Forbidden to delete image: %(e)s") % {'e': e}
+            msg = (_("Forbidden to delete image: %s") %
+                   utils.exception_to_str(e))
             for line in msg.split('\n'):
                 LOG.info(line)
             raise HTTPForbidden(explanation=msg,
                                 request=req,
                                 content_type="text/plain")
+        except exception.InUseByStore as e:
+            msg = (_LI("Image %s could not be deleted because it is in use: "
+                       "%s") % (id, utils.exception_to_str(e)))
+            for line in msg.split('\n'):
+                LOG.info(line)
+            raise HTTPConflict(explanation=msg,
+                               request=req,
+                               content_type="text/plain")
         else:
             self.notifier.info('image.delete', redact_loc(image))
             return Response(body='', status=200)
@@ -1043,12 +1116,12 @@ class Controller(controller.BaseController):
         :param request: The WSGI/Webob Request object
         :param scheme: The backend store scheme
 
-        :raises HTTPNotFound if store does not exist
+        :raises HTTPBadRequest if store does not exist
         """
         try:
-            return get_store_from_scheme(request.context, scheme)
-        except exception.UnknownScheme:
-            msg = _("Store for scheme %s not found") % scheme
+            return store.get_store_from_scheme(scheme)
+        except store.UnknownScheme:
+            msg = "Store for scheme %s not found" % scheme
             LOG.debug(msg)
             raise HTTPBadRequest(explanation=msg,
                                  request=request,
@@ -1063,7 +1136,7 @@ class ImageDeserializer(wsgi.JSONRequestDeserializer):
         try:
             result['image_meta'] = utils.get_image_meta_from_headers(request)
         except exception.InvalidParameterValue as e:
-            msg = unicode(e)
+            msg = utils.exception_to_str(e)
             LOG.warn(msg, exc_info=True)
             raise HTTPBadRequest(explanation=e.msg, request=request)
 
@@ -1081,16 +1154,16 @@ class ImageDeserializer(wsgi.JSONRequestDeserializer):
         if image_size is None and data is not None:
             data = utils.LimitingReader(data, CONF.image_size_cap)
 
-            #NOTE(bcwaldon): this is a hack to make sure the downstream code
+            # NOTE(bcwaldon): this is a hack to make sure the downstream code
             # gets the correct image data
             request.body_file = data
 
         elif image_size > CONF.image_size_cap:
             max_image_size = CONF.image_size_cap
-            msg = _("Denying attempt to upload image larger than %d bytes.")
-            LOG.warn(msg % max_image_size)
-            raise HTTPBadRequest(explanation=msg % max_image_size,
-                                 request=request)
+            msg = (_("Denying attempt to upload image larger than %d"
+                     " bytes.") % max_image_size)
+            LOG.warn(msg)
+            raise HTTPBadRequest(explanation=msg, request=request)
 
         result['image_data'] = data
         return result
@@ -1140,7 +1213,6 @@ class ImageSerializer(wsgi.JSONResponseSerializer):
     def meta(self, response, result):
         image_meta = result['image_meta']
         self._inject_image_meta_headers(response, image_meta)
-        self._inject_location_header(response, image_meta)
         self._inject_checksum_header(response, image_meta)
         return response
 
@@ -1157,7 +1229,6 @@ class ImageSerializer(wsgi.JSONResponseSerializer):
         response.headers['Content-Type'] = 'application/octet-stream'
 
         self._inject_image_meta_headers(response, image_meta)
-        self._inject_location_header(response, image_meta)
         self._inject_checksum_header(response, image_meta)
 
         return response
@@ -1166,7 +1237,6 @@ class ImageSerializer(wsgi.JSONResponseSerializer):
         image_meta = result['image_meta']
         response.body = self.to_json(dict(image=image_meta))
         response.headers['Content-Type'] = 'application/json'
-        self._inject_location_header(response, image_meta)
         self._inject_checksum_header(response, image_meta)
         return response
 
