@@ -15,7 +15,9 @@
 
 import re
 
+import glance_store
 from oslo.config import cfg
+import six
 import six.moves.urllib.parse as urlparse
 import webob.exc
 
@@ -27,13 +29,15 @@ from glance.common import wsgi
 import glance.db
 import glance.gateway
 import glance.notifier
+from glance.openstack.common import gettextutils
 from glance.openstack.common import jsonutils as json
 import glance.openstack.common.log as logging
 from glance.openstack.common import timeutils
 import glance.schema
-import glance.store
 
 LOG = logging.getLogger(__name__)
+_LI = gettextutils._LI
+_LW = gettextutils._LW
 
 CONF = cfg.CONF
 CONF.import_opt('disk_formats', 'glance.common.config', group='image_format')
@@ -47,7 +51,7 @@ class ImagesController(object):
         self.db_api = db_api or glance.db.get_api()
         self.policy = policy_enforcer or policy.Enforcer()
         self.notifier = notifier or glance.notifier.Notifier()
-        self.store_api = store_api or glance.store
+        self.store_api = store_api or glance_store
         self.gateway = glance.gateway.Gateway(self.db_api, self.store_api,
                                               self.notifier, self.policy)
 
@@ -66,9 +70,11 @@ class ImagesController(object):
         except exception.InvalidParameterValue as e:
             raise webob.exc.HTTPBadRequest(explanation=e.msg)
         except exception.LimitExceeded as e:
-            LOG.info(unicode(e))
+            LOG.info(utils.exception_to_str(e))
             raise webob.exc.HTTPRequestEntityTooLarge(
                 explanation=e.msg, request=req, content_type='text/plain')
+        except exception.Duplicate as dupex:
+            raise webob.exc.HTTPConflict(explanation=dupex.msg)
 
         return image
 
@@ -121,7 +127,7 @@ class ImagesController(object):
                 change_method(req, image, change)
 
             if changes:
-                    image_repo.save(image)
+                image_repo.save(image)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
         except exception.Forbidden as e:
@@ -129,13 +135,13 @@ class ImagesController(object):
         except exception.InvalidParameterValue as e:
             raise webob.exc.HTTPBadRequest(explanation=e.msg)
         except exception.StorageQuotaFull as e:
-            msg = (_("Denying attempt to upload image because it exceeds the ."
-                     "quota: %s") % e)
+            msg = (_LI("Denying attempt to upload image because it exceeds the"
+                       " .quota: %s") % utils.exception_to_str(e))
             LOG.info(msg)
             raise webob.exc.HTTPRequestEntityTooLarge(
                 explanation=msg, request=req, content_type='text/plain')
         except exception.LimitExceeded as e:
-            LOG.info(unicode(e))
+            LOG.info(utils.exception_to_str(e))
             raise webob.exc.HTTPRequestEntityTooLarge(
                 explanation=e.msg, request=req, content_type='text/plain')
 
@@ -198,6 +204,11 @@ class ImagesController(object):
                    {'image_id': image_id})
             LOG.info(msg)
             raise webob.exc.HTTPNotFound(explanation=msg)
+        except exception.InUseByStore as e:
+            msg = (_LI("Image %s could not be deleted "
+                       "because it is in use: %s") % (image_id, e.msg))
+            LOG.info(msg)
+            raise webob.exc.HTTPConflict(explanation=msg)
 
     def _get_locations_op_pos(self, path_pos, max_pos, allow_max):
         if path_pos is None or max_pos is None:
@@ -230,7 +241,8 @@ class ImagesController(object):
             except (exception.BadStoreUri, exception.DuplicateLocation) as bse:
                 raise webob.exc.HTTPBadRequest(explanation=bse.msg)
             except ValueError as ve:    # update image status failed.
-                raise webob.exc.HTTPBadRequest(explanation=unicode(ve))
+                raise webob.exc.HTTPBadRequest(explanation=
+                                               utils.exception_to_str(ve))
 
     def _do_add_locations(self, image, path_pos, value):
         pos = self._get_locations_op_pos(path_pos,
@@ -245,7 +257,8 @@ class ImagesController(object):
         except (exception.BadStoreUri, exception.DuplicateLocation) as bse:
             raise webob.exc.HTTPBadRequest(explanation=bse.msg)
         except ValueError as ve:    # update image status failed.
-            raise webob.exc.HTTPBadRequest(explanation=unicode(ve))
+            raise webob.exc.HTTPBadRequest(explanation=
+                                           utils.exception_to_str(ve))
 
     def _do_remove_locations(self, image, path_pos):
         pos = self._get_locations_op_pos(path_pos,
@@ -258,7 +271,8 @@ class ImagesController(object):
             # from the backend store.
             image.locations.pop(pos)
         except Exception as e:
-            raise webob.exc.HTTPInternalServerError(explanation=unicode(e))
+            raise webob.exc.HTTPInternalServerError(explanation=
+                                                    utils.exception_to_str(e))
         if (len(image.locations) == 0) and (image.status == 'active'):
             image.status = 'queued'
 
@@ -293,7 +307,8 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
         for key in cls._disallowed_properties:
             if key in image:
                 msg = _("Attribute '%s' is read-only.") % key
-                raise webob.exc.HTTPForbidden(explanation=unicode(msg))
+                raise webob.exc.HTTPForbidden(explanation=
+                                              utils.exception_to_str(msg))
 
     def create(self, request):
         body = self._get_request_body(request)
@@ -307,7 +322,13 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
         tags = properties.pop('tags', None)
         for key in self._base_properties:
             try:
-                image[key] = properties.pop(key)
+                # NOTE(flwang): Instead of changing the _check_unexpected
+                # of ImageFactory. It would be better to do the mapping
+                # at here.
+                if key == 'id':
+                    image['image_id'] = properties.pop(key)
+                else:
+                    image[key] = properties.pop(key)
             except KeyError:
                 pass
         return dict(image=image, extra_properties=properties, tags=tags)
@@ -392,10 +413,10 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
         path_root = change['path'][0]
         if path_root in self._readonly_properties:
             msg = _("Attribute '%s' is read-only.") % path_root
-            raise webob.exc.HTTPForbidden(explanation=unicode(msg))
+            raise webob.exc.HTTPForbidden(explanation=six.text_type(msg))
         if path_root in self._reserved_properties:
             msg = _("Attribute '%s' is reserved.") % path_root
-            raise webob.exc.HTTPForbidden(explanation=unicode(msg))
+            raise webob.exc.HTTPForbidden(explanation=six.text_type(msg))
 
         if change['op'] == 'delete':
             return
@@ -403,8 +424,8 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
         partial_image = None
         if len(change['path']) == 1:
             partial_image = {path_root: change['value']}
-        elif ((path_root in _get_base_properties().keys()) and
-              (_get_base_properties()[path_root].get('type', '') == 'array')):
+        elif ((path_root in get_base_properties().keys()) and
+              (get_base_properties()[path_root].get('type', '') == 'array')):
             # NOTE(zhiyan): cient can use PATCH API to adding element to
             # the image's existing set property directly.
             # Such as: 1. using '/locations/N' path to adding a location
@@ -426,7 +447,7 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
         if len(path) != limits.get(op, 1):
             msg = _("Invalid JSON pointer for this resource: "
                     "'/%s'") % '/'.join(path)
-            raise webob.exc.HTTPBadRequest(explanation=unicode(msg))
+            raise webob.exc.HTTPBadRequest(explanation=six.text_type(msg))
 
     def _parse_json_schema_change(self, raw_change, draft_version):
         if draft_version == 10:
@@ -449,7 +470,8 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
             'application/openstack-images-v2.1-json-patch': 10,
         }
         if request.content_type not in content_types:
-            headers = {'Accept-Patch': ', '.join(content_types.keys())}
+            headers = {'Accept-Patch':
+                       ', '.join(sorted(content_types.keys()))}
             raise webob.exc.HTTPUnsupportedMediaType(headers=headers)
 
         json_schema_version = content_types[request.content_type]
@@ -513,6 +535,10 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
             if visibility not in ['public', 'private', 'shared']:
                 msg = _('Invalid visibility value: %s') % visibility
                 raise webob.exc.HTTPBadRequest(explanation=msg)
+        changes_since = filters.get('changes-since', None)
+        if changes_since:
+            msg = _('The "changes-since" filter is no longer available on v2.')
+            raise webob.exc.HTTPBadRequest(explanation=msg)
 
         return filters
 
@@ -575,19 +601,30 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
             image_view['updated_at'] = timeutils.isotime(image.updated_at)
 
             if CONF.show_multiple_locations:
-                if image.locations:
-                    image_view['locations'] = list(image.locations)
+                locations = list(image.locations)
+                if locations:
+                    image_view['locations'] = []
+                    for loc in locations:
+                        tmp = dict(loc)
+                        tmp.pop('id', None)
+                        tmp.pop('status', None)
+                        image_view['locations'].append(tmp)
                 else:
                     # NOTE (flwang): We will still show "locations": [] if
                     # image.locations is None to indicate it's allowed to show
                     # locations but it's just non-existent.
                     image_view['locations'] = []
+                    LOG.debug("There is not available location "
+                              "for image %s" % image.image_id)
 
-            if CONF.show_image_direct_url and image.locations:
-                # Choose best location configured strategy
-                best_location = (
-                    location_strategy.choose_best_location(image.locations))
-                image_view['direct_url'] = best_location['url']
+            if CONF.show_image_direct_url:
+                if image.locations:
+                    # Choose best location configured strategy
+                    l = location_strategy.choose_best_location(image.locations)
+                    image_view['direct_url'] = l['url']
+                else:
+                    LOG.debug("There is not available location "
+                              "for image %s" % image.image_id)
 
             image_view['tags'] = list(image.tags)
             image_view['self'] = self._get_image_href(image)
@@ -606,13 +643,13 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
     def show(self, response, image):
         image_view = self._format_image(image)
         body = json.dumps(image_view, ensure_ascii=False)
-        response.unicode_body = unicode(body)
+        response.unicode_body = six.text_type(body)
         response.content_type = 'application/json'
 
     def update(self, response, image):
         image_view = self._format_image(image)
         body = json.dumps(image_view, ensure_ascii=False)
-        response.unicode_body = unicode(body)
+        response.unicode_body = six.text_type(body)
         response.content_type = 'application/json'
 
     def index(self, response, result):
@@ -630,14 +667,15 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
             params['marker'] = result['next_marker']
             next_query = urlparse.urlencode(params)
             body['next'] = '/v2/images?%s' % next_query
-        response.unicode_body = unicode(json.dumps(body, ensure_ascii=False))
+        response.unicode_body = six.text_type(json.dumps(body,
+                                                         ensure_ascii=False))
         response.content_type = 'application/json'
 
     def delete(self, response, result):
         response.status_int = 204
 
 
-def _get_base_properties():
+def get_base_properties():
     return {
         'id': {
             'type': 'string',
@@ -771,13 +809,17 @@ def _get_base_links():
 
 
 def get_schema(custom_properties=None):
-    properties = _get_base_properties()
+    properties = get_base_properties()
     links = _get_base_links()
     if CONF.allow_additional_image_properties:
         schema = glance.schema.PermissiveSchema('image', properties, links)
     else:
         schema = glance.schema.Schema('image', properties)
-    schema.merge_properties(custom_properties or {})
+
+    if custom_properties:
+        for property_value in custom_properties.values():
+            property_value['is_base'] = False
+        schema.merge_properties(custom_properties)
     return schema
 
 
@@ -791,13 +833,13 @@ def load_custom_properties():
     filename = 'schema-image.json'
     match = CONF.find_file(filename)
     if match:
-        schema_file = open(match)
-        schema_data = schema_file.read()
+        with open(match, 'r') as schema_file:
+            schema_data = schema_file.read()
         return json.loads(schema_data)
     else:
-        msg = _('Could not find schema properties file %s. Continuing '
-                'without custom properties')
-        LOG.warn(msg % filename)
+        msg = (_LW('Could not find schema properties file %s. Continuing '
+                   'without custom properties') % filename)
+        LOG.warn(msg)
         return {}
 
 
