@@ -13,15 +13,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import http.client as http
-import urllib.parse as urlparse
-
-from oslo_config import cfg
-from oslo_log import log as logging
-from oslo_serialization import jsonutils
-from oslo_utils import encodeutils
+from oslo.config import cfg
+import six
+import six.moves.urllib.parse as urlparse
 import webob.exc
-from wsme.rest import json
+from wsme.rest.json import fromjson
+from wsme.rest.json import tojson
 
 from glance.api import policy
 from glance.api.v2.model.metadef_namespace import Namespace
@@ -29,53 +26,39 @@ from glance.api.v2.model.metadef_namespace import Namespaces
 from glance.api.v2.model.metadef_object import MetadefObject
 from glance.api.v2.model.metadef_property_type import PropertyType
 from glance.api.v2.model.metadef_resource_type import ResourceTypeAssociation
-from glance.api.v2.model.metadef_tag import MetadefTag
-from glance.api.v2 import policy as api_policy
 from glance.common import exception
 from glance.common import utils
 from glance.common import wsgi
 from glance.common import wsme_utils
 import glance.db
 import glance.gateway
-from glance.i18n import _, _LE
+from glance import i18n
 import glance.notifier
+from glance.openstack.common import jsonutils as json
+import glance.openstack.common.log as logging
 import glance.schema
 
 LOG = logging.getLogger(__name__)
+_LE = i18n._LE
+_LW = i18n._LW
+_LI = i18n._LI
 
 CONF = cfg.CONF
 
 
 class NamespaceController(object):
-    def __init__(self, db_api=None, policy_enforcer=None, notifier=None):
+    def __init__(self, db_api=None, policy_enforcer=None):
         self.db_api = db_api or glance.db.get_api()
         self.policy = policy_enforcer or policy.Enforcer()
-        self.notifier = notifier or glance.notifier.Notifier()
         self.gateway = glance.gateway.Gateway(db_api=self.db_api,
-                                              notifier=self.notifier,
                                               policy_enforcer=self.policy)
         self.ns_schema_link = '/v2/schemas/metadefs/namespace'
         self.obj_schema_link = '/v2/schemas/metadefs/object'
-        self.tag_schema_link = '/v2/schemas/metadefs/tag'
 
     def index(self, req, marker=None, limit=None, sort_key='created_at',
               sort_dir='desc', filters=None):
         try:
-            ns_repo = self.gateway.get_metadef_namespace_repo(
-                req.context, authorization_layer=False)
-
-            policy_check = api_policy.MetadefAPIPolicy(
-                req.context,
-                enforcer=self.policy)
-            # NOTE(abhishekk): This is just a "do you have permission to
-            # list namespace" check. Each namespace is checked against
-            # get_metadef_namespace below.
-            policy_check.get_metadef_namespaces()
-
-            # NOTE(abhishekk): We also need to fetch resource_types associated
-            # with namespaces, so better to check we have permission for the
-            # same in advance.
-            policy_check.list_metadef_resource_types()
+            ns_repo = self.gateway.get_metadef_namespace_repo(req.context)
 
             # Get namespace id
             if marker:
@@ -85,31 +68,15 @@ class NamespaceController(object):
             database_ns_list = ns_repo.list(
                 marker=marker, limit=limit, sort_key=sort_key,
                 sort_dir=sort_dir, filters=filters)
-
-            ns_list = [
-                ns for ns in database_ns_list if api_policy.MetadefAPIPolicy(
-                    req.context, md_resource=ns, enforcer=self.policy).check(
-                    'get_metadef_namespace')]
-
-            rs_repo = (
-                self.gateway.get_metadef_resource_type_repo(
-                    req.context, authorization_layer=False))
-            for db_namespace in ns_list:
+            for db_namespace in database_ns_list:
                 # Get resource type associations
                 filters = dict()
                 filters['namespace'] = db_namespace.namespace
-                try:
-                    repo_rs_type_list = rs_repo.list(filters=filters)
-                except exception.NotFound:
-                    # NOTE(danms): If we fail to list resource_types
-                    # for this namespace, do not fail the entire
-                    # namespace list operation with NotFound.
-                    repo_rs_type_list = []
-                resource_type_list = [
-                    ResourceTypeAssociation.to_wsme_model(
-                        resource_type
-                    ) for resource_type in repo_rs_type_list]
-
+                rs_repo = (
+                    self.gateway.get_metadef_resource_type_repo(req.context))
+                repo_rs_type_list = rs_repo.list(filters=filters)
+                resource_type_list = [ResourceTypeAssociation.to_wsme_model(
+                    resource_type) for resource_type in repo_rs_type_list]
                 if resource_type_list:
                     db_namespace.resource_type_associations = (
                         resource_type_list)
@@ -117,18 +84,19 @@ class NamespaceController(object):
             namespace_list = [Namespace.to_wsme_model(
                 db_namespace,
                 get_namespace_href(db_namespace),
-                self.ns_schema_link) for db_namespace in ns_list]
+                self.ns_schema_link) for db_namespace in database_ns_list]
             namespaces = Namespaces()
             namespaces.namespaces = namespace_list
             if len(namespace_list) != 0 and len(namespace_list) == limit:
-                namespaces.next = ns_list[-1].namespace
+                namespaces.next = namespace_list[-1].namespace
 
         except exception.Forbidden as e:
-            LOG.debug("User not permitted to retrieve metadata namespaces "
-                      "index")
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
+        except Exception as e:
+            LOG.error(utils.exception_to_str(e))
+            raise webob.exc.HTTPInternalServerError()
         return namespaces
 
     @utils.mutating
@@ -137,42 +105,17 @@ class NamespaceController(object):
             namespace_created = False
             # Create Namespace
             ns_factory = self.gateway.get_metadef_namespace_factory(
-                req.context, authorization_layer=False)
-            ns_repo = self.gateway.get_metadef_namespace_repo(
-                req.context, authorization_layer=False)
-
-            # NOTE(abhishekk): Here we are going to check if user is authorized
-            # to create namespace, resource_types, objects, properties etc.
-            policy_check = api_policy.MetadefAPIPolicy(
-                req.context, enforcer=self.policy)
-            policy_check.add_metadef_namespace()
-
-            if namespace.resource_type_associations:
-                policy_check.add_metadef_resource_type_association()
-            if namespace.objects:
-                policy_check.add_metadef_object()
-            if namespace.properties:
-                policy_check.add_metadef_property()
-            if namespace.tags:
-                policy_check.add_metadef_tag()
-
-            # NOTE(abhishekk): As we are getting rid of auth layer, this
-            # is the place where we should add owner if it is not specified
-            # in request.
-            kwargs = namespace.to_dict()
-            if 'owner' not in kwargs:
-                kwargs.update({'owner': req.context.owner})
-
-            new_namespace = ns_factory.new_namespace(**kwargs)
+                req.context)
+            ns_repo = self.gateway.get_metadef_namespace_repo(req.context)
+            new_namespace = ns_factory.new_namespace(**namespace.to_dict())
             ns_repo.add(new_namespace)
             namespace_created = True
 
             # Create Resource Types
+            rs_factory = (
+                self.gateway.get_metadef_resource_type_factory(req.context))
+            rs_repo = self.gateway.get_metadef_resource_type_repo(req.context)
             if namespace.resource_type_associations:
-                rs_factory = (self.gateway.get_metadef_resource_type_factory(
-                    req.context, authorization_layer=False))
-                rs_repo = self.gateway.get_metadef_resource_type_repo(
-                    req.context, authorization_layer=False)
                 for resource_type in namespace.resource_type_associations:
                     new_resource = rs_factory.new_resource_type(
                         namespace=namespace.namespace,
@@ -180,35 +123,22 @@ class NamespaceController(object):
                     rs_repo.add(new_resource)
 
             # Create Objects
+            object_factory = self.gateway.get_metadef_object_factory(
+                req.context)
+            object_repo = self.gateway.get_metadef_object_repo(req.context)
+
             if namespace.objects:
-                object_factory = self.gateway.get_metadef_object_factory(
-                    req.context, authorization_layer=False)
-                object_repo = self.gateway.get_metadef_object_repo(
-                    req.context, authorization_layer=False)
                 for metadata_object in namespace.objects:
                     new_meta_object = object_factory.new_object(
                         namespace=namespace.namespace,
                         **metadata_object.to_dict())
                     object_repo.add(new_meta_object)
 
-            # Create Tags
-            if namespace.tags:
-                tag_factory = self.gateway.get_metadef_tag_factory(
-                    req.context, authorization_layer=False)
-                tag_repo = self.gateway.get_metadef_tag_repo(
-                    req.context, authorization_layer=False)
-                for metadata_tag in namespace.tags:
-                    new_meta_tag = tag_factory.new_tag(
-                        namespace=namespace.namespace,
-                        **metadata_tag.to_dict())
-                    tag_repo.add(new_meta_tag)
-
             # Create Namespace Properties
+            prop_factory = (
+                self.gateway.get_metadef_property_factory(req.context))
+            prop_repo = self.gateway.get_metadef_property_repo(req.context)
             if namespace.properties:
-                prop_factory = (self.gateway.get_metadef_property_factory(
-                    req.context, authorization_layer=False))
-                prop_repo = self.gateway.get_metadef_property_repo(
-                    req.context, authorization_layer=False)
                 for (name, value) in namespace.properties.items():
                     new_property_type = (
                         prop_factory.new_namespace_property(
@@ -216,13 +146,9 @@ class NamespaceController(object):
                             **self._to_property_dict(name, value)
                         ))
                     prop_repo.add(new_property_type)
-        except exception.Invalid as e:
-            msg = (_("Couldn't create metadata namespace: %s")
-                   % encodeutils.exception_to_unicode(e))
-            raise webob.exc.HTTPBadRequest(explanation=msg)
+
         except exception.Forbidden as e:
             self._cleanup_namespace(ns_repo, namespace, namespace_created)
-            LOG.debug("User not permitted to create metadata namespace")
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.NotFound as e:
             self._cleanup_namespace(ns_repo, namespace, namespace_created)
@@ -230,13 +156,15 @@ class NamespaceController(object):
         except exception.Duplicate as e:
             self._cleanup_namespace(ns_repo, namespace, namespace_created)
             raise webob.exc.HTTPConflict(explanation=e.msg)
+        except Exception as e:
+            LOG.error(utils.exception_to_str(e))
+            raise webob.exc.HTTPInternalServerError()
 
         # Return the user namespace as we don't expose the id to user
         new_namespace.properties = namespace.properties
         new_namespace.objects = namespace.objects
         new_namespace.resource_type_associations = (
             namespace.resource_type_associations)
-        new_namespace.tags = namespace.tags
         return Namespace.to_wsme_model(new_namespace,
                                        get_namespace_href(new_namespace),
                                        self.ns_schema_link)
@@ -244,7 +172,7 @@ class NamespaceController(object):
     def _to_property_dict(self, name, value):
         # Convert the model PropertyTypes dict to a JSON string
         db_property_type_dict = dict()
-        db_property_type_dict['schema'] = json.tojson(PropertyType, value)
+        db_property_type_dict['schema'] = tojson(PropertyType, value)
         db_property_type_dict['name'] = name
         return db_property_type_dict
 
@@ -254,42 +182,19 @@ class NamespaceController(object):
                 namespace_obj = namespace_repo.get(namespace.namespace)
                 namespace_obj.delete()
                 namespace_repo.remove(namespace_obj)
-                LOG.debug("Cleaned up namespace %(namespace)s ",
-                          {'namespace': namespace.namespace})
-            except Exception as e:
-                msg = (_LE("Failed to delete namespace %(namespace)s."
-                           "Exception: %(exception)s"),
-                       {'namespace': namespace.namespace,
-                        'exception': encodeutils.exception_to_unicode(e)})
+                msg = ("Cleaned up namespace %(namespace)s "
+                       % {'namespace': namespace.namespace})
+                LOG.debug(msg)
+            except exception:
+                msg = (_LE("Failed to delete namespace %(namespace)s ") %
+                       {'namespace': namespace.namespace})
                 LOG.error(msg)
 
     def show(self, req, namespace, filters=None):
         try:
             # Get namespace
-            ns_repo = self.gateway.get_metadef_namespace_repo(
-                req.context, authorization_layer=False)
-            try:
-                namespace_obj = ns_repo.get(namespace)
-                policy_check = api_policy.MetadefAPIPolicy(
-                    req.context,
-                    md_resource=namespace_obj,
-                    enforcer=self.policy)
-                policy_check.get_metadef_namespace()
-            except (exception.Forbidden, webob.exc.HTTPForbidden):
-                LOG.debug("User not permitted to show namespace '%s'",
-                          namespace)
-                # NOTE (abhishekk): Returning 404 Not Found as the
-                # namespace is outside of this user's project
-                raise webob.exc.HTTPNotFound()
-
-            # NOTE(abhishekk): We also need to fetch resource_types, objects,
-            # properties, tags associated with namespace, so better to check
-            # whether user has permissions for the same.
-            policy_check.list_metadef_resource_types()
-            policy_check.get_metadef_objects()
-            policy_check.get_metadef_properties()
-            policy_check.get_metadef_tags()
-
+            ns_repo = self.gateway.get_metadef_namespace_repo(req.context)
+            namespace_obj = ns_repo.get(namespace)
             namespace_detail = Namespace.to_wsme_model(
                 namespace_obj,
                 get_namespace_href(namespace_obj),
@@ -298,8 +203,7 @@ class NamespaceController(object):
             ns_filters['namespace'] = namespace
 
             # Get objects
-            object_repo = self.gateway.get_metadef_object_repo(
-                req.context, authorization_layer=False)
+            object_repo = self.gateway.get_metadef_object_repo(req.context)
             db_metaobject_list = object_repo.list(filters=ns_filters)
             object_list = [MetadefObject.to_wsme_model(
                 db_metaobject,
@@ -309,8 +213,7 @@ class NamespaceController(object):
                 namespace_detail.objects = object_list
 
             # Get resource type associations
-            rs_repo = self.gateway.get_metadef_resource_type_repo(
-                req.context, authorization_layer=False)
+            rs_repo = self.gateway.get_metadef_resource_type_repo(req.context)
             db_resource_type_list = rs_repo.list(filters=ns_filters)
             resource_type_list = [ResourceTypeAssociation.to_wsme_model(
                 resource_type) for resource_type in db_resource_type_list]
@@ -319,8 +222,7 @@ class NamespaceController(object):
                     resource_type_list)
 
             # Get properties
-            prop_repo = self.gateway.get_metadef_property_repo(
-                req.context, authorization_layer=False)
+            prop_repo = self.gateway.get_metadef_property_repo(req.context)
             db_properties = prop_repo.list(filters=ns_filters)
             property_list = Namespace.to_model_properties(db_properties)
             if property_list:
@@ -330,43 +232,19 @@ class NamespaceController(object):
                 namespace_detail = self._prefix_property_name(
                     namespace_detail, filters['resource_type'])
 
-            # Get tags
-            tag_repo = self.gateway.get_metadef_tag_repo(
-                req.context, authorization_layer=False)
-            db_metatag_list = tag_repo.list(filters=ns_filters)
-            tag_list = [MetadefTag(**{'name': db_metatag.name})
-                        for db_metatag in db_metatag_list]
-            if tag_list:
-                namespace_detail.tags = tag_list
-
         except exception.Forbidden as e:
-            LOG.debug("User not permitted to show metadata namespace "
-                      "'%s'", namespace)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
+        except Exception as e:
+            LOG.error(utils.exception_to_str(e))
+            raise webob.exc.HTTPInternalServerError()
         return namespace_detail
 
     def update(self, req, user_ns, namespace):
-        namespace_repo = self.gateway.get_metadef_namespace_repo(
-            req.context, authorization_layer=False)
+        namespace_repo = self.gateway.get_metadef_namespace_repo(req.context)
         try:
             ns_obj = namespace_repo.get(namespace)
-        except (exception.Forbidden, exception.NotFound):
-            # NOTE (abhishekk): Returning 404 Not Found as the
-            # namespace is outside of this user's project
-            msg = _("Namespace %s not found") % namespace
-            raise webob.exc.HTTPNotFound(explanation=msg)
-
-        try:
-            # NOTE(abhishekk): Here we are just checking if use is authorized
-            # to modify the namespace or not
-            api_policy.MetadefAPIPolicy(
-                req.context,
-                md_resource=ns_obj,
-                enforcer=self.policy).modify_metadef_namespace()
-
-            ns_obj._old_namespace = ns_obj.namespace
             ns_obj.namespace = wsme_utils._get_value(user_ns.namespace)
             ns_obj.display_name = wsme_utils._get_value(user_ns.display_name)
             ns_obj.description = wsme_utils._get_value(user_ns.description)
@@ -379,139 +257,61 @@ class NamespaceController(object):
             ns_obj.owner = (
                 wsme_utils._get_value(user_ns.owner) or req.context.owner)
             updated_namespace = namespace_repo.save(ns_obj)
-        except exception.Invalid as e:
-            msg = (_("Couldn't update metadata namespace: %s")
-                   % encodeutils.exception_to_unicode(e))
-            raise webob.exc.HTTPBadRequest(explanation=msg)
         except exception.Forbidden as e:
-            LOG.debug("User not permitted to update metadata namespace "
-                      "'%s'", namespace)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
         except exception.Duplicate as e:
             raise webob.exc.HTTPConflict(explanation=e.msg)
+        except Exception as e:
+            LOG.error(utils.exception_to_str(e))
+            raise webob.exc.HTTPInternalServerError()
 
         return Namespace.to_wsme_model(updated_namespace,
                                        get_namespace_href(updated_namespace),
                                        self.ns_schema_link)
 
     def delete(self, req, namespace):
-        namespace_repo = self.gateway.get_metadef_namespace_repo(
-            req.context, authorization_layer=False)
+        namespace_repo = self.gateway.get_metadef_namespace_repo(req.context)
         try:
             namespace_obj = namespace_repo.get(namespace)
-        except (exception.Forbidden, exception.NotFound):
-            # NOTE (abhishekk): Returning 404 Not Found as the
-            # namespace is outside of this user's project
-            msg = _("Namespace %s not found") % namespace
-            raise webob.exc.HTTPNotFound(explanation=msg)
-
-        try:
-            # NOTE(abhishekk): Here we are just checking user is authorized to
-            # delete the namespace or not.
-            api_policy.MetadefAPIPolicy(
-                req.context,
-                md_resource=namespace_obj,
-                enforcer=self.policy).delete_metadef_namespace()
             namespace_obj.delete()
             namespace_repo.remove(namespace_obj)
         except exception.Forbidden as e:
-            LOG.debug("User not permitted to delete metadata namespace "
-                      "'%s'", namespace)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
+        except Exception as e:
+            LOG.error(utils.exception_to_str(e))
+            raise webob.exc.HTTPInternalServerError()
 
     def delete_objects(self, req, namespace):
-        ns_repo = self.gateway.get_metadef_namespace_repo(
-            req.context, authorization_layer=False)
+        ns_repo = self.gateway.get_metadef_namespace_repo(req.context)
         try:
             namespace_obj = ns_repo.get(namespace)
-        except (exception.Forbidden, exception.NotFound):
-            # NOTE (abhishekk): Returning 404 Not Found as the
-            # namespace is outside of this user's project
-            msg = _("Namespace %s not found") % namespace
-            raise webob.exc.HTTPNotFound(explanation=msg)
-
-        try:
-            # NOTE(abhishekk): This call currently checks whether user
-            # has permission to delete the namespace or not before deleting
-            # the objects associated with it.
-            api_policy.MetadefAPIPolicy(
-                req.context,
-                md_resource=namespace_obj,
-                enforcer=self.policy).delete_metadef_namespace()
-
             namespace_obj.delete()
             ns_repo.remove_objects(namespace_obj)
         except exception.Forbidden as e:
-            LOG.debug("User not permitted to delete metadata objects "
-                      "within '%s' namespace", namespace)
-            raise webob.exc.HTTPForbidden(explanation=e.msg)
-        except exception.NotFound as e:
-            raise webob.exc.HTTPNotFound(explanation=e.msg)
-
-    def delete_tags(self, req, namespace):
-        ns_repo = self.gateway.get_metadef_namespace_repo(
-            req.context, authorization_layer=False)
-        try:
-            namespace_obj = ns_repo.get(namespace)
-        except (exception.Forbidden, exception.NotFound):
-            # NOTE (abhishekk): Returning 404 Not Found as the
-            # namespace is outside of this user's project
-            msg = _("Namespace %s not found") % namespace
-            raise webob.exc.HTTPNotFound(explanation=msg)
-
-        try:
-            # NOTE(abhishekk): This call currently checks whether user
-            # has permission to delete the namespace or not before deleting
-            # the objects associated with it.
-            policy_check = api_policy.MetadefAPIPolicy(
-                req.context,
-                md_resource=namespace_obj,
-                enforcer=self.policy)
-            policy_check.delete_metadef_namespace()
-            # NOTE(abhishekk): This call checks whether user
-            # has permission to delete the tags or not.
-            policy_check.delete_metadef_tags()
-
-            namespace_obj.delete()
-            ns_repo.remove_tags(namespace_obj)
-        except exception.Forbidden as e:
-            LOG.debug("User not permitted to delete metadata tags "
-                      "within '%s' namespace", namespace)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
+        except Exception as e:
+            LOG.error(utils.exception_to_str(e))
+            raise webob.exc.HTTPInternalServerError()
 
     def delete_properties(self, req, namespace):
-        ns_repo = self.gateway.get_metadef_namespace_repo(
-            req.context, authorization_layer=False)
+        ns_repo = self.gateway.get_metadef_namespace_repo(req.context)
         try:
             namespace_obj = ns_repo.get(namespace)
-        except (exception.Forbidden, exception.NotFound):
-            # NOTE (abhishekk): Returning 404 Not Found as the
-            # namespace is outside of this user's project
-            msg = _("Namespace %s not found") % namespace
-            raise webob.exc.HTTPNotFound(explanation=msg)
-        try:
-            # NOTE(abhishekk): This call currently checks whether user
-            # has permission to delete the namespace or not before deleting
-            # the objects associated with it.
-            api_policy.MetadefAPIPolicy(
-                req.context,
-                md_resource=namespace_obj,
-                enforcer=self.policy).delete_metadef_namespace()
-
             namespace_obj.delete()
             ns_repo.remove_properties(namespace_obj)
         except exception.Forbidden as e:
-            LOG.debug("User not permitted to delete metadata properties "
-                      "within '%s' namespace", namespace)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
+        except Exception as e:
+            LOG.error(utils.exception_to_str(e))
+            raise webob.exc.HTTPInternalServerError()
 
     def _prefix_property_name(self, namespace_detail, user_resource_type):
         prefix = None
@@ -630,7 +430,7 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
             self.schema.validate(body)
         except exception.InvalidObject as e:
             raise webob.exc.HTTPBadRequest(explanation=e.msg)
-        namespace = json.fromjson(Namespace, body)
+        namespace = fromjson(Namespace, body)
         return dict(namespace=namespace)
 
     def update(self, request):
@@ -640,7 +440,7 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
             self.schema.validate(body)
         except exception.InvalidObject as e:
             raise webob.exc.HTTPBadRequest(explanation=e.msg)
-        namespace = json.fromjson(Namespace, body)
+        namespace = fromjson(Namespace, body)
         return dict(user_ns=namespace)
 
 
@@ -650,12 +450,12 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
         self.schema = schema
 
     def create(self, response, namespace):
-        ns_json = json.tojson(Namespace, namespace)
-        response = self.__render(ns_json, response, http.CREATED)
+        ns_json = tojson(Namespace, namespace)
+        response = self.__render(ns_json, response, 201)
         response.location = get_namespace_href(namespace)
 
     def show(self, response, namespace):
-        ns_json = json.tojson(Namespace, namespace)
+        ns_json = tojson(Namespace, namespace)
         response = self.__render(ns_json, response)
 
     def index(self, response, result):
@@ -671,28 +471,25 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
             next_query = urlparse.urlencode(params)
             result.next = '/v2/metadefs/namespaces?%s' % next_query
 
-        ns_json = json.tojson(Namespaces, result)
+        ns_json = tojson(Namespaces, result)
         response = self.__render(ns_json, response)
 
     def update(self, response, namespace):
-        ns_json = json.tojson(Namespace, namespace)
-        response = self.__render(ns_json, response, http.OK)
+        ns_json = tojson(Namespace, namespace)
+        response = self.__render(ns_json, response, 200)
 
     def delete(self, response, result):
-        response.status_int = http.NO_CONTENT
+        response.status_int = 204
 
     def delete_objects(self, response, result):
-        response.status_int = http.NO_CONTENT
+        response.status_int = 204
 
     def delete_properties(self, response, result):
-        response.status_int = http.NO_CONTENT
-
-    def delete_tags(self, response, result):
-        response.status_int = http.NO_CONTENT
+        response.status_int = 204
 
     def __render(self, json_data, response, response_status=None):
-        body = jsonutils.dumps(json_data, ensure_ascii=False)
-        response.unicode_body = body
+        body = json.dumps(json_data, ensure_ascii=False)
+        response.unicode_body = six.text_type(body)
         response.content_type = 'application/json'
         if response_status:
             response.status_int = response_status
@@ -728,8 +525,7 @@ def get_schema_definitions():
                 "required": ["title", "type"],
                 "properties": {
                     "name": {
-                        "type": "string",
-                        "maxLength": 80
+                        "type": "string"
                     },
                     "title": {
                         "type": "string"
@@ -737,12 +533,6 @@ def get_schema_definitions():
                     "description": {
                         "type": "string"
                     },
-                    "operators": {
-                        "type": "array",
-                        "items": {
-                            "type": "string"
-                        }
-                    },
                     "type": {
                         "type": "string",
                         "enum": [
@@ -855,23 +645,20 @@ def _get_base_properties():
         },
         "created_at": {
             "type": "string",
-            "readOnly": True,
-            "description": _("Date and time of namespace creation"),
+            "description": _("Date and time of namespace creation"
+                             " (READ-ONLY)"),
             "format": "date-time"
         },
         "updated_at": {
             "type": "string",
-            "readOnly": True,
-            "description": _("Date and time of the last namespace"
-                             " modification"),
+            "description": _("Date and time of the last namespace modification"
+                             " (READ-ONLY)"),
             "format": "date-time"
         },
         "schema": {
-            'readOnly': True,
             "type": "string"
         },
         "self": {
-            'readOnly': True,
             "type": "string"
         },
         "resource_type_associations": {
@@ -913,18 +700,7 @@ def _get_base_properties():
                     },
                 }
             }
-        },
-        "tags": {
-            "type": "array",
-            "items": {
-                "type": "object",
-                "properties": {
-                    "name": {
-                        "type": "string"
-                    }
-                }
-            }
-        },
+        }
     }
 
 
@@ -947,20 +723,14 @@ def get_collection_schema():
 
 
 def get_namespace_href(namespace):
-    base_href = '/v2/metadefs/namespaces/%s' % namespace.namespace
-    return base_href
+        base_href = '/v2/metadefs/namespaces/%s' % namespace.namespace
+        return base_href
 
 
 def get_object_href(namespace_name, metadef_object):
-    base_href = ('/v2/metadefs/namespaces/%s/objects/%s' %
-                 (namespace_name, metadef_object.name))
-    return base_href
-
-
-def get_tag_href(namespace_name, metadef_tag):
-    base_href = ('/v2/metadefs/namespaces/%s/tags/%s' %
-                 (namespace_name, metadef_tag.name))
-    return base_href
+        base_href = ('/v2/metadefs/namespaces/%s/objects/%s' %
+                     (namespace_name, metadef_object.name))
+        return base_href
 
 
 def create_resource():
