@@ -13,171 +13,102 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import http.client as http
-
-from oslo_log import log as logging
-from oslo_serialization import jsonutils
+import six
 import webob.exc
-from wsme.rest import json
+from wsme.rest.json import fromjson
+from wsme.rest.json import tojson
 
 from glance.api import policy
 from glance.api.v2.model.metadef_resource_type import ResourceType
 from glance.api.v2.model.metadef_resource_type import ResourceTypeAssociation
 from glance.api.v2.model.metadef_resource_type import ResourceTypeAssociations
 from glance.api.v2.model.metadef_resource_type import ResourceTypes
-from glance.api.v2 import policy as api_policy
 from glance.common import exception
+from glance.common import utils
 from glance.common import wsgi
 import glance.db
 import glance.gateway
-from glance.i18n import _
+from glance import i18n
 import glance.notifier
+from glance.openstack.common import jsonutils as json
+import glance.openstack.common.log as logging
 import glance.schema
 
 LOG = logging.getLogger(__name__)
+_LE = i18n._LE
+_LI = i18n._LI
 
 
 class ResourceTypeController(object):
-    def __init__(self, db_api=None, policy_enforcer=None, notifier=None):
+    def __init__(self, db_api=None, policy_enforcer=None):
         self.db_api = db_api or glance.db.get_api()
         self.policy = policy_enforcer or policy.Enforcer()
-        self.notifier = notifier or glance.notifier.Notifier()
         self.gateway = glance.gateway.Gateway(db_api=self.db_api,
-                                              notifier=self.notifier,
                                               policy_enforcer=self.policy)
 
     def index(self, req):
         try:
-            filters = {'namespace': None}
+            filters = {}
+            filters['namespace'] = None
             rs_type_repo = self.gateway.get_metadef_resource_type_repo(
                 req.context)
-            # NOTE(abhishekk): Here we are just checking if user is
-            # authorized to view/list metadef resource types or not.
-            # Also there is no relation between list_metadef_resource_types
-            # and get_metadef_resource_type policies so can not enforce
-            # get_metadef_resource_type policy on individual resource
-            # type here.
-            api_policy.MetadefAPIPolicy(
-                req.context,
-                enforcer=self.policy).list_metadef_resource_types()
-
             db_resource_type_list = rs_type_repo.list(filters=filters)
             resource_type_list = [ResourceType.to_wsme_model(
                 resource_type) for resource_type in db_resource_type_list]
             resource_types = ResourceTypes()
             resource_types.resource_types = resource_type_list
         except exception.Forbidden as e:
-            LOG.debug("User not permitted to retrieve metadata resource types "
-                      "index")
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
+        except Exception as e:
+            LOG.error(utils.exception_to_str(e))
+            raise webob.exc.HTTPInternalServerError(e)
         return resource_types
 
     def show(self, req, namespace):
-        ns_repo = self.gateway.get_metadef_namespace_repo(req.context)
         try:
-            namespace_obj = ns_repo.get(namespace)
-        except (exception.Forbidden, exception.NotFound):
-            # NOTE (abhishekk): Returning 404 Not Found as the
-            # namespace is outside of this user's project
-            msg = _("Namespace %s not found") % namespace
-            raise webob.exc.HTTPNotFound(explanation=msg)
-
-        try:
-            # NOTE(abhishekk): Here we are just checking if user is
-            # authorized to view/list metadef resource types or not.
-            # Each resource_type is checked against
-            # get_metadef_resource_type below.
-            api_policy.MetadefAPIPolicy(
-                req.context,
-                md_resource=namespace_obj,
-                enforcer=self.policy).list_metadef_resource_types()
-
-            filters = {'namespace': namespace}
+            filters = {}
+            filters['namespace'] = namespace
             rs_type_repo = self.gateway.get_metadef_resource_type_repo(
                 req.context)
-            db_type_list = rs_type_repo.list(filters=filters)
-
-            rs_type_list = [
-                ResourceTypeAssociation.to_wsme_model(
-                    rs_type
-                ) for rs_type in db_type_list if api_policy.MetadefAPIPolicy(
-                    req.context, md_resource=rs_type.namespace,
-                    enforcer=self.policy
-                ).check('get_metadef_resource_type')]
-
+            db_resource_type_list = rs_type_repo.list(filters=filters)
+            resource_type_list = [ResourceTypeAssociation.to_wsme_model(
+                resource_type) for resource_type in db_resource_type_list]
             resource_types = ResourceTypeAssociations()
-            resource_types.resource_type_associations = rs_type_list
+            resource_types.resource_type_associations = resource_type_list
         except exception.Forbidden as e:
-            LOG.debug("User not permitted to retrieve metadata resource types "
-                      "within '%s' namespace", namespace)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
+        except Exception as e:
+            LOG.error(utils.exception_to_str(e))
+            raise webob.exc.HTTPInternalServerError(e)
         return resource_types
 
     def create(self, req, resource_type, namespace):
         rs_type_factory = self.gateway.get_metadef_resource_type_factory(
             req.context)
-        rs_type_repo = self.gateway.get_metadef_resource_type_repo(
-            req.context)
-        ns_repo = self.gateway.get_metadef_namespace_repo(
-            req.context)
+        rs_type_repo = self.gateway.get_metadef_resource_type_repo(req.context)
         try:
-            namespace_obj = ns_repo.get(namespace)
-        except (exception.Forbidden, exception.NotFound):
-            # NOTE (abhishekk): Returning 404 Not Found as the
-            # namespace is outside of this user's project
-            msg = _("Namespace %s not found") % namespace
-            raise webob.exc.HTTPNotFound(explanation=msg)
-
-        try:
-            # NOTE(abhishekk): Metadef resource type is created for Metadef
-            # namespaces. Here we are just checking if user is authorized
-            # to create metadef resource types or not.
-            api_policy.MetadefAPIPolicy(
-                req.context,
-                md_resource=namespace_obj,
-                enforcer=self.policy).add_metadef_resource_type_association()
-
             new_resource_type = rs_type_factory.new_resource_type(
                 namespace=namespace, **resource_type.to_dict())
             rs_type_repo.add(new_resource_type)
 
         except exception.Forbidden as e:
-            LOG.debug("User not permitted to create metadata resource type "
-                      "within '%s' namespace", namespace)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
         except exception.Duplicate as e:
             raise webob.exc.HTTPConflict(explanation=e.msg)
+        except Exception as e:
+            LOG.error(utils.exception_to_str(e))
+            raise webob.exc.HTTPInternalServerError()
         return ResourceTypeAssociation.to_wsme_model(new_resource_type)
 
     def delete(self, req, namespace, resource_type):
-        rs_type_repo = self.gateway.get_metadef_resource_type_repo(
-            req.context)
-        ns_repo = self.gateway.get_metadef_namespace_repo(
-            req.context)
+        rs_type_repo = self.gateway.get_metadef_resource_type_repo(req.context)
         try:
-            namespace_obj = ns_repo.get(namespace)
-        except (exception.Forbidden, exception.NotFound):
-            # NOTE (abhishekk): Returning 404 Not Found as the
-            # namespace is outside of this user's project
-            msg = _("Namespace %s not found") % namespace
-            raise webob.exc.HTTPNotFound(explanation=msg)
-
-        try:
-            # NOTE(abhishekk): Metadef resource type is created for Metadef
-            # namespaces. Here we are just checking if user is authorized
-            # to delete metadef resource types or not.
-            api_policy.MetadefAPIPolicy(
-                req.context,
-                md_resource=namespace_obj,
-                enforcer=self.policy
-            ).remove_metadef_resource_type_association()
-
             filters = {}
             found = False
             filters['namespace'] = namespace
@@ -190,14 +121,15 @@ class ResourceTypeController(object):
             if not found:
                 raise exception.NotFound()
         except exception.Forbidden as e:
-            LOG.debug("User not permitted to delete metadata resource type "
-                      "'%s' within '%s' namespace", resource_type, namespace)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
-        except exception.NotFound:
-            msg = (_("Failed to find resource type %(resourcetype)s to "
-                     "delete") % {'resourcetype': resource_type})
+        except exception.NotFound as e:
+            msg = (_LE("Failed to find resource type %(resourcetype)s to "
+                       "delete") % {'resourcetype': resource_type})
             LOG.error(msg)
             raise webob.exc.HTTPNotFound(explanation=msg)
+        except Exception as e:
+            LOG.error(utils.exception_to_str(e))
+            raise webob.exc.HTTPInternalServerError()
 
 
 class RequestDeserializer(wsgi.JSONRequestDeserializer):
@@ -228,7 +160,7 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
             self.schema.validate(body)
         except exception.InvalidObject as e:
             raise webob.exc.HTTPBadRequest(explanation=e.msg)
-        resource_type = json.fromjson(ResourceTypeAssociation, body)
+        resource_type = fromjson(ResourceTypeAssociation, body)
         return dict(resource_type=resource_type)
 
 
@@ -238,26 +170,26 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
         self.schema = schema
 
     def show(self, response, result):
-        resource_type_json = json.tojson(ResourceTypeAssociations, result)
-        body = jsonutils.dumps(resource_type_json, ensure_ascii=False)
-        response.unicode_body = body
+        resource_type_json = tojson(ResourceTypeAssociations, result)
+        body = json.dumps(resource_type_json, ensure_ascii=False)
+        response.unicode_body = six.text_type(body)
         response.content_type = 'application/json'
 
     def index(self, response, result):
-        resource_type_json = json.tojson(ResourceTypes, result)
-        body = jsonutils.dumps(resource_type_json, ensure_ascii=False)
-        response.unicode_body = body
+        resource_type_json = tojson(ResourceTypes, result)
+        body = json.dumps(resource_type_json, ensure_ascii=False)
+        response.unicode_body = six.text_type(body)
         response.content_type = 'application/json'
 
     def create(self, response, result):
-        resource_type_json = json.tojson(ResourceTypeAssociation, result)
-        response.status_int = http.CREATED
-        body = jsonutils.dumps(resource_type_json, ensure_ascii=False)
-        response.unicode_body = body
+        resource_type_json = tojson(ResourceTypeAssociation, result)
+        response.status_int = 201
+        body = json.dumps(resource_type_json, ensure_ascii=False)
+        response.unicode_body = six.text_type(body)
         response.content_type = 'application/json'
 
     def delete(self, response, result):
-        response.status_int = http.NO_CONTENT
+        response.status_int = 204
 
 
 def _get_base_properties():
@@ -266,7 +198,7 @@ def _get_base_properties():
             'type': 'string',
             'description': _('Resource type names should be aligned with Heat '
                              'resource types whenever possible: '
-                             'https://docs.openstack.org/heat/latest/'
+                             'http://docs.openstack.org/developer/heat/'
                              'template_guide/openstack.html'),
             'maxLength': 80,
         },
@@ -292,15 +224,14 @@ def _get_base_properties():
         },
         "created_at": {
             "type": "string",
-            "readOnly": True,
-            "description": _("Date and time of resource type association"),
+            "description": _("Date and time of resource type association"
+                             " (READ-ONLY)"),
             "format": "date-time"
         },
         "updated_at": {
             "type": "string",
-            "readOnly": True,
             "description": _("Date and time of the last resource type "
-                             "association modification"),
+                             "association modification (READ-ONLY)"),
             "format": "date-time"
         }
     }
