@@ -36,16 +36,36 @@ possible_topdir = os.path.normpath(os.path.join(os.path.abspath(sys.argv[0]),
 if os.path.exists(os.path.join(possible_topdir, 'glance', '__init__.py')):
     sys.path.insert(0, possible_topdir)
 
+from oslo.config import cfg
+import osprofiler.notifier
+import osprofiler.web
+
 from glance.common import config
 from glance.common import wsgi
+from glance import notifier
 from glance.openstack.common import log
 
+CONF = cfg.CONF
+CONF.import_group("profiler", "glance.common.wsgi")
+
 
 def main():
     try:
         config.parse_args()
+        wsgi.set_eventlet_hub()
         log.setup('glance')
 
+        if cfg.CONF.profiler.enabled:
+            _notifier = osprofiler.notifier.create("Messaging",
+                                                   notifier.messaging, {},
+                                                   notifier.get_transport(),
+                                                   "glance", "registry",
+                                                   cfg.CONF.bind_host)
+            osprofiler.notifier.set(_notifier)
+
+        else:
+            osprofiler.web.disable()
+
         server = wsgi.Server()
         server.start(config.load_paste_app('glance-registry'),
                      default_port=9191)
