@@ -32,6 +32,8 @@ except ImportError:
     import socket
     import ssl
 
+import osprofiler.web
+
 try:
     import sendfile  # noqa
     SENDFILE_SUPPORTED = True
@@ -306,7 +308,7 @@ class BaseClient(object):
 
             <http|https>://<host>:port/doc_root
         """
-        LOG.debug(_("Configuring from URL: %s"), url)
+        LOG.debug("Configuring from URL: %s", url)
         parsed = urlparse.urlparse(url)
         self.use_ssl = parsed.scheme == 'https'
         self.host = parsed.hostname
@@ -318,7 +320,7 @@ class BaseClient(object):
             if self.DEFAULT_DOC_ROOT:
                 doc_root = self.DEFAULT_DOC_ROOT.lstrip('/')
                 self.doc_root += '/' + doc_root
-                msg = (_("Appending doc_root %(doc_root)s to URL %(url)s") %
+                msg = ("Appending doc_root %(doc_root)s to URL %(url)s" %
                        {'doc_root': doc_root, 'url': url})
                 LOG.debug(msg)
 
@@ -423,7 +425,8 @@ class BaseClient(object):
                   names and values
         """
         to_str = strutils.safe_encode
-        return dict([(to_str(h), to_str(v)) for h, v in headers.iteritems()])
+        return dict([(to_str(h), to_str(v)) for h, v in
+                     six.iteritems(headers)])
 
     @handle_redirects
     def _do_request(self, method, url, body, headers):
@@ -456,6 +459,7 @@ class BaseClient(object):
         try:
             connection_type = self.get_connection_type()
             headers = self._encode_headers(headers or {})
+            headers.update(osprofiler.web.get_trace_id_headers())
 
             if 'x-auth-token' not in headers and self.auth_tok:
                 headers['x-auth-token'] = self.auth_tok
@@ -502,7 +506,7 @@ class BaseClient(object):
                     if use_sendfile or header.lower() != 'content-length':
                         c.putheader(header, str(value))
 
-                iter = self.image_iterator(c, headers, body)
+                iter = utils.chunkreadable(body)
 
                 if use_sendfile:
                     # send actual file without copying into userspace
@@ -568,14 +572,6 @@ class BaseClient(object):
     def _iterable(self, body):
         return isinstance(body, collections.Iterable)
 
-    def image_iterator(self, connection, headers, body):
-        if self._sendable(body):
-            return SendFileIterator(connection, body)
-        elif self._iterable(body):
-            return utils.chunkreadable(body)
-        else:
-            return ImageBodyIterator(body)
-
     def get_status_code(self, response):
         """
         Returns the integer status code from the response, which
