@@ -1,7 +1,6 @@
 #!/usr/bin/env python
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright 2011 OpenStack LLC.
+# Copyright 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -24,7 +23,6 @@ import logging
 import logging.config
 import logging.handlers
 import os
-import sys
 
 from oslo.config import cfg
 from paste import deploy
@@ -41,22 +39,77 @@ paste_deploy_opts = [
     cfg.StrOpt('config_file',
                help=_('Name of the paste configuration file.')),
 ]
+image_format_opts = [
+    cfg.ListOpt('container_formats',
+                default=['ami', 'ari', 'aki', 'bare', 'ovf', 'ova'],
+                help=_("Supported values for the 'container_format' "
+                       "image attribute"),
+                deprecated_opts=[cfg.DeprecatedOpt('container_formats',
+                                                   group='DEFAULT')]),
+    cfg.ListOpt('disk_formats',
+                default=['ami', 'ari', 'aki', 'vhd', 'vmdk', 'raw', 'qcow2',
+                         'vdi', 'iso'],
+                help=_("Supported values for the 'disk_format' "
+                       "image attribute"),
+                deprecated_opts=[cfg.DeprecatedOpt('disk_formats',
+                                                   group='DEFAULT')]),
+]
+task_opts = [
+    cfg.IntOpt('task_time_to_live',
+               default=48,
+               help=_("Time in hours for which a task lives after, either "
+                      "succeeding or failing"),
+               deprecated_opts=[cfg.DeprecatedOpt('task_time_to_live',
+                                                  group='DEFAULT')]),
+    cfg.StrOpt('task_executor',
+               default='eventlet',
+               help=_("Specifies which task executor to be used to run the "
+                      "task scripts.")),
+    cfg.IntOpt('eventlet_executor_pool_size',
+               default=1000,
+               help=_("Specifies the maximum number of eventlet threads which "
+                      "can be spun up by the eventlet based task executor to "
+                      "perform execution of Glance tasks.")),
+]
+manage_opts = [
+    cfg.BoolOpt('db_enforce_mysql_charset',
+                default=True,
+                help=_('DEPRECATED. TO BE REMOVED IN THE JUNO RELEASE. '
+                       'Whether or not to enforce that all DB tables have '
+                       'charset utf8. If your database tables do not have '
+                       'charset utf8 you will need to convert before this '
+                       'option is removed. This option is only relevant if '
+                       'your database engine is MySQL.'))
+]
 common_opts = [
     cfg.BoolOpt('allow_additional_image_properties', default=True,
                 help=_('Whether to allow users to specify image properties '
-                'beyond what the image schema provides')),
+                       'beyond what the image schema provides')),
+    cfg.IntOpt('image_member_quota', default=128,
+               help=_('Maximum number of image members per image. '
+                      'Negative values evaluate to unlimited.')),
+    cfg.IntOpt('image_property_quota', default=128,
+               help=_('Maximum number of properties allowed on an image. '
+                      'Negative values evaluate to unlimited.')),
+    cfg.IntOpt('image_tag_quota', default=128,
+               help=_('Maximum number of tags allowed on an image. '
+                      'Negative values evaluate to unlimited.')),
+    cfg.IntOpt('image_location_quota', default=10,
+               help=_('Maximum number of locations allowed on an image. '
+                      'Negative values evaluate to unlimited.')),
     cfg.StrOpt('data_api', default='glance.db.sqlalchemy.api',
                help=_('Python module path of data access API')),
     cfg.IntOpt('limit_param_default', default=25,
                help=_('Default value for the number of items returned by a '
-               'request if not specified explicitly in the request')),
+                      'request if not specified explicitly in the request')),
     cfg.IntOpt('api_limit_max', default=1000,
                help=_('Maximum permissible number of items that could be '
-               'returned by a request')),
+                      'returned by a request')),
     cfg.BoolOpt('show_image_direct_url', default=False,
                 help=_('Whether to include the backend image storage location '
-                'in image properties. Revealing storage location can be a '
-                'security risk, so use this setting with caution!')),
+                       'in image properties. Revealing storage location can '
+                       'be a security risk, so use this setting with '
+                       'caution!')),
     cfg.BoolOpt('show_multiple_locations', default=False,
                 help=_('Whether to include the backend image locations '
                        'in image properties. Revealing storage location can '
@@ -65,15 +118,25 @@ common_opts = [
     cfg.IntOpt('image_size_cap', default=1099511627776,
                help=_("Maximum size of image a user can upload in bytes. "
                       "Defaults to 1099511627776 bytes (1 TB).")),
-    cfg.IntOpt('user_storage_quota', default=0,
-               help=_("Set a system wide quota for every user.  This value is "
-                      "the total number of bytes that a user can use across "
-                      "all storage systems.  A value of 0 means unlimited.")),
+    cfg.StrOpt('user_storage_quota', default='0',
+               help=_("Set a system wide quota for every user. This value is "
+                      "the total capacity that a user can use across "
+                      "all storage systems. A value of 0 means unlimited."
+                      "Optional unit can be specified for the value. Accepted "
+                      "units are B, KB, MB, GB and TB representing "
+                      "Bytes, KiloBytes, MegaBytes, GigaBytes and TeraBytes"
+                      "respectively. If no unit is specified then Bytes is "
+                      "assumed. Note that there should not be any space "
+                      "between value and unit and units are case sensitive.")),
     cfg.BoolOpt('enable_v1_api', default=True,
-                help=_("Deploy the v1 OpenStack Images API. ")),
+                help=_("Deploy the v1 OpenStack Images API.")),
     cfg.BoolOpt('enable_v2_api', default=True,
-                help=_("Deploy the v2 OpenStack Images API. ")),
-    cfg.StrOpt('pydev_worker_debug_host', default=None,
+                help=_("Deploy the v2 OpenStack Images API.")),
+    cfg.BoolOpt('enable_v1_registry', default=True,
+                help=_("Deploy the v1 OpenStack Registry API.")),
+    cfg.BoolOpt('enable_v2_registry', default=True,
+                help=_("Deploy the v2 OpenStack Registry API.")),
+    cfg.StrOpt('pydev_worker_debug_host',
                help=_('The hostname/IP of the pydev process listening for '
                       'debug connections')),
     cfg.IntOpt('pydev_worker_debug_port', default=5678,
@@ -86,18 +149,11 @@ common_opts = [
 
 CONF = cfg.CONF
 CONF.register_opts(paste_deploy_opts, group='paste_deploy')
+CONF.register_opts(image_format_opts, group='image_format')
+CONF.register_opts(task_opts, group='task')
+CONF.register_opts(manage_opts)
 CONF.register_opts(common_opts)
 
-CONF.import_opt('verbose', 'glance.openstack.common.log')
-CONF.import_opt('debug', 'glance.openstack.common.log')
-CONF.import_opt('log_dir', 'glance.openstack.common.log')
-CONF.import_opt('log_file', 'glance.openstack.common.log')
-CONF.import_opt('log_config', 'glance.openstack.common.log')
-CONF.import_opt('log_format', 'glance.openstack.common.log')
-CONF.import_opt('log_date_format', 'glance.openstack.common.log')
-CONF.import_opt('use_syslog', 'glance.openstack.common.log')
-CONF.import_opt('syslog_log_facility', 'glance.openstack.common.log')
-
 
 def parse_args(args=None, usage=None, default_config_files=None):
     CONF(args=args,
@@ -112,51 +168,6 @@ def parse_cache_args(args=None):
     parse_args(args=args, default_config_files=config_files)
 
 
-def setup_logging():
-    """
-    Sets up the logging options for a log with supplied name
-    """
-
-    if CONF.log_config:
-        # Use a logging configuration file for all settings...
-        if os.path.exists(CONF.log_config):
-            logging.config.fileConfig(CONF.log_config)
-            return
-        else:
-            raise RuntimeError("Unable to locate specified logging "
-                               "config file: %s" % CONF.log_config)
-
-    root_logger = logging.root
-    if CONF.debug:
-        root_logger.setLevel(logging.DEBUG)
-    elif CONF.verbose:
-        root_logger.setLevel(logging.INFO)
-    else:
-        root_logger.setLevel(logging.WARNING)
-
-    formatter = logging.Formatter(CONF.log_format, CONF.log_date_format)
-
-    if CONF.use_syslog:
-        try:
-            facility = getattr(logging.handlers.SysLogHandler,
-                               CONF.syslog_log_facility)
-        except AttributeError:
-            raise ValueError(_("Invalid syslog facility"))
-
-        handler = logging.handlers.SysLogHandler(address='/dev/log',
-                                                 facility=facility)
-    elif CONF.log_file:
-        logfile = CONF.log_file
-        if CONF.log_dir:
-            logfile = os.path.join(CONF.log_dir, logfile)
-        handler = logging.handlers.WatchedFileHandler(logfile)
-    else:
-        handler = logging.StreamHandler(sys.stdout)
-
-    handler.setFormatter(formatter)
-    root_logger.addHandler(handler)
-
-
 def _get_deployment_flavor(flavor=None):
     """
     Retrieve the paste_deploy.flavor config item, formatted appropriately
@@ -191,7 +202,7 @@ def _get_deployment_config_file():
     if not path:
         path = _get_paste_config_path()
     if not path:
-        msg = "Unable to locate paste config file for %s." % CONF.prog
+        msg = _("Unable to locate paste config file for %s.") % CONF.prog
         raise RuntimeError(msg)
     return os.path.abspath(path)
 
@@ -219,7 +230,7 @@ def load_paste_app(app_name, flavor=None, conf_file=None):
 
     try:
         logger = logging.getLogger(__name__)
-        logger.debug(_("Loading %(app_name)s from %(conf_file)s"),
+        logger.debug("Loading %(app_name)s from %(conf_file)s",
                      {'conf_file': conf_file, 'app_name': app_name})
 
         app = deploy.loadapp("config:%s" % conf_file, name=app_name)
@@ -230,8 +241,10 @@ def load_paste_app(app_name, flavor=None, conf_file=None):
 
         return app
     except (LookupError, ImportError) as e:
-        msg = _("Unable to load %(app_name)s from "
-                "configuration file %(conf_file)s."
-                "\nGot: %(e)r") % locals()
+        msg = (_("Unable to load %(app_name)s from "
+                 "configuration file %(conf_file)s."
+                 "\nGot: %(e)r") % {'app_name': app_name,
+                                    'conf_file': conf_file,
+                                    'e': e})
         logger.error(msg)
         raise RuntimeError(msg)
