@@ -17,23 +17,27 @@ __all__ = [
     'run',
 ]
 
-from oslo_concurrency import lockutils
-from oslo_log import log as logging
-from oslo_utils import encodeutils
-from oslo_utils import excutils
+import six
 
 from glance.api.v2 import images as v2_api
 from glance.common import exception
 from glance.common.scripts import utils as script_utils
-from glance.common import store_utils
-from glance.i18n import _, _LE, _LI, _LW
+from glance.common import utils as common_utils
+from glance import i18n
+from glance.openstack.common import excutils
+from glance.openstack.common import lockutils
+import glance.openstack.common.log as logging
 
+
+_LE = i18n._LE
+_LI = i18n._LI
+_LW = i18n._LW
 LOG = logging.getLogger(__name__)
 
 
 def run(t_id, context, task_repo, image_repo, image_factory):
     LOG.info(_LI('Task %(task_id)s beginning import '
-                 'execution.'), {'task_id': t_id})
+                 'execution.') % {'task_id': t_id})
     _execute(t_id, task_repo, image_repo, image_factory)
 
 
@@ -62,14 +66,14 @@ def _execute(t_id, task_repo, image_repo, image_factory):
         # Note: The message string contains Error in it to indicate
         # in the task.message that it's a error message for the user.
 
-        # TODO(nikhil): need to bring back save_and_reraise_exception when
+        #TODO(nikhil): need to bring back save_and_reraise_exception when
         # necessary
-        err_msg = ("Error: " + str(type(e)) + ': ' +
-                   encodeutils.exception_to_unicode(e))
-        log_msg = _LE(err_msg + ("Task ID %s" % task.task_id))  # noqa
+        err_msg = ("Error: " + six.text_type(type(e)) + ': ' +
+                   common_utils.exception_to_str(e))
+        log_msg = _LE(err_msg + ("Task ID %s" % task.task_id))
         LOG.exception(log_msg)
 
-        task.fail(_LE(err_msg))  # noqa
+        task.fail(_LE(err_msg))
     finally:
         task_repo.save(task)
 
@@ -80,51 +84,46 @@ def import_image(image_repo, image_factory, task_input, task_id, uri):
     # NOTE: set image status to saving just before setting data
     original_image.status = 'saving'
     image_repo.save(original_image)
+    set_image_data(original_image, uri, None)
+
+    # NOTE: Check if the Image is not deleted after setting the data
+    # before setting it's status to active. We need to set the status
+    # explicitly here using the Image object returned from image_repo .The
+    # Image object returned from create_image method does not have appropriate
+    # factories wrapped around it.
     image_id = original_image.image_id
-
-    # NOTE: Retrieving image from the database because the Image object
-    # returned from create_image method does not have appropriate factories
-    # wrapped around it.
     new_image = image_repo.get(image_id)
-    set_image_data(new_image, uri, task_id)
+    if new_image.status in ['saving']:
+        new_image.status = 'active'
+        new_image.size = original_image.size
+        new_image.virtual_size = original_image.virtual_size
+        new_image.checksum = original_image.checksum
+    else:
+        msg = _LE("The Image %(image_id)s object being created by this task "
+                  "%(task_id)s, is no longer in valid status for further "
+                  "processing." % {"image_id": new_image.image_id,
+                                   "task_id": task_id})
+        raise exception.Conflict(msg)
+    image_repo.save(new_image)
 
-    try:
-        # NOTE: Check if the Image is not deleted after setting the data
-        # before saving the active image. Here if image status is
-        # saving, then new_image is saved as it contains updated location,
-        # size, virtual_size and checksum information and the status of
-        # new_image is already set to active in set_image_data() call.
-        image = image_repo.get(image_id)
-        if image.status == 'saving':
-            image_repo.save(new_image)
-            return image_id
-        else:
-            msg = _("The Image %(image_id)s object being created by this task "
-                    "%(task_id)s, is no longer in valid status for further "
-                    "processing.") % {"image_id": image_id,
-                                      "task_id": task_id}
-            raise exception.Conflict(msg)
-    except (exception.Conflict, exception.NotFound,
-            exception.NotAuthenticated):
-        with excutils.save_and_reraise_exception():
-            if new_image.locations:
-                for location in new_image.locations:
-                    store_utils.delete_image_location_from_backend(
-                        new_image.context,
-                        image_id,
-                        location)
+    return image_id
 
 
 def create_image(image_repo, image_factory, image_properties, task_id):
+    _base_properties = []
+    for k, v in v2_api.get_base_properties().items():
+        _base_properties.append(k)
+
     properties = {}
     # NOTE: get the base properties
-    for key in v2_api.get_base_properties():
+    for key in _base_properties:
         try:
             properties[key] = image_properties.pop(key)
         except KeyError:
-            LOG.debug("Task ID %(task_id)s: Ignoring property %(k)s for "
-                      "setting base properties while creating "
-                      "Image.", {'task_id': task_id, 'k': key})
+            msg = _("Task ID %(task_id)s: Ignoring property %(k)s for setting "
+                    "base properties while creating "
+                    "Image.") % {'task_id': task_id, 'k': key}
+            LOG.debug(msg)
 
     # NOTE: get the rest of the properties and pass them as
     # extra_properties for Image to be created with them.
@@ -136,28 +135,21 @@ def create_image(image_repo, image_factory, image_properties, task_id):
     return image
 
 
-def set_image_data(image, uri, task_id, backend=None, set_active=True,
-                   callback=None):
+def set_image_data(image, uri, task_id):
     data_iter = None
     try:
         LOG.info(_LI("Task %(task_id)s: Got image data uri %(data_uri)s to be "
-                 "imported"), {"data_uri": uri, "task_id": task_id})
+                 "imported") % {"data_uri": uri, "task_id": task_id})
         data_iter = script_utils.get_image_data_iter(uri)
-        if callback:
-            # If a callback was provided, wrap our data iterator to call
-            # the function every 60 seconds.
-            data_iter = script_utils.CallbackIterator(
-                data_iter, callback, min_interval=60)
-        image.set_data(data_iter, backend=backend, set_active=set_active)
+        image.set_data(data_iter)
     except Exception as e:
         with excutils.save_and_reraise_exception():
-            LOG.warning(_LW("Task %(task_id)s failed with exception "
-                            "%(error)s"),
-                        {"error": encodeutils.exception_to_unicode(e),
-                         "task_id": task_id})
+            LOG.warn(_LW("Task %(task_id)s failed with exception %(error)s") %
+                     {"error": common_utils.exception_to_str(e),
+                      "task_id": task_id})
             LOG.info(_LI("Task %(task_id)s: Could not import image file"
-                         " %(image_data)s"), {"image_data": uri,
-                                              "task_id": task_id})
+                         " %(image_data)s") % {"image_data": uri,
+                                               "task_id": task_id})
     finally:
-        if hasattr(data_iter, 'close'):
+        if isinstance(data_iter, file):
             data_iter.close()
