@@ -20,10 +20,14 @@
 
 """Defines interface for DB access."""
 
-import functools
-import time
+import threading
 
 from oslo.config import cfg
+from oslo.db import exception as db_exception
+from oslo.db.sqlalchemy import session
+import osprofiler.sqlalchemy
+from retrying import retry
+import six
 from six.moves import xrange
 import sqlalchemy
 import sqlalchemy.orm as sa_orm
@@ -31,15 +35,23 @@ import sqlalchemy.sql as sa_sql
 
 from glance.common import exception
 from glance.db.sqlalchemy import models
-from glance.openstack.common.db import exception as db_exception
-from glance.openstack.common.db.sqlalchemy import session
+from glance import i18n
 import glance.openstack.common.log as os_logging
 from glance.openstack.common import timeutils
 
+from glance.db.sqlalchemy.metadef_api import namespace as metadef_namespace_api
+from glance.db.sqlalchemy.metadef_api import object as metadef_object_api
+from glance.db.sqlalchemy.metadef_api import property as metadef_property_api
+from glance.db.sqlalchemy.metadef_api\
+    import resource_type as metadef_resource_type_api
+from glance.db.sqlalchemy.metadef_api\
+    import resource_type_association as metadef_association_api
 
 BASE = models.BASE
 sa_logger = None
 LOG = os_logging.getLogger(__name__)
+_LI = i18n._LI
+_LW = i18n._LW
 
 
 STATUSES = ['active', 'saving', 'queued', 'killed', 'pending_delete',
@@ -47,37 +59,32 @@ STATUSES = ['active', 'saving', 'queued', 'killed', 'pending_delete',
 
 CONF = cfg.CONF
 CONF.import_opt('debug', 'glance.openstack.common.log')
-CONF.import_opt('connection', 'glance.openstack.common.db.options',
-                group='database')
-
+CONF.import_group("profiler", "glance.common.wsgi")
 
 _FACADE = None
+_LOCK = threading.Lock()
 
 
-def _retry_on_deadlock(f):
+def _retry_on_deadlock(exc):
     """Decorator to retry a DB API call if Deadlock was received."""
-    @functools.wraps(f)
-    def wrapped(*args, **kwargs):
-        while True:
-            try:
-                return f(*args, **kwargs)
-            except db_exception.DBDeadlock:
-                LOG.warn(_("Deadlock detected when running "
-                           "'%(func_name)s': Retrying..."),
-                         dict(func_name=f.__name__))
-                # Retry!
-                time.sleep(0.5)
-                continue
-    functools.update_wrapper(wrapped, f)
-    return wrapped
+
+    if isinstance(exc, db_exception.DBDeadlock):
+        LOG.warn(_LW("Deadlock detected. Retrying..."))
+        return True
+    return False
 
 
 def _create_facade_lazily():
-    global _FACADE
+    global _LOCK, _FACADE
     if _FACADE is None:
-        _FACADE = session.EngineFacade(
-            CONF.database.connection,
-            **dict(CONF.database.iteritems()))
+        with _LOCK:
+            if _FACADE is None:
+                _FACADE = session.EngineFacade.from_config(CONF)
+
+                if CONF.profiler.enabled and CONF.profiler.trace_sqlalchemy:
+                    osprofiler.sqlalchemy.add_tracing(sqlalchemy,
+                                                      _FACADE.get_engine(),
+                                                      "db")
     return _FACADE
 
 
@@ -102,7 +109,7 @@ def clear_db_env():
 
 def _check_mutate_authorization(context, image_ref):
     if not is_image_mutable(context, image_ref):
-        LOG.info(_("Attempted to modify image user did not own."))
+        LOG.info(_LI("Attempted to modify image user did not own."))
         msg = _("You do not own this image")
         if image_ref.is_public:
             exc_class = exception.ForbiddenPublicImage
@@ -128,7 +135,6 @@ def image_update(context, image_id, values, purge_props=False,
                          from_state=from_state)
 
 
-@_retry_on_deadlock
 def image_destroy(context, image_id):
     """Destroy the image or raise if it does not exist."""
     session = get_session()
@@ -141,8 +147,7 @@ def image_destroy(context, image_id):
         image_ref.delete(session=session)
         delete_time = image_ref.deleted_at
 
-        _image_locations_delete_all(context, image_ref.id, delete_time,
-                                    session)
+        _image_locations_delete_all(context, image_id, delete_time, session)
 
         _image_property_delete_all(context, image_id, delete_time, session)
 
@@ -153,18 +158,37 @@ def image_destroy(context, image_id):
     return _normalize_locations(image_ref)
 
 
-def _normalize_locations(image):
-    undeleted_locations = filter(lambda x: not x.deleted, image['locations'])
-    image['locations'] = [{'url': loc['value'],
-                           'metadata': loc['meta_data']}
-                          for loc in undeleted_locations]
+def _normalize_locations(image, force_show_deleted=False):
+    """
+    Generate suitable dictionary list for locations field of image.
+
+    We don't need to set other data fields of location record which return
+    from image query.
+    """
+
+    if force_show_deleted:
+        locations = image['locations']
+    else:
+        locations = filter(lambda x: not x.deleted, image['locations'])
+    image['locations'] = [{'id': loc['id'],
+                           'url': loc['value'],
+                           'metadata': loc['meta_data'],
+                           'status': loc['status']}
+                          for loc in locations]
+    return image
+
+
+def _normalize_tags(image):
+    undeleted_tags = filter(lambda x: not x.deleted, image['tags'])
+    image['tags'] = [tag['value'] for tag in undeleted_tags]
     return image
 
 
 def image_get(context, image_id, session=None, force_show_deleted=False):
     image = _image_get(context, image_id, session=session,
                        force_show_deleted=force_show_deleted)
-    image = _normalize_locations(image.to_dict())
+    image = _normalize_locations(image.to_dict(),
+                                 force_show_deleted=force_show_deleted)
     return image
 
 
@@ -200,13 +224,13 @@ def _image_get(context, image_id, session=None, force_show_deleted=False):
         image = query.one()
 
     except sa_orm.exc.NoResultFound:
-        msg = (_("No image found with ID %s") % image_id)
+        msg = "No image found with ID %s" % image_id
         LOG.debug(msg)
         raise exception.NotFound(msg)
 
     # Make sure they can look at it
     if not is_image_visible(context, image):
-        msg = (_("Forbidding request, image %s not visible") % image_id)
+        msg = "Forbidding request, image %s not visible" % image_id
         LOG.debug(msg)
         raise exception.Forbidden(msg)
 
@@ -293,7 +317,7 @@ def _paginate_query(query, model, limit, sort_keys, marker=None,
     if 'id' not in sort_keys:
         # TODO(justinsb): If this ever gives a false-positive, check
         # the actual primary key, rather than assuming its id
-        LOG.warn(_('Id not in sort_keys; is sort_keys unique?'))
+        LOG.warn(_LW('Id not in sort_keys; is sort_keys unique?'))
 
     assert(not (sort_dir and sort_dirs))
 
@@ -500,7 +524,7 @@ def _select_images_query(context, image_conditions, admin_as_user,
 def image_get_all(context, filters=None, marker=None, limit=None,
                   sort_key='created_at', sort_dir='desc',
                   member_status='accepted', is_public=None,
-                  admin_as_user=False):
+                  admin_as_user=False, return_tag=False):
     """
     Get all images that match zero or more filters.
 
@@ -518,6 +542,9 @@ def image_get_all(context, filters=None, marker=None, limit=None,
     :param admin_as_user: For backwards compatibility. If true, then return to
                       an admin the equivalent set of images which it would see
                       if it were a regular user
+    :param return_tag: To indicates whether image entry in result includes it
+                       relevant tag entries. This could improve upper-layer
+                       query performance, to prevent using separated calls
     """
     filters = filters or {}
 
@@ -566,8 +593,18 @@ def image_get_all(context, filters=None, marker=None, limit=None,
 
     query = query.options(sa_orm.joinedload(models.Image.properties))\
                  .options(sa_orm.joinedload(models.Image.locations))
+    if return_tag:
+        query = query.options(sa_orm.joinedload(models.Image.tags))
 
-    return [_normalize_locations(image.to_dict()) for image in query.all()]
+    images = []
+    for image in query.all():
+        image_dict = image.to_dict()
+        image_dict = _normalize_locations(image_dict,
+                                          force_show_deleted=showing_deleted)
+        if return_tag:
+            image_dict = _normalize_tags(image_dict)
+        images.append(image_dict)
+    return images
 
 
 def _drop_protected_attrs(model_class, values):
@@ -586,14 +623,12 @@ def _image_get_disk_usage_by_owner(owner, session, image_id=None):
     if image_id is not None:
         query = query.filter(models.Image.id != image_id)
     query = query.filter(models.Image.size > 0)
-    query = query.filter(~models.Image.status.in_(['killed',
-                                                   'pending_delete',
-                                                   'deleted']))
+    query = query.filter(~models.Image.status.in_(['killed', 'deleted']))
     images = query.all()
 
     total = 0
     for i in images:
-        locations = [l for l in i.locations if not l['deleted']]
+        locations = [l for l in i.locations if l['status'] != 'deleted']
         total += (i.size * len(locations))
     return total
 
@@ -624,6 +659,8 @@ def _update_values(image_ref, values):
             setattr(image_ref, k, values[k])
 
 
+@retry(retry_on_exception=_retry_on_deadlock, wait_fixed=500,
+       stop_max_attempt_number=50)
 def _image_update(context, values, image_id, purge_props=False,
                   from_state=None):
     """
@@ -727,35 +764,118 @@ def _image_update(context, values, image_id, purge_props=False,
         _set_properties_for_image(context, image_ref, properties, purge_props,
                                   session)
 
-    if location_data is not None:
-        _image_locations_set(image_ref.id, location_data, session)
+        if location_data is not None:
+            _image_locations_set(context, image_ref.id, location_data,
+                                 session=session)
 
     return image_get(context, image_ref.id)
 
 
-def _image_locations_set(image_id, locations, session):
-    location_refs = session.query(models.ImageLocation)\
-                           .filter_by(image_id=image_id)\
-                           .filter_by(deleted=False)\
-                           .all()
-    for location_ref in location_refs:
-        location_ref.delete(session=session)
-
-    for location in locations:
-        location_ref = models.ImageLocation(image_id=image_id,
-                                            value=location['url'],
-                                            meta_data=location['metadata'])
-        location_ref.save()
+def image_location_add(context, image_id, location, session=None):
+    deleted = location['status'] in ('deleted', 'pending_delete')
+    delete_time = timeutils.utcnow() if deleted else None
+    location_ref = models.ImageLocation(image_id=image_id,
+                                        value=location['url'],
+                                        meta_data=location['metadata'],
+                                        status=location['status'],
+                                        deleted=deleted,
+                                        deleted_at=delete_time)
+    session = session or get_session()
+    location_ref.save(session=session)
 
 
-def _image_locations_delete_all(context, image_id, delete_time=None,
-                                session=None):
+def image_location_update(context, image_id, location, session=None):
+    loc_id = location.get('id')
+    if loc_id is None:
+        msg = _("The location data has an invalid ID: %d") % loc_id
+        raise exception.Invalid(msg)
+
+    try:
+        session = session or get_session()
+        location_ref = session.query(models.ImageLocation)\
+            .filter_by(id=loc_id)\
+            .filter_by(image_id=image_id)\
+            .one()
+
+        deleted = location['status'] in ('deleted', 'pending_delete')
+        updated_time = timeutils.utcnow()
+        delete_time = updated_time if deleted else None
+
+        location_ref.update({"value": location['url'],
+                             "meta_data": location['metadata'],
+                             "status": location['status'],
+                             "deleted": deleted,
+                             "updated_at": updated_time,
+                             "deleted_at": delete_time})
+        location_ref.save(session=session)
+    except sa_orm.exc.NoResultFound:
+        msg = (_("No location found with ID %(loc)s from image %(img)s") %
+               dict(loc=loc_id, img=image_id))
+        LOG.warn(msg)
+        raise exception.NotFound(msg)
+
+
+def image_location_delete(context, image_id, location_id, status,
+                          delete_time=None, session=None):
+    if status not in ('deleted', 'pending_delete'):
+        msg = _("The status of deleted image location can only be set to "
+                "'pending_delete' or 'deleted'")
+        raise exception.Invalid(msg)
+
+    try:
+        session = session or get_session()
+        location_ref = session.query(models.ImageLocation)\
+            .filter_by(id=location_id)\
+            .filter_by(image_id=image_id)\
+            .one()
+
+        delete_time = delete_time or timeutils.utcnow()
+
+        location_ref.update({"deleted": True,
+                             "status": status,
+                             "updated_at": delete_time,
+                             "deleted_at": delete_time})
+        location_ref.save(session=session)
+    except sa_orm.exc.NoResultFound:
+        msg = (_("No location found with ID %(loc)s from image %(img)s") %
+               dict(loc=location_id, img=image_id))
+        LOG.warn(msg)
+        raise exception.NotFound(msg)
+
+
+def _image_locations_set(context, image_id, locations, session=None):
+    # NOTE(zhiyan): 1. Remove records from DB for deleted locations
+    session = session or get_session()
+    query = session.query(models.ImageLocation) \
+        .filter_by(image_id=image_id) \
+        .filter_by(deleted=False) \
+        .filter(~models.ImageLocation.id.in_([loc['id']
+                                              for loc in locations
+                                              if loc.get('id')]))
+    for loc_id in [loc_ref.id for loc_ref in query.all()]:
+        image_location_delete(context, image_id, loc_id, 'deleted',
+                              session=session)
+
+    # NOTE(zhiyan): 2. Adding or update locations
+    for loc in locations:
+        if loc.get('id') is None:
+            image_location_add(context, image_id, loc, session=session)
+        else:
+            image_location_update(context, image_id, loc, session=session)
+
+
+def _image_locations_delete_all(context, image_id,
+                                delete_time=None, session=None):
     """Delete all image locations for given image"""
-    locs_updated_count = _image_child_entry_delete_all(models.ImageLocation,
-                                                       image_id,
-                                                       delete_time,
-                                                       session)
-    return locs_updated_count
+    session = session or get_session()
+    location_refs = session.query(models.ImageLocation) \
+        .filter_by(image_id=image_id) \
+        .filter_by(deleted=False) \
+        .all()
+
+    for loc_id in [loc_ref.id for loc_ref in location_refs]:
+        image_location_delete(context, image_id, loc_id, 'deleted',
+                              delete_time=delete_time, session=session)
 
 
 def _set_properties_for_image(context, image_ref, properties,
@@ -772,7 +892,7 @@ def _set_properties_for_image(context, image_ref, properties,
     for prop_ref in image_ref.properties:
         orig_properties[prop_ref.name] = prop_ref
 
-    for name, value in properties.iteritems():
+    for name, value in six.iteritems(properties):
         prop_values = {'image_id': image_ref.id,
                        'name': name,
                        'value': value}
@@ -994,20 +1114,21 @@ def _can_show_deleted(context):
 
 
 def image_tag_set_all(context, image_id, tags):
+    #NOTE(kragniz): tag ordering should match exactly what was provided, so a
+    # subsequent call to image_tag_get_all returns them in the correct order
+
     session = get_session()
-    existing_tags = set(image_tag_get_all(context, image_id, session))
-    tags = set(tags)
+    existing_tags = image_tag_get_all(context, image_id, session)
 
-    tags_to_create = tags - existing_tags
-    #NOTE(bcwaldon): we call 'reversed' here to ensure the ImageTag.id fields
-    # will be populated in the order required to reflect the correct ordering
-    # on a subsequent call to image_tag_get_all
-    for tag in reversed(list(tags_to_create)):
-        image_tag_create(context, image_id, tag, session)
+    tags_created = []
+    for tag in tags:
+        if tag not in tags_created and tag not in existing_tags:
+            tags_created.append(tag)
+            image_tag_create(context, image_id, tag, session)
 
-    tags_to_delete = existing_tags - tags
-    for tag in tags_to_delete:
-        image_tag_delete(context, image_id, tag, session)
+    for tag in existing_tags:
+        if tag not in tags:
+            image_tag_delete(context, image_id, tag, session)
 
 
 def image_tag_create(context, image_id, value, session=None):
@@ -1047,12 +1168,11 @@ def image_tag_get_all(context, image_id, session=None):
     """Get a list of tags for a specific image."""
     _check_image_id(image_id)
     session = session or get_session()
-    tags = session.query(models.ImageTag)\
+    tags = session.query(models.ImageTag.value)\
                   .filter_by(image_id=image_id)\
                   .filter_by(deleted=False)\
-                  .order_by(sqlalchemy.asc(models.ImageTag.created_at))\
                   .all()
-    return [tag['value'] for tag in tags]
+    return [tag[0] for tag in tags]
 
 
 def user_get_storage_usage(context, owner_id, image_id=None, session=None):
@@ -1103,7 +1223,7 @@ def _task_info_get(context, task_id, session=None):
     try:
         task_info_ref = query.one()
     except sa_orm.exc.NoResultFound:
-        msg = (_("TaskInfo was not found for task with id %(task_id)s") %
+        msg = ("TaskInfo was not found for task with id %(task_id)s" %
                {'task_id': task_id})
         LOG.debug(msg)
         task_info_ref = None
@@ -1269,13 +1389,13 @@ def _task_get(context, task_id, session=None, force_show_deleted=False):
     try:
         task_ref = query.one()
     except sa_orm.exc.NoResultFound:
-        msg = (_("No task found with ID %s") % task_id)
+        msg = "No task found with ID %s" % task_id
         LOG.debug(msg)
         raise exception.TaskNotFound(task_id=task_id)
 
     # Make sure the task is visible
     if not _is_task_visible(context, task_ref):
-        msg = (_("Forbidding request, task %s is not visible") % task_id)
+        msg = "Forbidding request, task %s is not visible" % task_id
         LOG.debug(msg)
         raise exception.Forbidden(msg)
 
@@ -1313,3 +1433,199 @@ def _task_format(task_ref, task_info_ref=None):
         task_dict.update(task_info_dict)
 
     return task_dict
+
+
+def metadef_namespace_get_all(context, marker=None, limit=None, sort_key=None,
+                              sort_dir=None, filters=None, session=None):
+    """List all available namespaces."""
+    session = session or get_session()
+    namespaces = metadef_namespace_api.get_all(
+        context, session, marker, limit, sort_key, sort_dir, filters)
+    return namespaces
+
+
+def metadef_namespace_get(context, namespace_name, session=None):
+    """Get a namespace or raise if it does not exist or is not visible."""
+    session = session or get_session()
+    return metadef_namespace_api.get(
+        context, namespace_name, session)
+
+
+def metadef_namespace_create(context, values, session=None):
+    """Create a namespace or raise if it already exists."""
+    session = session or get_session()
+    return metadef_namespace_api.create(context, values, session)
+
+
+def metadef_namespace_update(context, namespace_id, namespace_dict,
+                             session=None):
+    """Update a namespace or raise if it does not exist or not visible"""
+    session = session or get_session()
+    return metadef_namespace_api.\
+        update(context, namespace_id, namespace_dict, session)
+
+
+def metadef_namespace_delete(context, namespace_name, session=None):
+    """Delete the namespace and all foreign references"""
+    session = session or get_session()
+    return metadef_namespace_api.delete_cascade(
+        context, namespace_name, session)
+
+
+def metadef_object_get_all(context, namespace_name, session=None):
+    """Get a metadata-schema object or raise if it does not exist."""
+    session = session or get_session()
+    return metadef_object_api.get_all(
+        context, namespace_name, session)
+
+
+def metadef_object_get(context, namespace_name, object_name, session=None):
+    """Get a metadata-schema object or raise if it does not exist."""
+    session = session or get_session()
+    return metadef_object_api.get(
+        context, namespace_name, object_name, session)
+
+
+def metadef_object_create(context, namespace_name, object_dict,
+                          session=None):
+    """Create a metadata-schema object or raise if it already exists."""
+    session = session or get_session()
+    return metadef_object_api.create(
+        context, namespace_name, object_dict, session)
+
+
+def metadef_object_update(context, namespace_name, object_id, object_dict,
+                          session=None):
+    """Update an object or raise if it does not exist or not visible."""
+    session = session or get_session()
+    return metadef_object_api.update(
+        context, namespace_name, object_id, object_dict, session)
+
+
+def metadef_object_delete(context, namespace_name, object_name,
+                          session=None):
+    """Delete an object or raise if namespace or object doesn't exist."""
+    session = session or get_session()
+    return metadef_object_api.delete(
+        context, namespace_name, object_name, session)
+
+
+def metadef_object_delete_namespace_content(
+        context, namespace_name, session=None):
+    """Delete an object or raise if namespace or object doesn't exist."""
+    session = session or get_session()
+    return metadef_object_api.delete_by_namespace_name(
+        context, namespace_name, session)
+
+
+def metadef_object_count(context, namespace_name, session=None):
+    """Get count of properties for a namespace, raise if ns doesn't exist."""
+    session = session or get_session()
+    return metadef_object_api.count(context, namespace_name, session)
+
+
+def metadef_property_get_all(context, namespace_name, session=None):
+    """Get a metadef property or raise if it does not exist."""
+    session = session or get_session()
+    return metadef_property_api.get_all(context, namespace_name, session)
+
+
+def metadef_property_get(context, namespace_name,
+                         property_name, session=None):
+    """Get a metadef property or raise if it does not exist."""
+    session = session or get_session()
+    return metadef_property_api.get(
+        context, namespace_name, property_name, session)
+
+
+def metadef_property_create(context, namespace_name, property_dict,
+                            session=None):
+    """Create a metadef property or raise if it already exists."""
+    session = session or get_session()
+    return metadef_property_api.create(
+        context, namespace_name, property_dict, session)
+
+
+def metadef_property_update(context, namespace_name, property_id,
+                            property_dict, session=None):
+    """Update an object or raise if it does not exist or not visible."""
+    session = session or get_session()
+    return metadef_property_api.update(
+        context, namespace_name, property_id, property_dict, session)
+
+
+def metadef_property_delete(context, namespace_name, property_name,
+                            session=None):
+    """Delete a property or raise if it or namespace doesn't exist."""
+    session = session or get_session()
+    return metadef_property_api.delete(
+        context, namespace_name, property_name, session)
+
+
+def metadef_property_delete_namespace_content(
+        context, namespace_name, session=None):
+    """Delete a property or raise if it or namespace doesn't exist."""
+    session = session or get_session()
+    return metadef_property_api.delete_by_namespace_name(
+        context, namespace_name, session)
+
+
+def metadef_property_count(context, namespace_name, session=None):
+    """Get count of properties for a namespace, raise if ns doesn't exist."""
+    session = session or get_session()
+    return metadef_property_api.count(context, namespace_name, session)
+
+
+def metadef_resource_type_create(context, values, session=None):
+    """Create a resource_type"""
+    session = session or get_session()
+    return metadef_resource_type_api.create(
+        context, values, session)
+
+
+def metadef_resource_type_get(context, resource_type_name, session=None):
+    """Get a resource_type"""
+    session = session or get_session()
+    return metadef_resource_type_api.get(
+        context, resource_type_name, session)
+
+
+def metadef_resource_type_get_all(context, session=None):
+    """list all resource_types"""
+    session = session or get_session()
+    return metadef_resource_type_api.get_all(context, session)
+
+
+def metadef_resource_type_delete(context, resource_type_name, session=None):
+    """Get a resource_type"""
+    session = session or get_session()
+    return metadef_resource_type_api.delete(
+        context, resource_type_name, session)
+
+
+def metadef_resource_type_association_get(
+        context, namespace_name, resource_type_name, session=None):
+    session = session or get_session()
+    return metadef_association_api.get(
+        context, namespace_name, resource_type_name, session)
+
+
+def metadef_resource_type_association_create(
+        context, namespace_name, values, session=None):
+    session = session or get_session()
+    return metadef_association_api.create(
+        context, namespace_name, values, session)
+
+
+def metadef_resource_type_association_delete(
+        context, namespace_name, resource_type_name, session=None):
+    session = session or get_session()
+    return metadef_association_api.delete(
+        context, namespace_name, resource_type_name, session)
+
+
+def metadef_resource_type_association_get_all_by_namespace(
+        context, namespace_name, session=None):
+    session = session or get_session()
+    return metadef_association_api.\
+        get_all_by_namespace(context, namespace_name, session)
