@@ -16,14 +16,15 @@ import sqlalchemy
 from sqlalchemy.schema import (
     Column, ForeignKey, Index, MetaData, Table, UniqueConstraint)  # noqa
 
-from glance.common import timeutils
 from glance.db.sqlalchemy.migrate_repo.schema import (
-    Boolean, DateTime, Integer, String, Text, create_tables)  # noqa
+    Boolean, DateTime, Integer, String, Text, create_tables,
+    drop_tables)  # noqa
+from glance.openstack.common import timeutils
 
 
 RESOURCE_TYPES = [u'OS::Glance::Image', u'OS::Cinder::Volume',
                   u'OS::Nova::Flavor', u'OS::Nova::Aggregate',
-                  u'OS::Nova::Server']
+                  u'OS::Nova::Instance']
 
 
 def _get_metadef_resource_types_table(meta):
@@ -65,7 +66,6 @@ def define_metadef_namespaces_table(meta):
                        Column('updated_at', DateTime()),
                        UniqueConstraint('namespace', **_constr_kwargs),
                        mysql_engine='InnoDB',
-                       mysql_charset='utf8',
                        extend_existing=True)
 
     if meta.bind.name != 'ibm_db_sa':
@@ -95,7 +95,6 @@ def define_metadef_objects_table(meta):
                     UniqueConstraint('namespace_id', 'name',
                                      **_constr_kwargs),
                     mysql_engine='InnoDB',
-                    mysql_charset='utf8',
                     extend_existing=True)
 
     if meta.bind.name != 'ibm_db_sa':
@@ -124,7 +123,6 @@ def define_metadef_properties_table(meta):
         Column('updated_at', DateTime()),
         UniqueConstraint('namespace_id', 'name', **_constr_kwargs),
         mysql_engine='InnoDB',
-        mysql_charset='utf8',
         extend_existing=True)
 
     if meta.bind.name != 'ibm_db_sa':
@@ -151,7 +149,6 @@ def define_metadef_resource_types_table(meta):
         Column('updated_at', DateTime()),
         UniqueConstraint('name', **_constr_kwargs),
         mysql_engine='InnoDB',
-        mysql_charset='utf8',
         extend_existing=True)
 
     if meta.bind.name != 'ibm_db_sa':
@@ -183,7 +180,6 @@ def define_metadef_namespace_resource_types_table(meta):
         UniqueConstraint('resource_type_id', 'namespace_id',
                          **_constr_kwargs),
         mysql_engine='InnoDB',
-        mysql_charset='utf8',
         extend_existing=True)
 
     if meta.bind.name != 'ibm_db_sa':
@@ -206,3 +202,14 @@ def upgrade(migrate_engine):
 
     resource_types_table = _get_metadef_resource_types_table(meta)
     _populate_resource_types(resource_types_table)
+
+
+def downgrade(migrate_engine):
+    meta = MetaData()
+    meta.bind = migrate_engine
+    tables = [define_metadef_objects_table(meta),
+              define_metadef_properties_table(meta),
+              define_metadef_namespace_resource_types_table(meta),
+              define_metadef_resource_types_table(meta),
+              define_metadef_namespaces_table(meta)]
+    drop_tables(tables)
