@@ -23,11 +23,16 @@ from oslo.config import cfg
 
 from glance.common import exception
 from glance.common import utils
+from glance.openstack.common import excutils
+from glance.openstack.common import gettextutils
 from glance.openstack.common import importutils
 import glance.openstack.common.log as logging
 from glance.openstack.common import units
 
 LOG = logging.getLogger(__name__)
+_LE = gettextutils._LE
+_LI = gettextutils._LI
+_LW = gettextutils._LW
 
 image_cache_opts = [
     cfg.StrOpt('image_cache_driver', default='sqlite',
@@ -60,17 +65,17 @@ class ImageCache(object):
         driver_module = (__name__ + '.drivers.' + driver_name + '.Driver')
         try:
             self.driver_class = importutils.import_class(driver_module)
-            LOG.info(_("Image cache loaded driver '%s'.") %
+            LOG.info(_LI("Image cache loaded driver '%s'.") %
                      driver_name)
         except ImportError as import_err:
-            LOG.warn(_("Image cache driver "
-                       "'%(driver_name)s' failed to load. "
-                       "Got error: '%(import_err)s."),
+            LOG.warn(_LW("Image cache driver "
+                         "'%(driver_name)s' failed to load. "
+                         "Got error: '%(import_err)s."),
                      {'driver_name': driver_name,
                       'import_err': import_err})
 
             driver_module = __name__ + '.drivers.sqlite.Driver'
-            LOG.info(_("Defaulting to SQLite driver."))
+            LOG.info(_LI("Defaulting to SQLite driver."))
             self.driver_class = importutils.import_class(driver_module)
         self.configure_driver()
 
@@ -84,12 +89,12 @@ class ImageCache(object):
             self.driver.configure()
         except exception.BadDriverConfiguration as config_err:
             driver_module = self.driver_class.__module__
-            LOG.warn(_("Image cache driver "
-                       "'%(driver_module)s' failed to configure. "
-                       "Got error: '%(config_err)s"),
+            LOG.warn(_LW("Image cache driver "
+                         "'%(driver_module)s' failed to configure. "
+                         "Got error: '%(config_err)s"),
                      {'driver_module': driver_module,
                       'config_err': config_err})
-            LOG.info(_("Defaulting to SQLite driver."))
+            LOG.info(_LI("Defaulting to SQLite driver."))
             default_module = __name__ + '.drivers.sqlite.Driver'
             self.driver_class = importutils.import_class(default_module)
             self.driver = self.driver_class()
@@ -171,12 +176,12 @@ class ImageCache(object):
         max_size = CONF.image_cache_max_size
         current_size = self.driver.get_cache_size()
         if max_size > current_size:
-            LOG.debug(_("Image cache has free space, skipping prune..."))
+            LOG.debug("Image cache has free space, skipping prune...")
             return (0, 0)
 
         overage = current_size - max_size
-        LOG.debug(_("Image cache currently %(overage)d bytes over max "
-                    "size. Starting prune to max size of %(max_size)d "),
+        LOG.debug("Image cache currently %(overage)d bytes over max "
+                  "size. Starting prune to max size of %(max_size)d ",
                   {'overage': overage, 'max_size': max_size})
 
         total_bytes_pruned = 0
@@ -184,7 +189,7 @@ class ImageCache(object):
         entry = self.driver.get_least_recently_accessed()
         while entry and current_size > max_size:
             image_id, size = entry
-            LOG.debug(_("Pruning '%(image_id)s' to free %(size)d bytes"),
+            LOG.debug("Pruning '%(image_id)s' to free %(size)d bytes",
                       {'image_id': image_id, 'size': size})
             self.driver.delete_cached_image(image_id)
             total_bytes_pruned = total_bytes_pruned + size
@@ -192,9 +197,9 @@ class ImageCache(object):
             current_size = current_size - size
             entry = self.driver.get_least_recently_accessed()
 
-        LOG.debug(_("Pruning finished pruning. "
-                    "Pruned %(total_files_pruned)d and "
-                    "%(total_bytes_pruned)d."),
+        LOG.debug("Pruning finished pruning. "
+                  "Pruned %(total_files_pruned)d and "
+                  "%(total_bytes_pruned)d.",
                   {'total_files_pruned': total_files_pruned,
                    'total_bytes_pruned': total_bytes_pruned})
         return total_files_pruned, total_bytes_pruned
@@ -231,7 +236,7 @@ class ImageCache(object):
         if not self.driver.is_cacheable(image_id):
             return image_iter
 
-        LOG.debug(_("Tee'ing image '%s' into cache"), image_id)
+        LOG.debug("Tee'ing image '%s' into cache", image_id)
 
         return self.cache_tee_iter(image_id, image_iter, image_checksum)
 
@@ -255,16 +260,16 @@ class ImageCache(object):
                     raise exception.GlanceException(msg)
 
         except exception.GlanceException as e:
-            # image_iter has given us bad, (size_checked_iter has found a
-            # bad length), or corrupt data (checksum is wrong).
-            LOG.exception(e)
-            raise
+            with excutils.save_and_reraise_exception():
+                # image_iter has given us bad, (size_checked_iter has found a
+                # bad length), or corrupt data (checksum is wrong).
+                LOG.exception(utils.exception_to_str(e))
         except Exception as e:
-            LOG.exception(_("Exception encountered while tee'ing "
-                            "image '%(image_id)s' into cache: %(error)s. "
-                            "Continuing with response.") %
+            LOG.exception(_LE("Exception encountered while tee'ing "
+                              "image '%(image_id)s' into cache: %(error)s. "
+                              "Continuing with response.") %
                           {'image_id': image_id,
-                           'error': e})
+                           'error': utils.exception_to_str(e)})
 
             # If no checksum provided continue responding even if
             # caching failed.
