@@ -14,24 +14,19 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import glance_store
 from oslo.config import cfg
 from oslo import messaging
 import webob
 
 from glance.common import exception
+from glance.common import utils
 import glance.domain.proxy
+from glance.openstack.common import excutils
 import glance.openstack.common.log as logging
 from glance.openstack.common import timeutils
 
 notifier_opts = [
-    cfg.StrOpt('notifier_strategy', default='default',
-               help=_('Notifications can be sent when images are create, '
-                      'updated or deleted. There are three methods of sending '
-                      'notifications, logging (via the log_file directive), '
-                      'rabbit (via a rabbitmq queue), qpid (via a Qpid '
-                      'message queue), or noop (no notifications sent, the '
-                      'default). (DEPRECATED)')),
-
     cfg.StrOpt('default_publisher_id', default="image.localhost",
                help='Default publisher_id for outgoing notifications.'),
 ]
@@ -41,14 +36,6 @@ CONF.register_opts(notifier_opts)
 
 LOG = logging.getLogger(__name__)
 
-_STRATEGY_ALIASES = {
-    "logging": "log",
-    "rabbit": "messaging",
-    "qpid": "messaging",
-    "noop": "noop",
-    "default": "noop",
-}
-
 _ALIASES = {
     'glance.openstack.common.rpc.impl_kombu': 'rabbit',
     'glance.openstack.common.rpc.impl_qpid': 'qpid',
@@ -56,60 +43,17 @@ _ALIASES = {
 }
 
 
+def get_transport():
+    return messaging.get_transport(CONF, aliases=_ALIASES)
+
+
 class Notifier(object):
     """Uses a notification strategy to send out messages about events."""
 
-    def __init__(self, strategy=None):
-
-        _driver = None
-        _strategy = strategy
-
-        if CONF.notifier_strategy != 'default':
-            msg = _("notifier_strategy was deprecated in "
-                    "favor of `notification_driver`")
-            LOG.warn(msg)
-
-            # NOTE(flaper87): Use this to keep backwards
-            # compatibility. We'll try to get an oslo.messaging
-            # driver from the specified strategy.
-            _strategy = strategy or CONF.notifier_strategy
-            _driver = _STRATEGY_ALIASES.get(_strategy)
-
+    def __init__(self):
         publisher_id = CONF.default_publisher_id
-
-        try:
-            # NOTE(flaper87): Assume the user has configured
-            # the transport url.
-            self._transport = messaging.get_transport(CONF,
-                                                      aliases=_ALIASES)
-        except messaging.DriverLoadFailure:
-            # NOTE(flaper87): Catch driver load failures and re-raise
-            # them *just* if the `transport_url` option was set. This
-            # step is intended to keep backwards compatibility and avoid
-            # weird behaviors (like exceptions on missing dependencies)
-            # when the old notifier options are used.
-            if CONF.transport_url is not None:
-                with excutils.save_and_reraise_exception():
-                    LOG.exception(_('Error loading the notifier'))
-
-        # NOTE(flaper87): This needs to be checked
-        # here because the `get_transport` call
-        # registers `transport_url` into ConfigOpts.
-        if not CONF.transport_url:
-            # NOTE(flaper87): The next 3 lines help
-            # with the migration to oslo.messaging.
-            # Without them, gate tests won't know
-            # what driver should be loaded.
-            # Once this patch lands, devstack will be
-            # updated and then these lines will be removed.
-            url = None
-            if _strategy in ['rabbit', 'qpid']:
-                url = _strategy + '://'
-            self._transport = messaging.get_transport(CONF, url,
-                                                      aliases=_ALIASES)
-
+        self._transport = get_transport()
         self._notifier = messaging.Notifier(self._transport,
-                                            driver=_driver,
                                             publisher_id=publisher_id)
 
     def warn(self, event_type, payload):
@@ -178,8 +122,8 @@ class ImageRepoProxy(glance.domain.proxy.Repo):
                                              item_proxy_class=ImageProxy,
                                              item_proxy_kwargs=proxy_kwargs)
 
-    def save(self, image, from_state=None):
-        super(ImageRepoProxy, self).save(image, from_state=from_state)
+    def save(self, image):
+        super(ImageRepoProxy, self).save(image)
         self.notifier.info('image.update',
                            format_image_notification(image))
 
@@ -221,13 +165,13 @@ class ImageProxy(glance.domain.proxy.Image):
             'receiver_user_id': self.context.user,
         }
 
-    def get_data(self):
+    def _get_chunk_data_iterator(self, data, chunk_size=None):
         sent = 0
-        for chunk in self.image.get_data():
+        for chunk in data:
             yield chunk
             sent += len(chunk)
 
-        if sent != self.image.size:
+        if sent != (chunk_size or self.image.size):
             notify = self.notifier.error
         else:
             notify = self.notifier.info
@@ -240,60 +184,69 @@ class ImageProxy(glance.domain.proxy.Image):
                      " notification: %(err)s") % {'err': err})
             LOG.error(msg)
 
+    def get_data(self, offset=0, chunk_size=None):
+        # Due to the need of evaluating subsequent proxies, this one
+        # should return a generator, the call should be done before
+        # generator creation
+        data = self.image.get_data(offset=offset, chunk_size=chunk_size)
+        return self._get_chunk_data_iterator(data, chunk_size=chunk_size)
+
     def set_data(self, data, size=None):
         payload = format_image_notification(self.image)
         self.notifier.info('image.prepare', payload)
         try:
             self.image.set_data(data, size)
-        except exception.StorageFull as e:
-            msg = (_("Image storage media is full: %s") % e)
+        except glance_store.StorageFull as e:
+            msg = (_("Image storage media is full: %s") %
+                   utils.exception_to_str(e))
             self.notifier.error('image.upload', msg)
             raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg)
-        except exception.StorageWriteDenied as e:
+        except glance_store.StorageWriteDenied as e:
             msg = (_("Insufficient permissions on image storage media: %s")
-                   % e)
+                   % utils.exception_to_str(e))
             self.notifier.error('image.upload', msg)
             raise webob.exc.HTTPServiceUnavailable(explanation=msg)
         except ValueError as e:
             msg = (_("Cannot save data for image %(image_id)s: %(error)s") %
                    {'image_id': self.image.image_id,
-                    'error': e})
+                    'error': utils.exception_to_str(e)})
             self.notifier.error('image.upload', msg)
-            raise webob.exc.HTTPBadRequest(explanation=unicode(e))
+            raise webob.exc.HTTPBadRequest(explanation=
+                                           utils.exception_to_str(e))
         except exception.Duplicate as e:
             msg = (_("Unable to upload duplicate image data for image"
                      "%(image_id)s: %(error)s") %
                    {'image_id': self.image.image_id,
-                    'error': e})
+                    'error': utils.exception_to_str(e)})
             self.notifier.error('image.upload', msg)
             raise webob.exc.HTTPConflict(explanation=msg)
         except exception.Forbidden as e:
             msg = (_("Not allowed to upload image data for image %(image_id)s:"
                      " %(error)s") % {'image_id': self.image.image_id,
-                                      'error': e})
+                                      'error': utils.exception_to_str(e)})
             self.notifier.error('image.upload', msg)
             raise webob.exc.HTTPForbidden(explanation=msg)
         except exception.NotFound as e:
             msg = (_("Image %(image_id)s could not be found after upload."
                      " The image may have been deleted during the upload:"
                      " %(error)s") % {'image_id': self.image.image_id,
-                                      'error': e})
+                                      'error': utils.exception_to_str(e)})
             self.notifier.error('image.upload', msg)
-            raise webob.exc.HTTPNotFound(explanation=unicode(e))
+            raise webob.exc.HTTPNotFound(explanation=utils.exception_to_str(e))
         except webob.exc.HTTPError as e:
-            msg = (_("Failed to upload image data for image %(image_id)s"
-                     " due to HTTP error: %(error)s") %
-                   {'image_id': self.image.image_id,
-                    'error': e})
-            self.notifier.error('image.upload', msg)
-            raise
+            with excutils.save_and_reraise_exception():
+                msg = (_("Failed to upload image data for image %(image_id)s"
+                         " due to HTTP error: %(error)s") %
+                       {'image_id': self.image.image_id,
+                        'error': utils.exception_to_str(e)})
+                self.notifier.error('image.upload', msg)
         except Exception as e:
-            msg = (_("Failed to upload image data for image %(image_id)s "
-                     "due to internal error: %(error)s") %
-                   {'image_id': self.image.image_id,
-                    'error': e})
-            self.notifier.error('image.upload', msg)
-            raise
+            with excutils.save_and_reraise_exception():
+                msg = (_("Failed to upload image data for image %(image_id)s "
+                         "due to internal error: %(error)s") %
+                       {'image_id': self.image.image_id,
+                        'error': utils.exception_to_str(e)})
+                self.notifier.error('image.upload', msg)
         else:
             payload = format_image_notification(self.image)
             self.notifier.info('image.upload', payload)
@@ -310,14 +263,12 @@ class TaskRepoProxy(glance.domain.proxy.TaskRepo):
         super(TaskRepoProxy, self) \
             .__init__(task_repo,
                       task_proxy_class=TaskProxy,
-                      task_proxy_kwargs=proxy_kwargs,
-                      task_details_proxy_class=TaskDetailsProxy,
-                      task_details_proxy_kwargs=proxy_kwargs)
+                      task_proxy_kwargs=proxy_kwargs)
 
-    def add(self, task, task_details=None):
+    def add(self, task):
         self.notifier.info('task.create',
                            format_task_notification(task))
-        super(TaskRepoProxy, self).add(task, task_details)
+        super(TaskRepoProxy, self).add(task)
 
     def remove(self, task):
         payload = format_task_notification(task)
@@ -327,15 +278,26 @@ class TaskRepoProxy(glance.domain.proxy.TaskRepo):
         super(TaskRepoProxy, self).remove(task)
 
 
+class TaskStubRepoProxy(glance.domain.proxy.TaskStubRepo):
+
+    def __init__(self, task_stub_repo, context, notifier):
+        self.task_stub_repo = task_stub_repo
+        self.context = context
+        self.notifier = notifier
+        proxy_kwargs = {'context': self.context, 'notifier': self.notifier}
+        super(TaskStubRepoProxy, self) \
+            .__init__(task_stub_repo,
+                      task_stub_proxy_class=TaskStubProxy,
+                      task_stub_proxy_kwargs=proxy_kwargs)
+
+
 class TaskFactoryProxy(glance.domain.proxy.TaskFactory):
     def __init__(self, task_factory, context, notifier):
         kwargs = {'context': context, 'notifier': notifier}
         super(TaskFactoryProxy, self).__init__(
             task_factory,
             task_proxy_class=TaskProxy,
-            task_proxy_kwargs=kwargs,
-            task_details_proxy_class=TaskDetailsProxy,
-            task_details_proxy_kwargs=kwargs)
+            task_proxy_kwargs=kwargs)
 
 
 class TaskProxy(glance.domain.proxy.Task):
@@ -346,11 +308,6 @@ class TaskProxy(glance.domain.proxy.Task):
         self.notifier = notifier
         super(TaskProxy, self).__init__(task)
 
-    def run(self, executor):
-        self.notifier.info('task.run',
-                           format_task_notification(self.task))
-        return super(TaskProxy, self).run(executor)
-
     def begin_processing(self):
         self.notifier.info(
             'task.processing',
@@ -368,11 +325,16 @@ class TaskProxy(glance.domain.proxy.Task):
                            format_task_notification(self.task))
         return super(TaskProxy, self).fail(message)
 
+    def run(self, executor):
+        self.notifier.info('task.run',
+                           format_task_notification(self.task))
+        return super(TaskProxy, self).run(executor)
 
-class TaskDetailsProxy(glance.domain.proxy.TaskDetails):
 
-    def __init__(self, task_details, context, notifier):
-        self.task_details = task_details
+class TaskStubProxy(glance.domain.proxy.TaskStub):
+
+    def __init__(self, task, context, notifier):
+        self.task = task
         self.context = context
         self.notifier = notifier
-        super(TaskDetailsProxy, self).__init__(task_details)
+        super(TaskStubProxy, self).__init__(task)
