@@ -20,17 +20,24 @@ import os
 import time
 
 from oslo.config import cfg
+import six
 
 from glance.common import crypt
 from glance.common import exception
 from glance.common import utils
 from glance import context
+import glance.db as db_api
+from glance import i18n
 from glance.openstack.common import lockutils
 import glance.openstack.common.log as logging
 import glance.registry.client.v1.api as registry
 
 LOG = logging.getLogger(__name__)
 
+_LI = i18n._LI
+_LW = i18n._LW
+_LE = i18n._LE
+
 scrubber_opts = [
     cfg.StrOpt('scrubber_datadir',
                default='/var/lib/glance/scrubber',
@@ -46,11 +53,30 @@ scrubber_opts = [
                        'clean up the files it uses for taking data. Only '
                        'one server in your deployment should be designated '
                        'the cleanup host.')),
+    cfg.BoolOpt('delayed_delete', default=False,
+                help=_('Turn on/off delayed delete.')),
     cfg.IntOpt('cleanup_scrubber_time', default=86400,
                help=_('Items must have a modified time that is older than '
                       'this value in order to be candidates for cleanup.'))
 ]
 
+scrubber_cmd_opts = [
+    cfg.IntOpt('wakeup_time', default=300,
+               help=_('Loop time between checking for new '
+                      'items to schedule for delete.'))
+]
+
+scrubber_cmd_cli_opts = [
+    cfg.BoolOpt('daemon',
+                short='D',
+                default=False,
+                help=_('Run as a long-running process. When not '
+                       'specified (the default) run the scrub operation '
+                       'once and then exits. When specified do not exit '
+                       'and run scrub on wakeup_time interval as '
+                       'specified in the config.'))
+]
+
 CONF = cfg.CONF
 CONF.register_opts(scrubber_opts)
 CONF.import_opt('metadata_encryption_key', 'glance.common.config')
@@ -62,17 +88,21 @@ class ScrubQueue(object):
     The queue contains image's location which need to delete from backend.
     """
     def __init__(self):
+        self.scrub_time = CONF.scrub_time
+        self.metadata_encryption_key = CONF.metadata_encryption_key
         registry.configure_registry_client()
         registry.configure_registry_admin_creds()
         self.registry = registry.get_registry_client(context.RequestContext())
 
     @abc.abstractmethod
-    def add_location(self, image_id, uri, user_context=None):
+    def add_location(self, image_id, location, user_context=None):
         """Adding image location to scrub queue.
 
         :param image_id: The opaque image identifier
-        :param uri: The opaque image location uri
+        :param location: The opaque image location
         :param user_context: The user's request context
+
+        :retval A boolean value to indicate success or not
         """
         pass
 
@@ -108,32 +138,32 @@ class ScrubFileQueue(ScrubQueue):
         super(ScrubFileQueue, self).__init__()
         self.scrubber_datadir = CONF.scrubber_datadir
         utils.safe_mkdirs(self.scrubber_datadir)
-        self.scrub_time = CONF.scrub_time
-        self.metadata_encryption_key = CONF.metadata_encryption_key
 
     def _read_queue_file(self, file_path):
         """Reading queue file to loading deleted location and timestamp out.
 
         :param file_path: Queue file full path
 
-        :retval a list of image location timestamp tuple from queue file
+        :retval a list of image location id, uri and timestamp tuple
         """
+        loc_ids = []
         uris = []
         delete_times = []
 
         try:
             with open(file_path, 'r') as f:
                 while True:
-                    uri = f.readline().strip()
-                    if uri:
-                        uris.append(uri)
+                    loc_id = f.readline().strip()
+                    if loc_id:
+                        lid = six.text_type(loc_id)
+                        loc_ids.append(int(lid) if lid.isdigit() else lid)
+                        uris.append(unicode(f.readline().strip()))
                         delete_times.append(int(f.readline().strip()))
                     else:
                         break
+            return loc_ids, uris, delete_times
         except Exception:
-            LOG.error(_("%s file can not be read.") % file_path)
-
-        return uris, delete_times
+            LOG.error(_LE("%s file can not be read.") % file_path)
 
     def _update_queue_file(self, file_path, remove_record_idxs):
         """Updating queue file to remove such queue records.
@@ -148,21 +178,24 @@ class ScrubFileQueue(ScrubQueue):
             # keep record index be valid.
             remove_record_idxs.sort(reverse=True)
             for record_idx in remove_record_idxs:
-                # Each record has two lines
-                line_no = (record_idx + 1) * 2 - 1
-                del lines[line_no:line_no + 2]
+                # Each record has three lines:
+                # location id, uri and delete time.
+                line_no = (record_idx + 1) * 3 - 1
+                del lines[line_no:line_no + 3]
             with open(file_path, 'w') as f:
                 f.write(''.join(lines))
             os.chmod(file_path, 0o600)
         except Exception:
-            LOG.error(_("%s file can not be wrote.") % file_path)
+            LOG.error(_LE("%s file can not be wrote.") % file_path)
 
-    def add_location(self, image_id, uri, user_context=None):
+    def add_location(self, image_id, location, user_context=None):
         """Adding image location to scrub queue.
 
         :param image_id: The opaque image identifier
-        :param uri: The opaque image location uri
+        :param location: The opaque image location
         :param user_context: The user's request context
+
+        :retval A boolean value to indicate success or not
         """
         if user_context is not None:
             registry_client = registry.get_registry_client(user_context)
@@ -178,41 +211,50 @@ class ScrubFileQueue(ScrubQueue):
             try:
                 image = registry_client.get_image(image_id)
                 if image['status'] == 'deleted':
-                    return
+                    return True
             except exception.NotFound as e:
-                LOG.error(_("Failed to find image to delete: "
-                            "%(e)s"), {'e': e})
-                return
+                LOG.warn(_LW("Failed to find image to delete: %s"),
+                         utils.exception_to_str(e))
+                return False
 
+            loc_id = location.get('id', '-')
+            if self.metadata_encryption_key:
+                uri = crypt.urlsafe_encrypt(self.metadata_encryption_key,
+                                            location['url'], 64)
+            else:
+                uri = location['url']
             delete_time = time.time() + self.scrub_time
             file_path = os.path.join(self.scrubber_datadir, str(image_id))
 
-            if self.metadata_encryption_key is not None:
-                uri = crypt.urlsafe_encrypt(self.metadata_encryption_key,
-                                            uri, 64)
-
             if os.path.exists(file_path):
                 # Append the uri of location to the queue file
                 with open(file_path, 'a') as f:
                     f.write('\n')
-                    f.write('\n'.join([uri, str(int(delete_time))]))
+                    f.write('\n'.join([str(loc_id),
+                                       uri,
+                                       str(int(delete_time))]))
             else:
                 # NOTE(zhiyan): Protect the file before we write any data.
                 open(file_path, 'w').close()
                 os.chmod(file_path, 0o600)
                 with open(file_path, 'w') as f:
-                    f.write('\n'.join([uri, str(int(delete_time))]))
+                    f.write('\n'.join([str(loc_id),
+                                       uri,
+                                       str(int(delete_time))]))
             os.utime(file_path, (delete_time, delete_time))
 
+            return True
+
     def _walk_all_locations(self, remove=False):
         """Returns a list of image id and location tuple from scrub queue.
 
         :param remove: Whether remove location from queue or not after walk
 
-        :retval a list of image image_id and location tuple from scrub queue
+        :retval a list of image id, location id and uri tuple from scrub queue
         """
         if not os.path.exists(self.scrubber_datadir):
-            LOG.info(_("%s directory does not exist.") % self.scrubber_datadir)
+            LOG.info(_LI("%s directory does not exist.") %
+                     self.scrubber_datadir)
             return []
 
         ret = []
@@ -223,7 +265,8 @@ class ScrubFileQueue(ScrubQueue):
                 with lockutils.lock("scrubber-%s" % image_id,
                                     lock_file_prefix='glance-', external=True):
                     file_path = os.path.join(self.scrubber_datadir, image_id)
-                    uris, delete_times = self._read_queue_file(file_path)
+                    records = self._read_queue_file(file_path)
+                    loc_ids, uris, delete_times = records
 
                     remove_record_idxs = []
                     skipped = False
@@ -232,8 +275,11 @@ class ScrubFileQueue(ScrubQueue):
                             skipped = True
                             continue
                         else:
-                            ret.append((image_id, uris[record_idx]))
+                            ret.append((image_id,
+                                        loc_ids[record_idx],
+                                        uris[record_idx]))
                             remove_record_idxs.append(record_idx)
+
                     if remove:
                         if skipped:
                             # NOTE(zhiyan): remove location records from
@@ -273,23 +319,36 @@ class ScrubDBQueue(ScrubQueue):
     """Database-based image scrub queue class."""
     def __init__(self):
         super(ScrubDBQueue, self).__init__()
-        self.cleanup_scrubber_time = CONF.cleanup_scrubber_time
+        admin_tenant_name = CONF.admin_tenant_name
+        admin_token = self.registry.auth_tok
+        self.admin_context = context.RequestContext(user=CONF.admin_user,
+                                                    tenant=admin_tenant_name,
+                                                    auth_tok=admin_token)
 
-    def add_location(self, image_id, uri, user_context=None):
+    def add_location(self, image_id, location, user_context=None):
         """Adding image location to scrub queue.
 
         :param image_id: The opaque image identifier
-        :param uri: The opaque image location uri
+        :param location: The opaque image location
         :param user_context: The user's request context
+
+        :retval A boolean value to indicate success or not
         """
-        raise NotImplementedError
+        loc_id = location.get('id')
+        if loc_id:
+            db_api.get_api().image_location_delete(self.admin_context,
+                                                   image_id, loc_id,
+                                                   'pending_delete')
+            return True
+        else:
+            return False
 
     def _walk_all_locations(self, remove=False):
         """Returns a list of image id and location tuple from scrub queue.
 
         :param remove: Whether remove location from queue or not after walk
 
-        :retval a list of image id and location tuple from scrub queue
+        :retval a list of image id, location id and uri tuple from scrub queue
         """
         filters = {'deleted': True,
                    'is_public': 'none',
@@ -306,14 +365,28 @@ class ScrubDBQueue(ScrubQueue):
             delete_time = calendar.timegm(time.strptime(date_str,
                                                         "%Y-%m-%dT%H:%M:%S"))
 
-            if delete_time + self.cleanup_scrubber_time > time.time():
+            if delete_time + self.scrub_time > time.time():
                 continue
 
-            ret.extend([(image['id'], location['uri'])
-                        for location in image['location_data']])
+            for loc in image['location_data']:
+                if loc['status'] != 'pending_delete':
+                    continue
 
-            if remove:
-                self.registry.update_image(image['id'], {'status': 'deleted'})
+                if self.metadata_encryption_key:
+                    uri = crypt.urlsafe_encrypt(self.metadata_encryption_key,
+                                                loc['url'], 64)
+                else:
+                    uri = loc['url']
+
+                ret.append((image['id'], loc['id'], uri))
+
+                if remove:
+                    db_api.get_api().image_location_delete(self.admin_context,
+                                                           image['id'],
+                                                           loc['id'],
+                                                           'deleted')
+                    self.registry.update_image(image['id'],
+                                               {'status': 'deleted'})
         return ret
 
     def get_all_locations(self):
@@ -359,8 +432,8 @@ def get_scrub_queues():
 
 class Daemon(object):
     def __init__(self, wakeup_time=300, threads=1000):
-        LOG.info(_("Starting Daemon: wakeup_time=%(wakeup_time)s "
-                   "threads=%(threads)s"),
+        LOG.info(_LI("Starting Daemon: wakeup_time=%(wakeup_time)s "
+                     "threads=%(threads)s"),
                  {'wakeup_time': wakeup_time, 'threads': threads})
         self.wakeup_time = wakeup_time
         self.event = eventlet.event.Event()
@@ -373,24 +446,24 @@ class Daemon(object):
         try:
             self.event.wait()
         except KeyboardInterrupt:
-            msg = _("Daemon Shutdown on KeyboardInterrupt")
+            msg = _LI("Daemon Shutdown on KeyboardInterrupt")
             LOG.info(msg)
 
     def _run(self, application):
-        LOG.debug(_("Running application"))
+        LOG.debug("Running application")
         self.pool.spawn_n(application.run, self.pool, self.event)
         eventlet.spawn_after(self.wakeup_time, self._run, application)
-        LOG.debug(_("Next run scheduled in %s seconds") % self.wakeup_time)
+        LOG.debug("Next run scheduled in %s seconds" % self.wakeup_time)
 
 
 class Scrubber(object):
     def __init__(self, store_api):
-        LOG.info(_("Initializing scrubber with configuration: %s") %
-                 unicode({'scrubber_datadir': CONF.scrubber_datadir,
-                          'cleanup': CONF.cleanup_scrubber,
-                          'cleanup_time': CONF.cleanup_scrubber_time,
-                          'registry_host': CONF.registry_host,
-                          'registry_port': CONF.registry_port}))
+        LOG.info(_LI("Initializing scrubber with configuration: %s") %
+                 six.text_type({'scrubber_datadir': CONF.scrubber_datadir,
+                                'cleanup': CONF.cleanup_scrubber,
+                                'cleanup_time': CONF.cleanup_scrubber_time,
+                                'registry_host': CONF.registry_host,
+                                'registry_port': CONF.registry_port}))
 
         utils.safe_mkdirs(CONF.scrubber_datadir)
 
@@ -400,30 +473,55 @@ class Scrubber(object):
         registry.configure_registry_admin_creds()
         self.registry = registry.get_registry_client(context.RequestContext())
 
+        # Here we create a request context with credentials to support
+        # delayed delete when using multi-tenant backend storage
+        admin_tenant = CONF.admin_tenant_name
+        auth_token = self.registry.auth_tok
+        self.admin_context = context.RequestContext(user=CONF.admin_user,
+                                                    tenant=admin_tenant,
+                                                    auth_tok=auth_token)
+
         (self.file_queue, self.db_queue) = get_scrub_queues()
 
     def _get_delete_jobs(self, queue, pop):
         try:
             if pop:
-                image_id_uri_list = queue.pop_all_locations()
+                records = queue.pop_all_locations()
             else:
-                image_id_uri_list = queue.get_all_locations()
-        except Exception:
-            LOG.error(_("Can not %s scrub jobs from queue.") %
-                      'pop' if pop else 'get')
-            return None
+                records = queue.get_all_locations()
+        except Exception as err:
+            LOG.error(_LE("Can not %(op)s scrub jobs from queue: %(err)s") %
+                      {'op': 'pop' if pop else 'get',
+                       'err': utils.exception_to_str(err)})
+            return {}
 
         delete_jobs = {}
-        for image_id, image_uri in image_id_uri_list:
+        for image_id, loc_id, loc_uri in records:
             if image_id not in delete_jobs:
                 delete_jobs[image_id] = []
-            delete_jobs[image_id].append((image_id, image_uri))
+            delete_jobs[image_id].append((image_id, loc_id, loc_uri))
         return delete_jobs
 
+    def _merge_delete_jobs(self, file_jobs, db_jobs):
+        ret = {}
+        for image_id, file_job_items in file_jobs.iteritems():
+            ret[image_id] = file_job_items
+            db_job_items = db_jobs.get(image_id, [])
+            for db_item in db_job_items:
+                if db_item not in file_job_items:
+                    ret[image_id].append(db_item)
+        for image_id, db_job_items in db_jobs.iteritems():
+            if image_id not in ret:
+                ret[image_id] = db_job_items
+        return ret
+
     def run(self, pool, event=None):
-        delete_jobs = self._get_delete_jobs(self.file_queue, True)
+        file_jobs = self._get_delete_jobs(self.file_queue, True)
+        db_jobs = self._get_delete_jobs(self.db_queue, False)
+        delete_jobs = self._merge_delete_jobs(file_jobs, db_jobs)
+
         if delete_jobs:
-            for image_id, jobs in delete_jobs.iteritems():
+            for image_id, jobs in six.iteritems(delete_jobs):
                 self._scrub_image(pool, image_id, jobs)
 
         if CONF.cleanup_scrubber:
@@ -433,36 +531,32 @@ class Scrubber(object):
         if len(delete_jobs) == 0:
             return
 
-        LOG.info(_("Scrubbing image %(id)s from %(count)d locations.") %
+        LOG.info(_LI("Scrubbing image %(id)s from %(count)d locations.") %
                  {'id': image_id, 'count': len(delete_jobs)})
         # NOTE(bourke): The starmap must be iterated to do work
-        list(pool.starmap(self._delete_image_from_backend, delete_jobs))
+        list(pool.starmap(self._delete_image_location_from_backend,
+                          delete_jobs))
 
         image = self.registry.get_image(image_id)
         if (image['status'] == 'pending_delete' and
                 not self.file_queue.has_image(image_id)):
             self.registry.update_image(image_id, {'status': 'deleted'})
 
-    def _delete_image_from_backend(self, image_id, uri):
-        if CONF.metadata_encryption_key is not None:
+    def _delete_image_location_from_backend(self, image_id, loc_id, uri):
+        if CONF.metadata_encryption_key:
             uri = crypt.urlsafe_decrypt(CONF.metadata_encryption_key, uri)
 
         try:
-            LOG.debug(_("Deleting URI from image %(image_id)s.") %
-                      {'image_id': image_id})
-
-            # Here we create a request context with credentials to support
-            # delayed delete when using multi-tenant backend storage
-            admin_tenant = CONF.admin_tenant_name
-            auth_token = self.registry.auth_tok
-            admin_context = context.RequestContext(user=CONF.admin_user,
-                                                   tenant=admin_tenant,
-                                                   auth_tok=auth_token)
-
-            self.store_api.delete_from_backend(admin_context, uri)
+            LOG.debug("Deleting URI from image %s." % image_id)
+            self.store_api.delete_from_backend(self.admin_context, uri)
+            if loc_id != '-':
+                db_api.get_api().image_location_delete(self.admin_context,
+                                                       image_id,
+                                                       int(loc_id),
+                                                       'deleted')
+            LOG.info(_LI("Image %s has been deleted.") % image_id)
         except Exception:
-            msg = _("Failed to delete URI from image %(image_id)s")
-            LOG.error(msg % {'image_id': image_id})
+            LOG.warn(_LW("Unable to delete URI from image %s.") % image_id)
 
     def _read_cleanup_file(self, file_path):
         """Reading cleanup to get latest cleanup timestamp.
@@ -473,17 +567,17 @@ class Scrubber(object):
         """
         try:
             if not os.path.exists(file_path):
-                msg = _("%s file is not exists.") % unicode(file_path)
+                msg = _("%s file is not exists.") % six.text_type(file_path)
                 raise Exception(msg)
             atime = int(os.path.getatime(file_path))
             mtime = int(os.path.getmtime(file_path))
             if atime != mtime:
                 msg = _("%s file contains conflicting cleanup "
-                        "timestamp.") % unicode(file_path)
+                        "timestamp.") % six.text_type(file_path)
                 raise Exception(msg)
             return atime
         except Exception as e:
-            LOG.error(e)
+            LOG.error(utils.exception_to_str(e))
         return None
 
     def _update_cleanup_file(self, file_path, cleanup_time):
@@ -497,7 +591,8 @@ class Scrubber(object):
             os.chmod(file_path, 0o600)
             os.utime(file_path, (cleanup_time, cleanup_time))
         except Exception:
-            LOG.error(_("%s file can not be created.") % unicode(file_path))
+            LOG.error(_LE("%s file can not be created.") %
+                      six.text_type(file_path))
 
     def _cleanup(self, pool):
         now = time.time()
@@ -511,15 +606,15 @@ class Scrubber(object):
         if cleanup_time > now:
             return
 
-        LOG.info(_("Getting images deleted before "
-                   "%s") % CONF.cleanup_scrubber_time)
+        LOG.info(_LI("Getting images deleted before %s") %
+                 CONF.cleanup_scrubber_time)
         self._update_cleanup_file(cleanup_file, now)
 
         delete_jobs = self._get_delete_jobs(self.db_queue, False)
         if not delete_jobs:
             return
 
-        for image_id, jobs in delete_jobs.iteritems():
+        for image_id, jobs in six.iteritems(delete_jobs):
             with lockutils.lock("scrubber-%s" % image_id,
                                 lock_file_prefix='glance-', external=True):
                 if not self.file_queue.has_image(image_id):
