@@ -15,7 +15,7 @@
 
 # See http://code.google.com/p/python-nose/issues/detail?id=373
 # The code below enables tests to work with i18n _() blocks
-import __builtin__
+import six.moves.builtins as __builtin__
 setattr(__builtin__, '_', lambda x: x)
 
 # Set up logging to output debugging
