@@ -13,14 +13,20 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo.config import cfg
+from oslo.db import options
+
 import glance.db
 from glance.tests import functional
 import glance.tests.functional.db as db_tests
 from glance.tests.functional.db import base
+from glance.tests.functional.db import base_metadef
+
+CONF = cfg.CONF
 
 
 def get_db(config):
-    config(group='database', connection='sqlite://')
+    options.set_defaults(CONF, connection='sqlite://')
     config(data_api='glance.db.registry.api')
     return glance.db.get_api()
 
@@ -80,3 +86,28 @@ class TestRegistryQuota(base.DriverQuotaTests, FunctionalInitWrapper):
     def tearDown(self):
         self.registry_server.stop()
         super(TestRegistryQuota, self).tearDown()
+
+
+class TestRegistryMetadefDriver(base_metadef.TestMetadefDriver,
+                                base_metadef.MetadefDriverTests,
+                                FunctionalInitWrapper):
+
+    def setUp(self):
+        db_tests.load(get_db, reset_db)
+        super(TestRegistryMetadefDriver, self).setUp()
+        self.addCleanup(db_tests.reset)
+
+    def tearDown(self):
+        self.registry_server.stop()
+        super(TestRegistryMetadefDriver, self).tearDown()
+
+
+class TestTasksDriver(base.TaskTests, FunctionalInitWrapper):
+    def setUp(self):
+        db_tests.load(get_db, reset_db)
+        super(TestTasksDriver, self).setUp()
+        self.addCleanup(db_tests.reset)
+
+    def tearDown(self):
+        self.registry_server.stop()
+        super(TestTasksDriver, self).tearDown()
