@@ -15,14 +15,15 @@ import os.path
 import tempfile
 
 import fixtures
+import glance_store
 from oslo.config import cfg
+from oslo.db import options
 
 import glance.common.client
 from glance.common import config
 from glance.db import migration
 import glance.db.sqlalchemy.api
 import glance.registry.client.v1.client
-import glance.store
 from glance import tests as glance_tests
 from glance.tests import utils as test_utils
 
@@ -111,8 +112,6 @@ paste.filter_factory = glance.tests.utils:FakeAuthMiddleware.factory
 """
 
 CONF = cfg.CONF
-CONF.import_opt('filesystem_store_datadir', 'glance.store.filesystem')
-CONF.import_opt('backend', 'glance.openstack.common.db.api', group='database')
 
 
 class ApiTest(test_utils.BaseTestCase):
@@ -123,6 +122,7 @@ class ApiTest(test_utils.BaseTestCase):
     def init(self):
         self.test_dir = self.useFixture(fixtures.TempDir()).path
         self._configure_logging()
+        self._configure_policy()
         self._setup_database()
         self._setup_stores()
         self._setup_property_protection()
@@ -145,6 +145,10 @@ class ApiTest(test_utils.BaseTestCase):
         self.property_file = os.path.join(self.test_dir,
                                           'property-protections.conf')
 
+    def _configure_policy(self):
+        policy_file = self._copy_data_file('policy.json', self.test_dir)
+        self.config(policy_file=policy_file)
+
     def _configure_logging(self):
         self.config(default_log_levels=[
             'amqplib=WARN',
@@ -157,7 +161,7 @@ class ApiTest(test_utils.BaseTestCase):
 
     def _setup_database(self):
         sql_connection = 'sqlite:////%s/tests.sqlite' % self.test_dir
-        self.config(connection=sql_connection, group='database')
+        options.set_defaults(CONF, connection=sql_connection)
         glance.db.sqlalchemy.api.clear_db_env()
         glance_db_env = 'GLANCE_DB_TEST_SQLITE_FILE'
         if glance_db_env in os.environ:
@@ -189,9 +193,13 @@ class ApiTest(test_utils.BaseTestCase):
             atexit.register(_delete_cached_db)
 
     def _setup_stores(self):
+        glance_store.register_opts(CONF)
+
         image_dir = os.path.join(self.test_dir, "images")
-        self.config(filesystem_store_datadir=image_dir)
-        glance.store.create_stores()
+        self.config(group='glance_store',
+                    filesystem_store_datadir=image_dir)
+
+        glance_store.create_stores()
 
     def _load_paste_app(self, name, flavor, conf):
         conf_file_path = os.path.join(self.test_dir, '%s-paste.ini' % name)
