@@ -76,7 +76,7 @@ class TestProtectedImageRepoProxy(utils.BaseTestCase):
         self.assertEqual(result_extra_props['spl_read_prop'], 'r')
         self.assertEqual(result_extra_props['spl_update_prop'], 'u')
         self.assertEqual(result_extra_props['spl_delete_prop'], 'd')
-        self.assertFalse('forbidden' in result_extra_props.keys())
+        self.assertNotIn('forbidden', result_extra_props.keys())
 
     def test_list_image(self):
         result_images = self.image_repo.list()
@@ -86,14 +86,14 @@ class TestProtectedImageRepoProxy(utils.BaseTestCase):
         self.assertEqual(result_extra_props['spl_read_prop'], 'r')
         self.assertEqual(result_extra_props['spl_update_prop'], 'u')
         self.assertEqual(result_extra_props['spl_delete_prop'], 'd')
-        self.assertFalse('forbidden' in result_extra_props.keys())
+        self.assertNotIn('forbidden', result_extra_props.keys())
 
         result_extra_props = result_images[1].extra_properties
         self.assertEqual(result_extra_props, {})
 
         result_extra_props = result_images[2].extra_properties
         self.assertEqual(result_extra_props['spl_read_prop'], 'r')
-        self.assertFalse('forbidden' in result_extra_props.keys())
+        self.assertNotIn('forbidden', result_extra_props.keys())
 
 
 class TestProtectedImageProxy(utils.BaseTestCase):
@@ -116,7 +116,7 @@ class TestProtectedImageProxy(utils.BaseTestCase):
             image, context, self.property_rules)
         result_extra_props = result_image.extra_properties
         self.assertEqual(result_extra_props['spl_read_prop'], 'read')
-        self.assertFalse('spl_fake_prop' in result_extra_props.keys())
+        self.assertNotIn('spl_fake_prop', result_extra_props.keys())
 
 
 class TestExtraPropertiesProxy(utils.BaseTestCase):
@@ -168,6 +168,14 @@ class TestExtraPropertiesProxy(utils.BaseTestCase):
                           extra_prop_proxy.__setitem__, 'spl_create_prop',
                           'par')
 
+    def test_update_empty_extra_property(self):
+        extra_properties = {'foo': ''}
+        context = glance.context.RequestContext(roles=['admin'])
+        extra_prop_proxy = property_protections.ExtraPropertiesProxy(
+            context, extra_properties, self.property_rules)
+        extra_prop_proxy['foo'] = 'bar'
+        self.assertEqual('bar', extra_prop_proxy['foo'])
+
     def test_create_extra_property_admin(self):
         extra_properties = {}
         context = glance.context.RequestContext(roles=['admin'])
@@ -218,6 +226,14 @@ class TestExtraPropertiesProxy(utils.BaseTestCase):
         self.assertRaises(KeyError,
                           extra_prop_proxy.__delitem__, 'spl_read_prop')
 
+    def test_delete_empty_extra_property(self):
+        extra_properties = {'foo': ''}
+        context = glance.context.RequestContext(roles=['admin'])
+        extra_prop_proxy = property_protections.ExtraPropertiesProxy(
+            context, extra_properties, self.property_rules)
+        del extra_prop_proxy['foo']
+        self.assertNotIn('foo', extra_prop_proxy)
+
 
 class TestProtectedImageFactoryProxy(utils.BaseTestCase):
     def setUp(self):
