@@ -22,18 +22,26 @@ import eventlet.patcher
 import fixtures
 import gettext
 import mock
+import routes
+import six
 import webob
 
+from glance.api.v1 import router as router_v1
+from glance.api.v2 import router as router_v2
 from glance.common import exception
 from glance.common import utils
 from glance.common import wsgi
-from glance.openstack.common import gettextutils
+from glance import i18n
+from glance.openstack.common import jsonutils
 from glance.tests import utils as test_utils
 
 
 class RequestTest(test_utils.BaseTestCase):
 
-    def _set_expected_languages(self, all_locales=[], avail_locales=None):
+    def _set_expected_languages(self, all_locales=None, avail_locales=None):
+        if all_locales is None:
+            all_locales = []
+
         # Override localedata.locale_identifiers to return some locales.
         def returns_some_locales(*args, **kwargs):
             return all_locales
@@ -53,6 +61,20 @@ class RequestTest(test_utils.BaseTestCase):
 
         self.stubs.Set(gettext, 'find', fake_gettext_find)
 
+    def test_content_range(self):
+        request = wsgi.Request.blank('/tests/123')
+        request.headers["Content-Range"] = 'bytes 10-99/*'
+        range_ = request.get_content_range()
+        self.assertEqual(range_.start, 10)
+        self.assertEqual(range_.stop, 100)  # non-inclusive
+        self.assertIsNone(range_.length)
+
+    def test_content_range_invalid(self):
+        request = wsgi.Request.blank('/tests/123')
+        request.headers["Content-Range"] = 'bytes=0-99'
+        self.assertRaises(webob.exc.HTTPBadRequest,
+                          request.get_content_range)
+
     def test_content_type_missing(self):
         request = wsgi.Request.blank('/tests/123')
         self.assertRaises(exception.InvalidContentType,
@@ -145,6 +167,62 @@ class RequestTest(test_utils.BaseTestCase):
         request.headers.pop('Accept-Language')
         self.assertIsNone(request.best_match_language())
 
+    def test_http_error_response_codes(self):
+        sample_id, member_id, tag_val, task_id = 'abc', '123', '1', '2'
+
+        """Makes sure v1 unallowed methods return 405"""
+        unallowed_methods = [
+            ('/images', ['PUT', 'DELETE', 'HEAD', 'PATCH']),
+            ('/images/detail', ['POST', 'PUT', 'DELETE', 'PATCH']),
+            ('/images/%s' % sample_id, ['POST', 'PATCH']),
+            ('/images/%s/members' % sample_id,
+                ['POST', 'DELETE', 'HEAD', 'PATCH']),
+            ('/images/%s/members/%s' % (sample_id, member_id),
+                ['POST', 'HEAD', 'PATCH']),
+        ]
+        api = test_utils.FakeAuthMiddleware(router_v1.API(routes.Mapper()))
+        for uri, methods in unallowed_methods:
+            for method in methods:
+                req = webob.Request.blank(uri)
+                req.method = method
+                res = req.get_response(api)
+                self.assertEqual(405, res.status_int)
+
+        """Makes sure v2 unallowed methods return 405"""
+        unallowed_methods = [
+            ('/schemas/image', ['POST', 'PUT', 'DELETE', 'PATCH', 'HEAD']),
+            ('/schemas/images', ['POST', 'PUT', 'DELETE', 'PATCH', 'HEAD']),
+            ('/schemas/member', ['POST', 'PUT', 'DELETE', 'PATCH', 'HEAD']),
+            ('/schemas/members', ['POST', 'PUT', 'DELETE', 'PATCH', 'HEAD']),
+            ('/schemas/task', ['POST', 'PUT', 'DELETE', 'PATCH', 'HEAD']),
+            ('/schemas/tasks', ['POST', 'PUT', 'DELETE', 'PATCH', 'HEAD']),
+            ('/images', ['PUT', 'DELETE', 'PATCH', 'HEAD']),
+            ('/images/%s' % sample_id, ['POST', 'PUT', 'HEAD']),
+            ('/images/%s/file' % sample_id,
+                ['POST', 'DELETE', 'PATCH', 'HEAD']),
+            ('/images/%s/tags/%s' % (sample_id, tag_val),
+                ['GET', 'POST', 'PATCH', 'HEAD']),
+            ('/images/%s/members' % sample_id,
+                ['PUT', 'DELETE', 'PATCH', 'HEAD']),
+            ('/images/%s/members/%s' % (sample_id, member_id),
+                ['POST', 'PATCH', 'HEAD']),
+            ('/tasks', ['PUT', 'DELETE', 'PATCH', 'HEAD']),
+            ('/tasks/%s' % task_id, ['POST', 'PUT', 'PATCH', 'HEAD']),
+        ]
+        api = test_utils.FakeAuthMiddleware(router_v2.API(routes.Mapper()))
+        for uri, methods in unallowed_methods:
+            for method in methods:
+                req = webob.Request.blank(uri)
+                req.method = method
+                res = req.get_response(api)
+                self.assertEqual(405, res.status_int)
+
+        """Makes sure not implemented methods return 501"""
+        req = webob.Request.blank('/schemas/image')
+        req.method = 'NonexistentMethod'
+        res = req.get_response(api)
+        self.assertEqual(501, res.status_int)
+
 
 class ResourceTest(test_utils.BaseTestCase):
 
@@ -262,7 +340,7 @@ class ResourceTest(test_utils.BaseTestCase):
         self.assertEqual(message_es, str(e))
 
     @mock.patch.object(webob.acceptparse.AcceptLanguage, 'best_match')
-    @mock.patch.object(gettextutils, 'translate')
+    @mock.patch.object(i18n, 'translate')
     def test_translate_exception(self, mock_translate, mock_best_match):
 
         mock_translate.return_value = 'No Encontrado'
@@ -292,9 +370,11 @@ class JSONResponseSerializerTest(test_utils.BaseTestCase):
 
     def test_to_json_with_more_deep_format(self):
         fixture = {"is_public": True, "name": [{"name1": "test"}]}
-        expected = '{"is_public": true, "name": [{"name1": "test"}]}'
+        expected = {"is_public": True, "name": [{"name1": "test"}]}
         actual = wsgi.JSONResponseSerializer().to_json(fixture)
-        self.assertEqual(actual, expected)
+        actual = jsonutils.loads(actual)
+        for k in expected:
+            self.assertEqual(expected[k], actual[k])
 
     def test_to_json_with_set(self):
         fixture = set(["foo"])
@@ -334,7 +414,7 @@ class JSONRequestDeserializerTest(test_utils.BaseTestCase):
         request = wsgi.Request.blank('/')
         request.method = 'POST'
         request.body = 'asdf'
-        self.assertTrue('Content-Length' in request.headers)
+        self.assertIn('Content-Length', request.headers)
         self.assertTrue(wsgi.JSONRequestDeserializer().has_body(request))
 
     def test_no_body_no_content_length(self):
@@ -371,7 +451,7 @@ class JSONRequestDeserializerTest(test_utils.BaseTestCase):
         request.method = 'POST'
         request.body = 'fake_body'
         request.headers['transfer-encoding'] = 0
-        self.assertTrue('transfer-encoding' in request.headers)
+        self.assertIn('transfer-encoding', request.headers)
         self.assertTrue(wsgi.JSONRequestDeserializer().has_body(request))
 
     def test_get_bind_addr_default_value(self):
@@ -404,7 +484,7 @@ class TestHelpers(test_utils.BaseTestCase):
                    'location': "file:///tmp/glance-tests/2",
                    'properties': {'distro': 'Ubuntu 10.04 LTS'}}
         headers = utils.image_meta_to_http_headers(fixture)
-        for k, v in headers.iteritems():
+        for k, v in six.iteritems(headers):
             self.assertIsInstance(v, unicode)
 
     def test_data_passed_properly_through_headers(self):
@@ -426,11 +506,11 @@ class TestHelpers(test_utils.BaseTestCase):
         response = FakeResponse()
         response.headers = headers
         result = utils.get_image_meta_from_headers(response)
-        for k, v in fixture.iteritems():
+        for k, v in six.iteritems(fixture):
             if v is not None:
                 self.assertEqual(v, result[k])
             else:
-                self.assertFalse(k in result)
+                self.assertNotIn(k, result)
 
 
 class GetSocketTestCase(test_utils.BaseTestCase):
@@ -466,19 +546,19 @@ class GetSocketTestCase(test_utils.BaseTestCase):
             'glance.common.wsgi.eventlet.listen',
             lambda *x, **y: None))
         wsgi.get_socket(1234)
-        self.assertTrue(mock.call().setsockopt(
+        self.assertIn(mock.call().setsockopt(
             socket.SOL_SOCKET,
             socket.SO_REUSEADDR,
-            1) in mock_socket.mock_calls)
-        self.assertTrue(mock.call().setsockopt(
+            1), mock_socket.mock_calls)
+        self.assertIn(mock.call().setsockopt(
             socket.SOL_SOCKET,
             socket.SO_KEEPALIVE,
-            1) in mock_socket.mock_calls)
+            1), mock_socket.mock_calls)
         if hasattr(socket, 'TCP_KEEPIDLE'):
-            self.assertTrue(mock.call().setsockopt(
+            self.assertIn(mock.call().setsockopt(
                 socket.IPPROTO_TCP,
                 socket.TCP_KEEPIDLE,
-                wsgi.CONF.tcp_keepidle) in mock_socket.mock_calls)
+                wsgi.CONF.tcp_keepidle), mock_socket.mock_calls)
 
     def test_get_socket_without_all_ssl_reqs(self):
         wsgi.CONF.key_file = None
