@@ -18,12 +18,14 @@ import uuid
 
 import mock
 from oslo.config import cfg
+from oslo.db import exception as db_exc
 
 from glance.common import crypt
 from glance.common import exception
+from glance.common import utils
 import glance.context
 import glance.db
-from glance.openstack.common.db import exception as db_exception
+from glance.db.sqlalchemy import api
 import glance.tests.unit.utils as unit_test_utils
 import glance.tests.utils as test_utils
 
@@ -140,13 +142,15 @@ class TestImageRepo(test_utils.BaseTestCase):
                         name='1', size=256,
                         is_public=True, status='active',
                         locations=[{'url': UUID1_LOCATION,
-                                    'metadata': UUID1_LOCATION_METADATA}]),
+                                    'metadata': UUID1_LOCATION_METADATA,
+                                    'status': 'active'}]),
             _db_fixture(UUID2, owner=TENANT1, checksum=CHCKSUM1,
                         name='2', size=512, is_public=False),
             _db_fixture(UUID3, owner=TENANT3, checksum=CHCKSUM1,
                         name='3', size=1024, is_public=True,
                         locations=[{'url': UUID3_LOCATION,
-                                    'metadata': {}}]),
+                                    'metadata': {},
+                                    'status': 'active'}]),
             _db_fixture(UUID4, owner=TENANT4, name='4', size=2048),
         ]
         [self.db.image_create(None, image) for image in self.images]
@@ -189,7 +193,7 @@ class TestImageRepo(test_utils.BaseTestCase):
         fake_uuid = str(uuid.uuid4())
         exc = self.assertRaises(exception.NotFound, self.image_repo.get,
                                 fake_uuid)
-        self.assertTrue(fake_uuid in unicode(exc))
+        self.assertIn(fake_uuid, utils.exception_to_str(exc))
 
     def test_get_forbidden(self):
         self.assertRaises(exception.NotFound, self.image_repo.get, UUID4)
@@ -330,7 +334,7 @@ class TestImageRepo(test_utils.BaseTestCase):
         image.image_id = fake_uuid
         exc = self.assertRaises(exception.NotFound, self.image_repo.save,
                                 image)
-        self.assertTrue(fake_uuid in unicode(exc))
+        self.assertIn(fake_uuid, utils.exception_to_str(exc))
 
     def test_remove_image(self):
         image = self.image_repo.get(UUID1)
@@ -345,7 +349,7 @@ class TestImageRepo(test_utils.BaseTestCase):
         image.image_id = fake_uuid
         exc = self.assertRaises(exception.NotFound, self.image_repo.remove,
                                 image)
-        self.assertTrue(fake_uuid in unicode(exc))
+        self.assertIn(fake_uuid, utils.exception_to_str(exc))
 
 
 class TestEncryptedLocations(test_utils.BaseTestCase):
@@ -359,8 +363,10 @@ class TestEncryptedLocations(test_utils.BaseTestCase):
         self.image_factory = glance.domain.ImageFactory()
         self.crypt_key = '0123456789abcdef'
         self.config(metadata_encryption_key=self.crypt_key)
-        self.foo_bar_location = [{'url': 'foo', 'metadata': {}},
-                                 {'url': 'bar', 'metadata': {}}]
+        self.foo_bar_location = [{'url': 'foo', 'metadata': {},
+                                  'status': 'active'},
+                                 {'url': 'bar', 'metadata': {},
+                                  'status': 'active'}]
 
     def test_encrypt_locations_on_add(self):
         image = self.image_factory.new_image(UUID1)
@@ -387,30 +393,40 @@ class TestEncryptedLocations(test_utils.BaseTestCase):
 
     def test_decrypt_locations_on_get(self):
         url_loc = ['ping', 'pong']
-        orig_locations = [{'url': l, 'metadata': {}} for l in url_loc]
+        orig_locations = [{'url': l, 'metadata': {}, 'status': 'active'}
+                          for l in url_loc]
         encrypted_locs = [crypt.urlsafe_encrypt(self.crypt_key, l)
                           for l in url_loc]
-        encrypted_locations = [{'url': l, 'metadata': {}}
+        encrypted_locations = [{'url': l, 'metadata': {}, 'status': 'active'}
                                for l in encrypted_locs]
         self.assertNotEqual(encrypted_locations, orig_locations)
         db_data = _db_fixture(UUID1, owner=TENANT1,
                               locations=encrypted_locations)
         self.db.image_create(None, db_data)
         image = self.image_repo.get(UUID1)
+        self.assertIn('id', image.locations[0])
+        self.assertIn('id', image.locations[1])
+        image.locations[0].pop('id')
+        image.locations[1].pop('id')
         self.assertEqual(image.locations, orig_locations)
 
     def test_decrypt_locations_on_list(self):
         url_loc = ['ping', 'pong']
-        orig_locations = [{'url': l, 'metadata': {}} for l in url_loc]
+        orig_locations = [{'url': l, 'metadata': {}, 'status': 'active'}
+                          for l in url_loc]
         encrypted_locs = [crypt.urlsafe_encrypt(self.crypt_key, l)
                           for l in url_loc]
-        encrypted_locations = [{'url': l, 'metadata': {}}
+        encrypted_locations = [{'url': l, 'metadata': {}, 'status': 'active'}
                                for l in encrypted_locs]
         self.assertNotEqual(encrypted_locations, orig_locations)
         db_data = _db_fixture(UUID1, owner=TENANT1,
                               locations=encrypted_locations)
         self.db.image_create(None, db_data)
         image = self.image_repo.list()[0]
+        self.assertIn('id', image.locations[0])
+        self.assertIn('id', image.locations[1])
+        image.locations[0].pop('id')
+        image.locations[1].pop('id')
         self.assertEqual(image.locations, orig_locations)
 
 
@@ -474,7 +490,7 @@ class TestImageMemberRepo(test_utils.BaseTestCase):
         image = self.image_repo.get(UUID1)
         image_member = self.image_member_factory.new_image_member(image,
                                                                   TENANT4)
-        self.assertTrue(image_member.id is None)
+        self.assertIsNone(image_member.id)
         self.image_member_repo.add(image_member)
         retreived_image_member = self.image_member_repo.get(TENANT4)
         self.assertIsNotNone(retreived_image_member.id)
@@ -489,7 +505,7 @@ class TestImageMemberRepo(test_utils.BaseTestCase):
         image = self.image_repo.get(UUID1)
         image_member = self.image_member_factory.new_image_member(image,
                                                                   TENANT4)
-        self.assertTrue(image_member.id is None)
+        self.assertIsNone(image_member.id)
         self.image_member_repo.add(image_member)
         retreived_image_member = self.image_member_repo.get(TENANT4)
         self.assertIsNotNone(retreived_image_member.id)
@@ -537,7 +553,7 @@ class TestImageMemberRepo(test_utils.BaseTestCase):
         exc = self.assertRaises(exception.NotFound,
                                 self.image_member_repo.remove,
                                 fake_member)
-        self.assertTrue(fake_uuid in unicode(exc))
+        self.assertIn(fake_uuid, utils.exception_to_str(exc))
 
 
 class TestTaskRepo(test_utils.BaseTestCase):
@@ -586,99 +602,94 @@ class TestTaskRepo(test_utils.BaseTestCase):
         [self.db.task_create(None, task) for task in self.tasks]
 
     def test_get(self):
-        task, task_details = self.task_repo.get_task_and_details(UUID1)
+        task = self.task_repo.get(UUID1)
         self.assertEqual(task.task_id, UUID1)
         self.assertEqual(task.type, 'import')
         self.assertEqual(task.status, 'pending')
-        self.assertEqual(task.task_id, task_details.task_id)
-        self.assertEqual(task_details.input, self.fake_task_input)
-        self.assertEqual(task_details.result, '')
+        self.assertEqual(task.task_input, self.fake_task_input)
+        self.assertEqual(task.result, '')
+        self.assertEqual(task.message, '')
         self.assertEqual(task.owner, TENANT1)
 
     def test_get_not_found(self):
         self.assertRaises(exception.NotFound,
-                          self.task_repo.get_task_and_details,
+                          self.task_repo.get,
                           str(uuid.uuid4()))
 
     def test_get_forbidden(self):
         self.assertRaises(exception.NotFound,
-                          self.task_repo.get_task_and_details,
+                          self.task_repo.get,
                           UUID4)
 
     def test_list(self):
-        tasks = self.task_repo.list_tasks()
+        tasks = self.task_repo.list()
         task_ids = set([i.task_id for i in tasks])
         self.assertEqual(set([UUID1, UUID2, UUID3]), task_ids)
 
     def test_list_with_type(self):
         filters = {'type': 'import'}
-        tasks = self.task_repo.list_tasks(filters=filters)
+        tasks = self.task_repo.list(filters=filters)
         task_ids = set([i.task_id for i in tasks])
         self.assertEqual(set([UUID1, UUID2, UUID3]), task_ids)
 
     def test_list_with_status(self):
         filters = {'status': 'failure'}
-        tasks = self.task_repo.list_tasks(filters=filters)
+        tasks = self.task_repo.list(filters=filters)
         task_ids = set([i.task_id for i in tasks])
         self.assertEqual(set([UUID3]), task_ids)
 
     def test_list_with_marker(self):
-        full_tasks = self.task_repo.list_tasks()
+        full_tasks = self.task_repo.list()
         full_ids = [i.task_id for i in full_tasks]
-        marked_tasks = self.task_repo.list_tasks(marker=full_ids[0])
+        marked_tasks = self.task_repo.list(marker=full_ids[0])
         actual_ids = [i.task_id for i in marked_tasks]
         self.assertEqual(actual_ids, full_ids[1:])
 
     def test_list_with_last_marker(self):
-        tasks = self.task_repo.list_tasks()
-        marked_tasks = self.task_repo.list_tasks(marker=tasks[-1].task_id)
+        tasks = self.task_repo.list()
+        marked_tasks = self.task_repo.list(marker=tasks[-1].task_id)
         self.assertEqual(len(marked_tasks), 0)
 
     def test_limited_list(self):
-        limited_tasks = self.task_repo.list_tasks(limit=2)
+        limited_tasks = self.task_repo.list(limit=2)
         self.assertEqual(len(limited_tasks), 2)
 
     def test_list_with_marker_and_limit(self):
-        full_tasks = self.task_repo.list_tasks()
+        full_tasks = self.task_repo.list()
         full_ids = [i.task_id for i in full_tasks]
-        marked_tasks = self.task_repo.list_tasks(marker=full_ids[0], limit=1)
+        marked_tasks = self.task_repo.list(marker=full_ids[0], limit=1)
         actual_ids = [i.task_id for i in marked_tasks]
         self.assertEqual(actual_ids, full_ids[1:2])
 
     def test_sorted_list(self):
-        tasks = self.task_repo.list_tasks(sort_key='status', sort_dir='desc')
+        tasks = self.task_repo.list(sort_key='status', sort_dir='desc')
         task_ids = [i.task_id for i in tasks]
         self.assertEqual([UUID2, UUID1, UUID3], task_ids)
 
     def test_add_task(self):
         task_type = 'import'
-        task = self.task_factory.new_task(task_type, None)
+        task = self.task_factory.new_task(task_type, None,
+                                          task_input=self.fake_task_input)
         self.assertEqual(task.updated_at, task.created_at)
-        task_details = self.task_factory.new_task_details(task.task_id,
-                                                          self.fake_task_input)
-
-        self.task_repo.add(task, task_details)
-        retrieved_task, retrieved_task_details = \
-            self.task_repo.get_task_and_details(task.task_id)
-        self.assertEqual(retrieved_task.updated_at, task.updated_at)
-        self.assertEqual(retrieved_task_details.task_id,
-                         retrieved_task.task_id)
-        self.assertEqual(retrieved_task_details.input, task_details.input)
+        self.task_repo.add(task)
+        retrieved_task = self.task_repo.get(task.task_id)
+        self.assertEqual(task.updated_at, retrieved_task.updated_at)
+        self.assertEqual(self.fake_task_input, retrieved_task.task_input)
 
     def test_save_task(self):
-        task, task_details = self.task_repo.get_task_and_details(UUID1)
+        task = self.task_repo.get(UUID1)
         original_update_time = task.updated_at
         self.task_repo.save(task)
         current_update_time = task.updated_at
         self.assertTrue(current_update_time > original_update_time)
-        task, task_details = self.task_repo.get_task_and_details(UUID1)
+        task = self.task_repo.get(UUID1)
         self.assertEqual(task.updated_at, current_update_time)
 
     def test_remove_task(self):
-        task, task_details = self.task_repo.get_task_and_details(UUID1)
+        task = self.task_repo.get(UUID1)
         self.task_repo.remove(task)
         self.assertRaises(exception.NotFound,
-                          self.task_repo.get_task_and_details,
+                          self.task_repo.get,
                           task.task_id)
 
 
@@ -696,15 +707,13 @@ class RetryOnDeadlockTestCase(test_utils.BaseTestCase):
                 self.attempts -= 1
                 if self.attempts <= 0:
                     raise TestException("Exit")
-                raise db_exception.DBDeadlock("Fake Exception")
+                raise db_exc.DBDeadlock("Fake Exception")
             return _raise_exceptions
 
-        # Test retry on image destroy if db deadlock occurs
-        with mock.patch.object(glance.db.sqlalchemy.api,
-                               'get_session') as sess:
+        with mock.patch.object(api, 'get_session') as sess:
             sess.side_effect = _mock_get_session()
 
             try:
-                glance.db.sqlalchemy.api.image_destroy(None, 'fake-id')
+                api._image_update(None, {}, 'fake-id')
             except TestException:
                 self.assertEqual(sess.call_count, 3)
