@@ -74,7 +74,7 @@ class TestProxyRepoPlain(test_utils.BaseTestCase):
         self._test_method('add', 'snuff', 'enough')
 
     def test_save(self):
-        self._test_method('save', 'snuff', 'enough', from_state=None)
+        self._test_method('save', 'snuff', 'enough')
 
     def test_remove(self):
         self._test_method('add', None, 'flying')
@@ -121,17 +121,17 @@ class TestProxyRepoWrapping(test_utils.BaseTestCase):
             self.assertEqual(results[i].args, tuple())
             self.assertEqual(results[i].kwargs, {'a': 1})
 
-    def _test_method_with_proxied_argument(self, name, result, **kwargs):
+    def _test_method_with_proxied_argument(self, name, result):
         self.fake_repo.result = result
         item = FakeProxy('snoop')
         method = getattr(self.proxy_repo, name)
         proxy_result = method(item)
 
-        self.assertEqual(('snoop',), self.fake_repo.args)
-        self.assertEqual(kwargs, self.fake_repo.kwargs)
+        self.assertEqual(self.fake_repo.args, ('snoop',))
+        self.assertEqual(self.fake_repo.kwargs, {})
 
         if result is None:
-            self.assertTrue(proxy_result is None)
+            self.assertIsNone(proxy_result)
         else:
             self.assertIsInstance(proxy_result, FakeProxy)
             self.assertEqual(proxy_result.base, result)
@@ -145,12 +145,10 @@ class TestProxyRepoWrapping(test_utils.BaseTestCase):
         self._test_method_with_proxied_argument('add', None)
 
     def test_save(self):
-        self._test_method_with_proxied_argument('save', 'dog',
-                                                from_state=None)
+        self._test_method_with_proxied_argument('save', 'dog')
 
     def test_save_with_no_result(self):
-        self._test_method_with_proxied_argument('save', None,
-                                                from_state=None)
+        self._test_method_with_proxied_argument('save', None)
 
     def test_remove(self):
         self._test_method_with_proxied_argument('remove', 'dog')
@@ -307,24 +305,13 @@ class TestTaskFactory(test_utils.BaseTestCase):
             owner=self.fake_owner
         )
 
-        proxy_factory.new_task_details("task_01", "input")
-
-        self.factory.new_task_details.assert_called_once_with(
-            "task_01",
-            "input",
-            None, None
-        )
-
     def test_proxy_wrapping(self):
         proxy_factory = proxy.TaskFactory(
             self.factory,
             task_proxy_class=FakeProxy,
-            task_proxy_kwargs={'dog': 'bark'},
-            task_details_proxy_class=FakeProxy,
-            task_details_proxy_kwargs={'dog': 'bark'})
+            task_proxy_kwargs={'dog': 'bark'})
 
         self.factory.new_task.return_value = 'fake_task'
-        self.factory.new_task_details.return_value = 'fake_task_detail'
 
         task = proxy_factory.new_task(
             type=self.fake_type,
@@ -337,14 +324,3 @@ class TestTaskFactory(test_utils.BaseTestCase):
         )
         self.assertIsInstance(task, FakeProxy)
         self.assertEqual(task.base, 'fake_task')
-
-        task_details = proxy_factory.new_task_details('task_01', "input")
-
-        self.factory.new_task_details.assert_called_once_with(
-            'task_01',
-            "input",
-            None, None
-        )
-
-        self.assertIsInstance(task_details, FakeProxy)
-        self.assertEqual(task_details.base, 'fake_task_detail')
