@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2011 OpenStack, LLC
+# Copyright 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -19,21 +17,21 @@ from contextlib import contextmanager
 import datetime
 import hashlib
 import os
-import StringIO
-import tempfile
 import time
 
 import fixtures
+import six
+from six.moves import xrange
 import stubout
 
 from glance.common import exception
 from glance import image_cache
+from glance.openstack.common import units
 #NOTE(bcwaldon): This is imported to load the registry config options
-import glance.registry
-import glance.store.filesystem as fs_store
-import glance.store.s3 as s3_store
+import glance.registry  # noqa
 from glance.tests import utils as test_utils
-from glance.tests.utils import skip_if_disabled, xattr_writes_supported
+from glance.tests.utils import skip_if_disabled
+from glance.tests.utils import xattr_writes_supported
 
 FIXTURE_LENGTH = 1024
 FIXTURE_DATA = '*' * FIXTURE_LENGTH
@@ -42,7 +40,7 @@ FIXTURE_DATA = '*' * FIXTURE_LENGTH
 class ImageCacheTestCase(object):
 
     def _setup_fixture_file(self):
-        FIXTURE_FILE = StringIO.StringIO(FIXTURE_DATA)
+        FIXTURE_FILE = six.StringIO(FIXTURE_DATA)
 
         self.assertFalse(self.cache.is_cached(1))
 
@@ -52,22 +50,20 @@ class ImageCacheTestCase(object):
 
     @skip_if_disabled
     def test_is_cached(self):
-        """
-        Verify is_cached(1) returns 0, then add something to the cache
+        """Verify is_cached(1) returns 0, then add something to the cache
         and verify is_cached(1) returns 1.
         """
         self._setup_fixture_file()
 
     @skip_if_disabled
     def test_read(self):
-        """
-        Verify is_cached(1) returns 0, then add something to the cache
+        """Verify is_cached(1) returns 0, then add something to the cache
         and verify after a subsequent read from the cache that
         is_cached(1) returns 1.
         """
         self._setup_fixture_file()
 
-        buff = StringIO.StringIO()
+        buff = six.StringIO()
         with self.cache.open_for_read(1) as cache_file:
             for chunk in cache_file:
                 buff.write(chunk)
@@ -76,13 +72,12 @@ class ImageCacheTestCase(object):
 
     @skip_if_disabled
     def test_open_for_read(self):
-        """
-        Test convenience wrapper for opening a cache file via
+        """Test convenience wrapper for opening a cache file via
         its image identifier.
         """
         self._setup_fixture_file()
 
-        buff = StringIO.StringIO()
+        buff = six.StringIO()
         with self.cache.open_for_read(1) as cache_file:
             for chunk in cache_file:
                 buff.write(chunk)
@@ -91,8 +86,7 @@ class ImageCacheTestCase(object):
 
     @skip_if_disabled
     def test_get_image_size(self):
-        """
-        Test convenience wrapper for querying cache file size via
+        """Test convenience wrapper for querying cache file size via
         its image identifier.
         """
         self._setup_fixture_file()
@@ -103,9 +97,7 @@ class ImageCacheTestCase(object):
 
     @skip_if_disabled
     def test_delete(self):
-        """
-        Test delete method that removes an image from the cache
-        """
+        """Test delete method that removes an image from the cache."""
         self._setup_fixture_file()
 
         self.cache.delete_cached_image(1)
@@ -114,14 +106,12 @@ class ImageCacheTestCase(object):
 
     @skip_if_disabled
     def test_delete_all(self):
-        """
-        Test delete method that removes an image from the cache
-        """
+        """Test delete method that removes an image from the cache."""
         for image_id in (1, 2):
             self.assertFalse(self.cache.is_cached(image_id))
 
         for image_id in (1, 2):
-            FIXTURE_FILE = StringIO.StringIO(FIXTURE_DATA)
+            FIXTURE_FILE = six.StringIO(FIXTURE_DATA)
             self.assertTrue(self.cache.cache_image_file(image_id,
                                                         FIXTURE_FILE))
 
@@ -135,9 +125,7 @@ class ImageCacheTestCase(object):
 
     @skip_if_disabled
     def test_clean_stalled(self):
-        """
-        Test the clean method removes expected images
-        """
+        """Test the clean method removes expected images."""
         incomplete_file_path = os.path.join(self.cache_dir, 'incomplete', '1')
         incomplete_file = open(incomplete_file_path, 'w')
         incomplete_file.write(FIXTURE_DATA)
@@ -191,23 +179,23 @@ class ImageCacheTestCase(object):
         # prune it. We should see only 5 images left after
         # pruning, and the images that are least recently accessed
         # should be the ones pruned...
-        for x in xrange(0, 10):
-            FIXTURE_FILE = StringIO.StringIO(FIXTURE_DATA)
+        for x in xrange(10):
+            FIXTURE_FILE = six.StringIO(FIXTURE_DATA)
             self.assertTrue(self.cache.cache_image_file(x,
                                                         FIXTURE_FILE))
 
-        self.assertEqual(10 * 1024, self.cache.get_cache_size())
+        self.assertEqual(10 * units.Ki, self.cache.get_cache_size())
 
         # OK, hit the images that are now cached...
-        for x in xrange(0, 10):
-            buff = StringIO.StringIO()
+        for x in xrange(10):
+            buff = six.StringIO()
             with self.cache.open_for_read(x) as cache_file:
                 for chunk in cache_file:
                     buff.write(chunk)
 
         self.cache.prune()
 
-        self.assertEqual(5 * 1024, self.cache.get_cache_size())
+        self.assertEqual(5 * units.Ki, self.cache.get_cache_size())
 
         for x in xrange(0, 5):
             self.assertFalse(self.cache.is_cached(x),
@@ -225,13 +213,13 @@ class ImageCacheTestCase(object):
         """
         self.assertEqual(0, self.cache.get_cache_size())
 
-        FIXTURE_FILE = StringIO.StringIO(FIXTURE_DATA)
+        FIXTURE_FILE = six.StringIO(FIXTURE_DATA)
         self.assertTrue(self.cache.cache_image_file('xxx', FIXTURE_FILE))
 
         self.assertEqual(1024, self.cache.get_cache_size())
 
         # OK, hit the image that is now cached...
-        buff = StringIO.StringIO()
+        buff = six.StringIO()
         with self.cache.open_for_read('xxx') as cache_file:
             for chunk in cache_file:
                 buff.write(chunk)
@@ -251,7 +239,7 @@ class ImageCacheTestCase(object):
         self.assertFalse(self.cache.is_cached(1))
         self.assertFalse(self.cache.is_queued(1))
 
-        FIXTURE_FILE = StringIO.StringIO(FIXTURE_DATA)
+        FIXTURE_FILE = six.StringIO(FIXTURE_DATA)
 
         self.assertTrue(self.cache.queue_image(1))
 
@@ -276,7 +264,7 @@ class ImageCacheTestCase(object):
 
         self.cache.delete_cached_image(1)
 
-        for x in xrange(0, 3):
+        for x in xrange(3):
             self.assertTrue(self.cache.queue_image(x))
 
         self.assertEqual(self.cache.get_queued_images(),
@@ -312,10 +300,10 @@ class ImageCacheTestCase(object):
         image_id = '1'
         self.assertFalse(self.cache.is_cached(image_id))
         try:
-            with self.cache.driver.open_for_write(image_id) as cache_file:
+            with self.cache.driver.open_for_write(image_id):
                 raise IOError
         except Exception as e:
-            self.assertEqual(type(e), IOError)
+            self.assertIsInstance(e, IOError)
         self.assertFalse(self.cache.is_cached(image_id),
                          "Image %s was cached!" % image_id)
         # make sure it has tidied up
@@ -367,7 +355,7 @@ class ImageCacheTestCase(object):
         def consume(image_id):
             caching_iter = self.cache.get_caching_iter(image_id, None,
                                                        faulty_backend())
-            # excercise the caching_iter
+            # exercise the caching_iter
             list(caching_iter)
 
         image_id = '1'
@@ -418,44 +406,6 @@ class ImageCacheTestCase(object):
         # checksum is valid, fake image should be cached:
         self.assertTrue(cache.is_cached(image_id))
 
-    def test_gate_caching_iter_fs_chunked_file(self):
-        """Tests get_caching_iter when using a filesystem ChunkedFile"""
-        image_id = 123
-
-        with tempfile.NamedTemporaryFile() as test_data_file:
-            test_data_file.write(FIXTURE_DATA)
-            test_data_file.seek(0)
-            image = fs_store.ChunkedFile(test_data_file.name)
-            md5 = hashlib.md5()
-            md5.update(FIXTURE_DATA)
-            checksum = md5.hexdigest()
-
-            cache = image_cache.ImageCache()
-            img_iter = cache.get_caching_iter(image_id, checksum, image)
-            for chunk in img_iter:
-                pass
-            # checksum is valid, fake image should be cached:
-            self.assertTrue(cache.is_cached(image_id))
-
-    def test_gate_caching_iter_s3_chunked_file(self):
-        """Tests get_caching_iter when using an S3 ChunkedFile"""
-        image_id = 123
-
-        with tempfile.NamedTemporaryFile() as test_data_file:
-            test_data_file.write(FIXTURE_DATA)
-            test_data_file.seek(0)
-            image = s3_store.ChunkedFile(test_data_file)
-            md5 = hashlib.md5()
-            md5.update(FIXTURE_DATA)
-            checksum = md5.hexdigest()
-
-            cache = image_cache.ImageCache()
-            img_iter = cache.get_caching_iter(image_id, checksum, image)
-            for chunk in img_iter:
-                pass
-            # checksum is valid, fake image should be cached:
-            self.assertTrue(cache.is_cached(image_id))
-
     def test_gate_caching_iter_bad_checksum(self):
         image = "12345678990abcdefghijklmnop"
         image_id = 123
@@ -492,7 +442,7 @@ class TestImageCacheXattr(test_utils.BaseTestCase,
 
         if not getattr(self, 'inited', False):
             try:
-                import xattr
+                import xattr  # noqa
             except ImportError:
                 self.inited = True
                 self.disabled = True
@@ -503,7 +453,7 @@ class TestImageCacheXattr(test_utils.BaseTestCase,
         self.disabled = False
         self.config(image_cache_dir=self.cache_dir,
                     image_cache_driver='xattr',
-                    image_cache_max_size=1024 * 5)
+                    image_cache_max_size=5 * units.Ki)
         self.cache = image_cache.ImageCache()
 
         if not xattr_writes_supported(self.cache_dir):
@@ -530,7 +480,7 @@ class TestImageCacheSqlite(test_utils.BaseTestCase,
 
         if not getattr(self, 'inited', False):
             try:
-                import sqlite3
+                import sqlite3  # noqa
             except ImportError:
                 self.inited = True
                 self.disabled = True
@@ -542,7 +492,7 @@ class TestImageCacheSqlite(test_utils.BaseTestCase,
         self.cache_dir = self.useFixture(fixtures.TempDir()).path
         self.config(image_cache_dir=self.cache_dir,
                     image_cache_driver='sqlite',
-                    image_cache_max_size=1024 * 5)
+                    image_cache_max_size=5 * units.Ki)
         self.cache = image_cache.ImageCache()
 
 
