@@ -16,12 +16,15 @@
 
 import datetime
 
+import glance_store
 import mock
+from oslo.config import cfg
+from oslo import messaging
 import webob
 
+import glance.async
 from glance.common import exception
 import glance.context
-from glance import domain
 from glance import notifier
 from glance.openstack.common import timeutils
 import glance.tests.unit.utils as unit_test_utils
@@ -38,7 +41,7 @@ TENANT2 = '2c014f32-55eb-467d-8fcb-4bd706012f81'
 
 
 class ImageStub(glance.domain.Image):
-    def get_data(self):
+    def get_data(self, offset=0, chunk_size=None):
         return ['01234', '56789']
 
     def set_data(self, data, size=None):
@@ -63,10 +66,12 @@ class ImageRepoStub(object):
         return ['images_from_list']
 
 
-class TaskStub(glance.domain.Task):
+class TaskStub(glance.domain.TaskStub):
     def run(self, executor):
         pass
 
+
+class Task(glance.domain.Task):
     def succeed(self, result):
         pass
 
@@ -84,7 +89,7 @@ class TaskRepoStub(object):
     def add(self, *args, **kwargs):
         return 'task_from_add'
 
-    def get(self, *args, **kwargs):
+    def get_task(self, *args, **kwargs):
         return 'task_from_get'
 
     def list(self, *args, **kwargs):
@@ -93,42 +98,21 @@ class TaskRepoStub(object):
 
 class TestNotifier(utils.BaseTestCase):
 
-    def test_load_rabbit(self):
-        nfier = notifier.Notifier('rabbit')
-        self.assertIsNotNone(nfier._transport)
-
-    def test_load_qpid(self):
-        nfier = notifier.Notifier('qpid')
-        self.assertIsNotNone(nfier._transport)
-        self.assertEqual(str(nfier._transport._driver._url),
-                         'qpid:///')
-
-    def test_notifier_strategy(self):
-        self.config(notifier_strategy='qpid')
+    @mock.patch.object(messaging, 'Notifier')
+    @mock.patch.object(messaging, 'get_transport')
+    def _test_load_strategy(self,
+                            mock_get_transport, mock_notifier,
+                            url, driver):
         nfier = notifier.Notifier()
+        mock_get_transport.assert_called_with(cfg.CONF,
+                                              aliases=notifier._ALIASES)
         self.assertIsNotNone(nfier._transport)
-        self.assertEqual(str(nfier._transport._driver._url),
-                         'qpid:///')
+        mock_notifier.assert_called_with(nfier._transport,
+                                         publisher_id='image.localhost')
+        self.assertIsNotNone(nfier._notifier)
 
-    def test_transport_url(self):
-        transport_url = "qpid://superhost:5672/"
-        self.config(transport_url=transport_url)
-        notify = notifier.Notifier()
-        self.assertEqual(str(notify._transport._driver._url),
-                         transport_url)
-
-    def test_notification_driver_option(self):
-        self.config(rpc_backend='qpid')
-        self.config(notification_driver='messaging')
-        self.config(notifier_strategy='rabbit')
-        notify = notifier.Notifier()
-        self.assertEqual(str(notify._transport._driver._url),
-                         'rabbit:///')
-
-        self.config(notifier_strategy='default')
-        notify = notifier.Notifier()
-        self.assertEqual(str(notify._transport._driver._url),
-                         'qpid:///')
+    def test_notifier_load(self):
+        self._test_load_strategy(url=None, driver=None)
 
 
 class TestImageNotifications(utils.BaseTestCase):
@@ -196,6 +180,12 @@ class TestImageNotifications(utils.BaseTestCase):
         self.assertIsInstance(images[0], glance.notifier.ImageProxy)
         self.assertEqual(images[0].image, 'images_from_list')
 
+    def test_image_get_data_should_call_next_image_get_data(self):
+        with mock.patch.object(self.image, 'get_data') as get_data_mock:
+            self.image_proxy.get_data()
+
+            self.assertTrue(get_data_mock.called)
+
     def test_image_get_data_notification(self):
         self.image_proxy.size = 10
         data = ''.join(self.image_proxy.get_data())
@@ -265,7 +255,7 @@ class TestImageNotifications(utils.BaseTestCase):
         def data_iterator():
             self.notifier.log = []
             yield 'abcde'
-            raise exception.StorageFull('Modern Major General')
+            raise glance_store.StorageFull(message='Modern Major General')
 
         self.assertRaises(webob.exc.HTTPRequestEntityTooLarge,
                           self.image_proxy.set_data, data_iterator(), 10)
@@ -275,7 +265,7 @@ class TestImageNotifications(utils.BaseTestCase):
         output_log = output_logs[0]
         self.assertEqual(output_log['notification_type'], 'ERROR')
         self.assertEqual(output_log['event_type'], 'image.upload')
-        self.assertTrue('Modern Major General' in output_log['payload'])
+        self.assertIn('Modern Major General', output_log['payload'])
 
     def test_image_set_data_value_error(self):
         def data_iterator():
@@ -292,7 +282,7 @@ class TestImageNotifications(utils.BaseTestCase):
         output_log = output_logs[0]
         self.assertEqual(output_log['notification_type'], 'ERROR')
         self.assertEqual(output_log['event_type'], 'image.upload')
-        self.assertTrue('value wrong' in output_log['payload'])
+        self.assertIn('value wrong', output_log['payload'])
 
     def test_image_set_data_duplicate(self):
         def data_iterator():
@@ -309,13 +299,13 @@ class TestImageNotifications(utils.BaseTestCase):
         output_log = output_logs[0]
         self.assertEqual(output_log['notification_type'], 'ERROR')
         self.assertEqual(output_log['event_type'], 'image.upload')
-        self.assertTrue('Cant have duplicates' in output_log['payload'])
+        self.assertIn('Cant have duplicates', output_log['payload'])
 
     def test_image_set_data_storage_write_denied(self):
         def data_iterator():
             self.notifier.log = []
             yield 'abcde'
-            raise exception.StorageWriteDenied('The Very Model')
+            raise glance_store.StorageWriteDenied(message='The Very Model')
 
         self.assertRaises(webob.exc.HTTPServiceUnavailable,
                           self.image_proxy.set_data, data_iterator(), 10)
@@ -326,7 +316,7 @@ class TestImageNotifications(utils.BaseTestCase):
         output_log = output_logs[0]
         self.assertEqual(output_log['notification_type'], 'ERROR')
         self.assertEqual(output_log['event_type'], 'image.upload')
-        self.assertTrue('The Very Model' in output_log['payload'])
+        self.assertIn('The Very Model', output_log['payload'])
 
     def test_image_set_data_forbidden(self):
         def data_iterator():
@@ -343,7 +333,7 @@ class TestImageNotifications(utils.BaseTestCase):
         output_log = output_logs[0]
         self.assertEqual(output_log['notification_type'], 'ERROR')
         self.assertEqual(output_log['event_type'], 'image.upload')
-        self.assertTrue('Not allowed' in output_log['payload'])
+        self.assertIn('Not allowed', output_log['payload'])
 
     def test_image_set_data_not_found(self):
         def data_iterator():
@@ -360,7 +350,7 @@ class TestImageNotifications(utils.BaseTestCase):
         output_log = output_logs[0]
         self.assertEqual(output_log['notification_type'], 'ERROR')
         self.assertEqual(output_log['event_type'], 'image.upload')
-        self.assertTrue('Not found' in output_log['payload'])
+        self.assertIn('Not found', output_log['payload'])
 
     def test_image_set_data_HTTP_error(self):
         def data_iterator():
@@ -377,7 +367,7 @@ class TestImageNotifications(utils.BaseTestCase):
         output_log = output_logs[0]
         self.assertEqual(output_log['notification_type'], 'ERROR')
         self.assertEqual(output_log['event_type'], 'image.upload')
-        self.assertTrue('Http issue' in output_log['payload'])
+        self.assertIn('Http issue', output_log['payload'])
 
     def test_image_set_data_error(self):
         def data_iterator():
@@ -394,7 +384,7 @@ class TestImageNotifications(utils.BaseTestCase):
         output_log = output_logs[0]
         self.assertEqual(output_log['notification_type'], 'ERROR')
         self.assertEqual(output_log['event_type'], 'image.upload')
-        self.assertTrue('Failed' in output_log['payload'])
+        self.assertIn('Failed', output_log['payload'])
 
 
 class TestTaskNotifications(utils.BaseTestCase):
@@ -402,19 +392,29 @@ class TestTaskNotifications(utils.BaseTestCase):
 
     def setUp(self):
         super(TestTaskNotifications, self).setUp()
-        self.task = TaskStub(
+        task_input = {"loc": "fake"}
+        self.task_stub = TaskStub(
             task_id='aaa',
             task_type='import',
             status='pending',
             owner=TENANT2,
             expires_at=None,
             created_at=DATETIME,
-            updated_at=DATETIME
+            updated_at=DATETIME,
+        )
+
+        self.task = Task(
+            task_id='aaa',
+            task_type='import',
+            status='pending',
+            owner=TENANT2,
+            expires_at=None,
+            created_at=DATETIME,
+            updated_at=DATETIME,
+            task_input=task_input,
+            result='res',
+            message='blah'
         )
-        self.task_details = domain.TaskDetails(task_id=self.task.task_id,
-                                               task_input={"loc": "fake"},
-                                               result='',
-                                               message='')
         self.context = glance.context.RequestContext(
             tenant=TENANT2,
             user=USER1
@@ -431,9 +431,11 @@ class TestTaskNotifications(utils.BaseTestCase):
             self.context,
             self.notifier
         )
-        self.task_details_proxy = notifier.TaskDetailsProxy(self.task_details,
-                                                            self.context,
-                                                            self.notifier)
+        self.task_stub_proxy = glance.notifier.TaskStubProxy(
+            self.task_stub,
+            self.context,
+            self.notifier
+        )
         self.patcher = mock.patch.object(timeutils, 'utcnow')
         mock_utcnow = self.patcher.start()
         mock_utcnow.return_value = datetime.datetime.utcnow()
@@ -443,7 +445,7 @@ class TestTaskNotifications(utils.BaseTestCase):
         self.patcher.stop()
 
     def test_task_create_notification(self):
-        self.task_repo_proxy.add(self.task_proxy, self.task_details_proxy)
+        self.task_repo_proxy.add(self.task_stub_proxy)
         output_logs = self.notifier.get_logs()
         self.assertEqual(len(output_logs), 1)
         output_log = output_logs[0]
@@ -463,7 +465,7 @@ class TestTaskNotifications(utils.BaseTestCase):
 
     def test_task_delete_notification(self):
         now = timeutils.isotime()
-        self.task_repo_proxy.remove(self.task_proxy)
+        self.task_repo_proxy.remove(self.task_stub_proxy)
         output_logs = self.notifier.get_logs()
         self.assertEqual(len(output_logs), 1)
         output_log = output_logs[0]
@@ -486,7 +488,10 @@ class TestTaskNotifications(utils.BaseTestCase):
             self.fail('Notification contained location field.')
 
     def test_task_run_notification(self):
-        self.task_proxy.run(executor=None)
+        with mock.patch('glance.async.TaskExecutor') as mock_executor:
+            executor = mock_executor.return_value
+            executor._run.return_value = mock.Mock()
+            self.task_proxy.run(executor=mock_executor)
         output_logs = self.notifier.get_logs()
         self.assertEqual(len(output_logs), 1)
         output_log = output_logs[0]
