@@ -44,10 +44,19 @@ class ImageRepoStub(object):
 
 
 class ImageStub(object):
-    def __init__(self, image_id, visibility='private'):
+    def __init__(self, image_id=None, visibility='private',
+                 container_format='bear', disk_format='raw',
+                 status='active', extra_properties=None):
+
+        if extra_properties is None:
+            extra_properties = {}
+
         self.image_id = image_id
         self.visibility = visibility
-        self.status = 'active'
+        self.container_format = container_format
+        self.disk_format = disk_format
+        self.status = status
+        self.extra_properties = extra_properties
 
     def delete(self):
         self.status = 'deleted'
@@ -69,7 +78,7 @@ class MemberRepoStub(object):
     def get(self, *args, **kwargs):
         return 'member_repo_get'
 
-    def save(self, image_member, from_state=None):
+    def save(self, image_member):
         image_member.output = 'member_repo_save'
 
     def list(self, *args, **kwargs):
@@ -85,13 +94,13 @@ class ImageMembershipStub(object):
 
 
 class TaskRepoStub(object):
-    def get_task_and_details(self, *args, **kwargs):
-        return 'task_from_get', 'task_details_from_get'
+    def get(self, *args, **kwargs):
+        return 'task_from_get'
 
     def add(self, *args, **kwargs):
         return 'task_from_add'
 
-    def list_tasks(self, *args, **kwargs):
+    def list(self, *args, **kwargs):
         return ['task_from_list_0', 'task_from_list_1']
 
 
@@ -294,11 +303,19 @@ class TestImagePolicy(test_utils.BaseTestCase):
         image_factory.new_image(visibility='public')
         self.policy.enforce.assert_called_once_with({}, "publicize_image", {})
 
-    def test_image_get_data(self):
+    def test_image_get_data_policy_enforced_with_target(self):
+        extra_properties = {
+            'test_key': 'test_4321'
+        }
+        image_stub = ImageStub(UUID1, extra_properties=extra_properties)
+        image = glance.api.policy.ImageProxy(image_stub, {}, self.policy)
         self.policy.enforce.side_effect = exception.Forbidden
-        image = glance.api.policy.ImageProxy(self.image_stub, {}, self.policy)
+        glance.api.policy.ImageTarget = mock.Mock()
+        target = glance.api.policy.ImageTarget(image)
+
         self.assertRaises(exception.Forbidden, image.get_data)
-        self.policy.enforce.assert_called_once_with({}, "download_image", {})
+        self.policy.enforce.assert_called_once_with({}, "download_image",
+                                                    target=target)
 
     def test_image_set_data(self):
         self.policy.enforce.side_effect = exception.Forbidden
@@ -386,7 +403,7 @@ class TestTaskPolicy(test_utils.BaseTestCase):
             self.policy
         )
         self.assertRaises(exception.Forbidden,
-                          task_repo.get_task_and_details,
+                          task_repo.get,
                           UUID1)
 
     def test_get_task_allowed(self):
@@ -397,32 +414,32 @@ class TestTaskPolicy(test_utils.BaseTestCase):
             {},
             self.policy
         )
-        task, task_details = task_repo.get_task_and_details(UUID1)
+        task = task_repo.get(UUID1)
         self.assertIsInstance(task, glance.api.policy.TaskProxy)
         self.assertEqual(task.task, 'task_from_get')
 
     def test_get_tasks_not_allowed(self):
         rules = {"get_tasks": False}
         self.policy.set_rules(rules)
-        task_repo = glance.api.policy.TaskRepoProxy(
+        task_repo = glance.api.policy.TaskStubRepoProxy(
             self.task_repo_stub,
             {},
             self.policy
         )
-        self.assertRaises(exception.Forbidden, task_repo.list_tasks)
+        self.assertRaises(exception.Forbidden, task_repo.list)
 
     def test_get_tasks_allowed(self):
         rules = {"get_task": True}
         self.policy.set_rules(rules)
-        task_repo = glance.api.policy.TaskRepoProxy(
+        task_repo = glance.api.policy.TaskStubRepoProxy(
             self.task_repo_stub,
             {},
             self.policy
         )
-        tasks = task_repo.list_tasks()
+        tasks = task_repo.list()
         for i, task in enumerate(tasks):
-            self.assertIsInstance(task, glance.api.policy.TaskProxy)
-            self.assertEqual(task.task, 'task_from_list_%d' % i)
+            self.assertIsInstance(task, glance.api.policy.TaskStubProxy)
+            self.assertEqual(task.task_stub, 'task_from_list_%d' % i)
 
     def test_add_task_not_allowed(self):
         rules = {"add_task": False}
