@@ -20,8 +20,9 @@ import uuid
 import six
 
 from glance.common import exception
+from glance.common import store_utils
+from glance.openstack.common import units
 import glance.quota
-import glance.store
 from glance.tests.unit import utils as unit_test_utils
 from glance.tests import utils as test_utils
 
@@ -44,6 +45,9 @@ class FakeImage(object):
         for d in data:
             self.size += len(d)
 
+    def __init__(self, **kwargs):
+        self.extra_properties = kwargs.get('extra_properties', {})
+
 
 class TestImageQuota(test_utils.BaseTestCase):
     def setUp(self):
@@ -55,14 +59,16 @@ class TestImageQuota(test_utils.BaseTestCase):
     def _get_image(self, location_count=1, image_size=10):
         context = FakeContext()
         db_api = unit_test_utils.FakeDB()
+        store_api = unit_test_utils.FakeStoreAPI()
+        store = unit_test_utils.FakeStoreUtils(store_api)
         base_image = FakeImage()
         base_image.image_id = 'xyz'
         base_image.size = image_size
-        image = glance.quota.ImageProxy(base_image, context, db_api)
+        image = glance.quota.ImageProxy(base_image, context, db_api, store)
         locations = []
         for i in range(location_count):
             locations.append({'url': 'file:///g/there/it/is%d' % i,
-                              'metadata': {}})
+                              'metadata': {}, 'status': 'active'})
         image_values = {'id': 'xyz', 'owner': context.owner,
                         'status': 'active', 'size': image_size,
                         'locations': locations}
@@ -71,32 +77,65 @@ class TestImageQuota(test_utils.BaseTestCase):
 
     def test_quota_allowed(self):
         quota = 10
-        self.config(user_storage_quota=quota)
+        self.config(user_storage_quota=str(quota))
         context = FakeContext()
         db_api = unit_test_utils.FakeDB()
+        store_api = unit_test_utils.FakeStoreAPI()
+        store = unit_test_utils.FakeStoreUtils(store_api)
         base_image = FakeImage()
         base_image.image_id = 'id'
-        image = glance.quota.ImageProxy(base_image, context, db_api)
+        image = glance.quota.ImageProxy(base_image, context, db_api, store)
         data = '*' * quota
         base_image.set_data(data, size=None)
         image.set_data(data)
         self.assertEqual(quota, base_image.size)
 
+    def _test_quota_allowed_unit(self, data_length, config_quota):
+        self.config(user_storage_quota=config_quota)
+        context = FakeContext()
+        db_api = unit_test_utils.FakeDB()
+        store_api = unit_test_utils.FakeStoreAPI()
+        store = unit_test_utils.FakeStoreUtils(store_api)
+        base_image = FakeImage()
+        base_image.image_id = 'id'
+        image = glance.quota.ImageProxy(base_image, context, db_api, store)
+        data = '*' * data_length
+        base_image.set_data(data, size=None)
+        image.set_data(data)
+        self.assertEqual(data_length, base_image.size)
+
+    def test_quota_allowed_unit_b(self):
+        self._test_quota_allowed_unit(10, '10B')
+
+    def test_quota_allowed_unit_kb(self):
+        self._test_quota_allowed_unit(10, '1KB')
+
+    def test_quota_allowed_unit_mb(self):
+        self._test_quota_allowed_unit(10, '1MB')
+
+    def test_quota_allowed_unit_gb(self):
+        self._test_quota_allowed_unit(10, '1GB')
+
+    def test_quota_allowed_unit_tb(self):
+        self._test_quota_allowed_unit(10, '1TB')
+
     def _quota_exceeded_size(self, quota, data,
                              deleted=True, size=None):
         self.config(user_storage_quota=quota)
         context = FakeContext()
         db_api = unit_test_utils.FakeDB()
+        store_api = unit_test_utils.FakeStoreAPI()
+        store = unit_test_utils.FakeStoreUtils(store_api)
         base_image = FakeImage()
         base_image.image_id = 'id'
-        image = glance.quota.ImageProxy(base_image, context, db_api)
+        image = glance.quota.ImageProxy(base_image, context, db_api, store)
 
         if deleted:
-            with patch.object(glance.store, 'safe_delete_from_backend'):
-                glance.store.safe_delete_from_backend(
+            with patch.object(store_utils, 'safe_delete_from_backend'):
+                store_utils.safe_delete_from_backend(
                     context,
-                    base_image.locations[0]['url'],
-                    image.image_id)
+                    image.image_id,
+                    base_image.locations[0])
 
         self.assertRaises(exception.StorageQuotaFull,
                           image.set_data,
@@ -113,20 +152,35 @@ class TestImageQuota(test_utils.BaseTestCase):
         # That's why 'get_remaining_quota' is mocked with return_value = 0.
         with patch.object(glance.api.common, 'get_remaining_quota',
                           return_value=0):
-            self._quota_exceeded_size(quota, data)
+            self._quota_exceeded_size(str(quota), data)
 
     def test_quota_exceeded_with_right_size(self):
         quota = 10
         data = '*' * (quota + 1)
-        self._quota_exceeded_size(quota, data, size=len(data), deleted=False)
+        self._quota_exceeded_size(str(quota), data, size=len(data),
+                                  deleted=False)
+
+    def test_quota_exceeded_with_right_size_b(self):
+        quota = 10
+        data = '*' * (quota + 1)
+        self._quota_exceeded_size('10B', data, size=len(data),
+                                  deleted=False)
+
+    def test_quota_exceeded_with_right_size_kb(self):
+        quota = units.Ki
+        data = '*' * (quota + 1)
+        self._quota_exceeded_size('1KB', data, size=len(data),
+                                  deleted=False)
 
     def test_quota_exceeded_with_lie_size(self):
         quota = 10
         data = '*' * (quota + 1)
-        self._quota_exceeded_size(quota, data, deleted=False, size=quota - 1)
+        self._quota_exceeded_size(str(quota), data, deleted=False,
+                                  size=quota - 1)
 
     def test_append_location(self):
-        new_location = {'url': 'file:///a/path', 'metadata': {}}
+        new_location = {'url': 'file:///a/path', 'metadata': {},
+                        'status': 'active'}
         image = self._get_image()
         pre_add_locations = image.locations[:]
         image.locations.append(new_location)
@@ -134,7 +188,8 @@ class TestImageQuota(test_utils.BaseTestCase):
         self.assertEqual(image.locations, pre_add_locations)
 
     def test_insert_location(self):
-        new_location = {'url': 'file:///a/path', 'metadata': {}}
+        new_location = {'url': 'file:///a/path', 'metadata': {},
+                        'status': 'active'}
         image = self._get_image()
         pre_add_locations = image.locations[:]
         image.locations.insert(0, new_location)
@@ -142,7 +197,8 @@ class TestImageQuota(test_utils.BaseTestCase):
         self.assertEqual(image.locations, pre_add_locations)
 
     def test_extend_location(self):
-        new_location = {'url': 'file:///a/path', 'metadata': {}}
+        new_location = {'url': 'file:///a/path', 'metadata': {},
+                        'status': 'active'}
         image = self._get_image()
         pre_add_locations = image.locations[:]
         image.locations.extend([new_location])
@@ -150,7 +206,8 @@ class TestImageQuota(test_utils.BaseTestCase):
         self.assertEqual(image.locations, pre_add_locations)
 
     def test_iadd_location(self):
-        new_location = {'url': 'file:///a/path', 'metadata': {}}
+        new_location = {'url': 'file:///a/path', 'metadata': {},
+                        'status': 'active'}
         image = self._get_image()
         pre_add_locations = image.locations[:]
         image.locations += [new_location]
@@ -158,14 +215,15 @@ class TestImageQuota(test_utils.BaseTestCase):
         self.assertEqual(image.locations, pre_add_locations)
 
     def test_set_location(self):
-        new_location = {'url': 'file:///a/path', 'metadata': {}}
+        new_location = {'url': 'file:///a/path', 'metadata': {},
+                        'status': 'active'}
         image = self._get_image()
         image.locations = [new_location]
         self.assertEqual(image.locations, [new_location])
 
     def _make_image_with_quota(self, image_size=10, location_count=2):
         quota = image_size * location_count
-        self.config(user_storage_quota=quota)
+        self.config(user_storage_quota=str(quota))
         return self._get_image(image_size=image_size,
                                location_count=location_count)
 
@@ -173,30 +231,36 @@ class TestImageQuota(test_utils.BaseTestCase):
         image = self._make_image_with_quota()
         self.assertRaises(exception.StorageQuotaFull,
                           image.locations.append,
-                          {'url': 'file:///a/path', 'metadata': {}})
+                          {'url': 'file:///a/path', 'metadata': {},
+                           'status': 'active'})
 
     def test_exceed_insert_location(self):
         image = self._make_image_with_quota()
         self.assertRaises(exception.StorageQuotaFull,
                           image.locations.insert,
                           0,
-                          {'url': 'file:///a/path', 'metadata': {}})
+                          {'url': 'file:///a/path', 'metadata': {},
+                           'status': 'active'})
 
     def test_exceed_extend_location(self):
         image = self._make_image_with_quota()
         self.assertRaises(exception.StorageQuotaFull,
                           image.locations.extend,
-                          [{'url': 'file:///a/path', 'metadata': {}}])
+                          [{'url': 'file:///a/path', 'metadata': {},
+                            'status': 'active'}])
 
     def test_set_location_under(self):
         image = self._make_image_with_quota(location_count=1)
-        image.locations = [{'url': 'file:///a/path', 'metadata': {}}]
+        image.locations = [{'url': 'file:///a/path', 'metadata': {},
+                            'status': 'active'}]
 
     def test_set_location_exceed(self):
         image = self._make_image_with_quota(location_count=1)
         try:
-            image.locations = [{'url': 'file:///a/path', 'metadata': {}},
-                               {'url': 'file:///a/path2', 'metadata': {}}]
+            image.locations = [{'url': 'file:///a/path', 'metadata': {},
+                                'status': 'active'},
+                               {'url': 'file:///a/path2', 'metadata': {},
+                                'status': 'active'}]
             self.fail('Should have raised the quota exception')
         except exception.StorageQuotaFull:
             pass
@@ -204,7 +268,8 @@ class TestImageQuota(test_utils.BaseTestCase):
     def test_iadd_location_exceed(self):
         image = self._make_image_with_quota(location_count=1)
         try:
-            image.locations += [{'url': 'file:///a/path', 'metadata': {}}]
+            image.locations += [{'url': 'file:///a/path', 'metadata': {},
+                                 'status': 'active'}]
             self.fail('Should have raised the quota exception')
         except exception.StorageQuotaFull:
             pass
@@ -212,12 +277,14 @@ class TestImageQuota(test_utils.BaseTestCase):
     def test_append_location_for_queued_image(self):
         context = FakeContext()
         db_api = unit_test_utils.FakeDB()
+        store_api = unit_test_utils.FakeStoreAPI()
+        store = unit_test_utils.FakeStoreUtils(store_api)
         base_image = FakeImage()
         base_image.image_id = str(uuid.uuid4())
-        image = glance.quota.ImageProxy(base_image, context, db_api)
+        image = glance.quota.ImageProxy(base_image, context, db_api, store)
         self.assertIsNone(image.size)
 
-        self.stubs.Set(glance.store, 'get_size_from_backend',
+        self.stubs.Set(store_api, 'get_size_from_backend',
                        unit_test_utils.fake_get_size_from_backend)
         image.locations.append({'url': 'file:///fake.img.tar.gz',
                                 'metadata': {}})
@@ -227,12 +294,14 @@ class TestImageQuota(test_utils.BaseTestCase):
     def test_insert_location_for_queued_image(self):
         context = FakeContext()
         db_api = unit_test_utils.FakeDB()
+        store_api = unit_test_utils.FakeStoreAPI()
+        store = unit_test_utils.FakeStoreUtils(store_api)
         base_image = FakeImage()
         base_image.image_id = str(uuid.uuid4())
-        image = glance.quota.ImageProxy(base_image, context, db_api)
+        image = glance.quota.ImageProxy(base_image, context, db_api, store)
         self.assertIsNone(image.size)
 
-        self.stubs.Set(glance.store, 'get_size_from_backend',
+        self.stubs.Set(store_api, 'get_size_from_backend',
                        unit_test_utils.fake_get_size_from_backend)
         image.locations.insert(0,
                                {'url': 'file:///fake.img.tar.gz',
@@ -243,12 +312,14 @@ class TestImageQuota(test_utils.BaseTestCase):
     def test_set_location_for_queued_image(self):
         context = FakeContext()
         db_api = unit_test_utils.FakeDB()
+        store_api = unit_test_utils.FakeStoreAPI()
+        store = unit_test_utils.FakeStoreUtils(store_api)
         base_image = FakeImage()
         base_image.image_id = str(uuid.uuid4())
-        image = glance.quota.ImageProxy(base_image, context, db_api)
+        image = glance.quota.ImageProxy(base_image, context, db_api, store)
         self.assertIsNone(image.size)
 
-        self.stubs.Set(glance.store, 'get_size_from_backend',
+        self.stubs.Set(store_api, 'get_size_from_backend',
                        unit_test_utils.fake_get_size_from_backend)
         image.locations = [{'url': 'file:///fake.img.tar.gz', 'metadata': {}}]
         self.assertEqual([{'url': 'file:///fake.img.tar.gz', 'metadata': {}}],
@@ -257,12 +328,14 @@ class TestImageQuota(test_utils.BaseTestCase):
     def test_iadd_location_for_queued_image(self):
         context = FakeContext()
         db_api = unit_test_utils.FakeDB()
+        store_api = unit_test_utils.FakeStoreAPI()
+        store = unit_test_utils.FakeStoreUtils(store_api)
         base_image = FakeImage()
         base_image.image_id = str(uuid.uuid4())
-        image = glance.quota.ImageProxy(base_image, context, db_api)
+        image = glance.quota.ImageProxy(base_image, context, db_api, store)
         self.assertIsNone(image.size)
 
-        self.stubs.Set(glance.store, 'get_size_from_backend',
+        self.stubs.Set(store_api, 'get_size_from_backend',
                        unit_test_utils.fake_get_size_from_backend)
         image.locations += [{'url': 'file:///fake.img.tar.gz', 'metadata': {}}]
         self.assertIn({'url': 'file:///fake.img.tar.gz', 'metadata': {}},
@@ -272,16 +345,20 @@ class TestImageQuota(test_utils.BaseTestCase):
 class TestImagePropertyQuotas(test_utils.BaseTestCase):
     def setUp(self):
         super(TestImagePropertyQuotas, self).setUp()
-        self.base_image = mock.Mock()
+        self.base_image = FakeImage()
         self.image = glance.quota.ImageProxy(self.base_image,
+                                             mock.Mock(),
                                              mock.Mock(),
                                              mock.Mock())
 
         self.image_repo_mock = mock.Mock()
+        self.image_repo_mock.add.return_value = self.base_image
+        self.image_repo_mock.save.return_value = self.base_image
 
         self.image_repo_proxy = glance.quota.ImageRepoProxy(
             self.image_repo_mock,
             mock.Mock(),
+            mock.Mock(),
             mock.Mock())
 
     def test_save_image_with_image_property(self):
@@ -290,8 +367,7 @@ class TestImagePropertyQuotas(test_utils.BaseTestCase):
         self.image.extra_properties = {'foo': 'bar'}
         self.image_repo_proxy.save(self.image)
 
-        self.image_repo_mock.save.assert_called_once_with(self.base_image,
-                                                          from_state=None)
+        self.image_repo_mock.save.assert_called_once_with(self.base_image)
 
     def test_save_image_too_many_image_properties(self):
         self.config(image_property_quota=1)
@@ -299,7 +375,7 @@ class TestImagePropertyQuotas(test_utils.BaseTestCase):
         self.image.extra_properties = {'foo': 'bar', 'foo2': 'bar2'}
         exc = self.assertRaises(exception.ImagePropertyLimitExceeded,
                                 self.image_repo_proxy.save, self.image)
-        self.assertTrue("Attempted: 2, Maximum: 1" in six.text_type(exc))
+        self.assertIn("Attempted: 2, Maximum: 1", six.text_type(exc))
 
     def test_save_image_unlimited_image_properties(self):
         self.config(image_property_quota=-1)
@@ -307,8 +383,7 @@ class TestImagePropertyQuotas(test_utils.BaseTestCase):
         self.image.extra_properties = {'foo': 'bar'}
         self.image_repo_proxy.save(self.image)
 
-        self.image_repo_mock.save.assert_called_once_with(self.base_image,
-                                                          from_state=None)
+        self.image_repo_mock.save.assert_called_once_with(self.base_image)
 
     def test_add_image_with_image_property(self):
         self.config(image_property_quota=1)
@@ -324,7 +399,7 @@ class TestImagePropertyQuotas(test_utils.BaseTestCase):
         self.image.extra_properties = {'foo': 'bar', 'foo2': 'bar2'}
         exc = self.assertRaises(exception.ImagePropertyLimitExceeded,
                                 self.image_repo_proxy.add, self.image)
-        self.assertTrue("Attempted: 2, Maximum: 1" in six.text_type(exc))
+        self.assertIn("Attempted: 2, Maximum: 1", six.text_type(exc))
 
     def test_add_image_unlimited_image_properties(self):
         self.config(image_property_quota=-1)
@@ -334,13 +409,77 @@ class TestImagePropertyQuotas(test_utils.BaseTestCase):
 
         self.image_repo_mock.add.assert_called_once_with(self.base_image)
 
+    def _quota_exceed_setup(self):
+        self.config(image_property_quota=2)
+        self.base_image.extra_properties = {'foo': 'bar', 'spam': 'ham'}
+        self.image = glance.quota.ImageProxy(self.base_image,
+                                             mock.Mock(),
+                                             mock.Mock(),
+                                             mock.Mock())
+
+    def test_modify_image_properties_when_quota_exceeded(self):
+        self._quota_exceed_setup()
+        self.config(image_property_quota=1)
+        self.image.extra_properties = {'foo': 'frob', 'spam': 'eggs'}
+        self.image_repo_proxy.save(self.image)
+        self.image_repo_mock.save.assert_called_once_with(self.base_image)
+        self.assertEqual('frob', self.base_image.extra_properties['foo'])
+        self.assertEqual('eggs', self.base_image.extra_properties['spam'])
+
+    def test_delete_image_properties_when_quota_exceeded(self):
+        self._quota_exceed_setup()
+        self.config(image_property_quota=1)
+        del self.image.extra_properties['foo']
+        self.image_repo_proxy.save(self.image)
+        self.image_repo_mock.save.assert_called_once_with(self.base_image)
+        self.assertNotIn('foo', self.base_image.extra_properties)
+        self.assertEqual('ham', self.base_image.extra_properties['spam'])
+
+    def test_exceed_quota_during_patch_operation(self):
+        self._quota_exceed_setup()
+        self.image.extra_properties['frob'] = 'baz'
+        self.image.extra_properties['lorem'] = 'ipsum'
+        self.assertEqual('bar', self.base_image.extra_properties['foo'])
+        self.assertEqual('ham', self.base_image.extra_properties['spam'])
+        self.assertEqual('baz', self.base_image.extra_properties['frob'])
+        self.assertEqual('ipsum', self.base_image.extra_properties['lorem'])
+
+        del self.image.extra_properties['frob']
+        del self.image.extra_properties['lorem']
+        self.image_repo_proxy.save(self.image)
+        call_args = mock.call(self.base_image)
+        self.assertEqual(call_args, self.image_repo_mock.save.call_args)
+        self.assertEqual('bar', self.base_image.extra_properties['foo'])
+        self.assertEqual('ham', self.base_image.extra_properties['spam'])
+        self.assertNotIn('frob', self.base_image.extra_properties)
+        self.assertNotIn('lorem', self.base_image.extra_properties)
+
+    def test_quota_exceeded_after_delete_image_properties(self):
+        self.config(image_property_quota=3)
+        self.base_image.extra_properties = {'foo': 'bar',
+                                            'spam': 'ham',
+                                            'frob': 'baz'}
+        self.image = glance.quota.ImageProxy(self.base_image,
+                                             mock.Mock(),
+                                             mock.Mock(),
+                                             mock.Mock())
+        self.config(image_property_quota=1)
+        del self.image.extra_properties['foo']
+        self.image_repo_proxy.save(self.image)
+        self.image_repo_mock.save.assert_called_once_with(self.base_image)
+        self.assertNotIn('foo', self.base_image.extra_properties)
+        self.assertEqual('ham', self.base_image.extra_properties['spam'])
+        self.assertEqual('baz', self.base_image.extra_properties['frob'])
+
 
 class TestImageTagQuotas(test_utils.BaseTestCase):
     def setUp(self):
         super(TestImageTagQuotas, self).setUp()
         self.base_image = mock.Mock()
         self.base_image.tags = set([])
+        self.base_image.extra_properties = {}
         self.image = glance.quota.ImageProxy(self.base_image,
+                                             mock.Mock(),
                                              mock.Mock(),
                                              mock.Mock())
 
@@ -348,6 +487,7 @@ class TestImageTagQuotas(test_utils.BaseTestCase):
         self.image_repo_proxy = glance.quota.ImageRepoProxy(
             self.image_repo_mock,
             mock.Mock(),
+            mock.Mock(),
             mock.Mock())
 
     def test_replace_image_tag(self):
@@ -360,7 +500,7 @@ class TestImageTagQuotas(test_utils.BaseTestCase):
 
         exc = self.assertRaises(exception.ImageTagLimitExceeded,
                                 setattr, self.image, 'tags', ['foo', 'bar'])
-        self.assertTrue('Attempted: 2, Maximum: 0' in six.text_type(exc))
+        self.assertIn('Attempted: 2, Maximum: 0', six.text_type(exc))
         self.assertEqual(len(self.image.tags), 0)
 
     def test_replace_unlimited_image_tags(self):
@@ -378,7 +518,7 @@ class TestImageTagQuotas(test_utils.BaseTestCase):
         self.image.tags.add('foo')
         exc = self.assertRaises(exception.ImageTagLimitExceeded,
                                 self.image.tags.add, 'bar')
-        self.assertTrue('Attempted: 2, Maximum: 1' in six.text_type(exc))
+        self.assertIn('Attempted: 2, Maximum: 1', six.text_type(exc))
 
     def test_add_unlimited_image_tags(self):
         self.config(image_tag_quota=-1)
@@ -401,14 +541,14 @@ class TestQuotaImageTagsProxy(test_utils.BaseTestCase):
     def test_add(self):
         proxy = glance.quota.QuotaImageTagsProxy(set([]))
         proxy.add('foo')
-        self.assertTrue('foo' in proxy)
+        self.assertIn('foo', proxy)
 
     def test_add_too_many_tags(self):
         self.config(image_tag_quota=0)
         proxy = glance.quota.QuotaImageTagsProxy(set([]))
         exc = self.assertRaises(exception.ImageTagLimitExceeded,
                                 proxy.add, 'bar')
-        self.assertTrue('Attempted: 1, Maximum: 0' in six.text_type(exc))
+        self.assertIn('Attempted: 1, Maximum: 0', six.text_type(exc))
 
     def test_equals(self):
         proxy = glance.quota.QuotaImageTagsProxy(set([]))
@@ -416,7 +556,7 @@ class TestQuotaImageTagsProxy(test_utils.BaseTestCase):
 
     def test_contains(self):
         proxy = glance.quota.QuotaImageTagsProxy(set(['foo']))
-        self.assertTrue('foo' in proxy)
+        self.assertIn('foo', proxy)
 
     def test_len(self):
         proxy = glance.quota.QuotaImageTagsProxy(set(['foo',
@@ -438,12 +578,14 @@ class TestImageMemberQuotas(test_utils.BaseTestCase):
     def setUp(self):
         super(TestImageMemberQuotas, self).setUp()
         db_api = unit_test_utils.FakeDB()
+        store_api = unit_test_utils.FakeStoreAPI()
+        store = unit_test_utils.FakeStoreUtils(store_api)
         context = FakeContext()
         self.image = mock.Mock()
         self.base_image_member_factory = mock.Mock()
         self.image_member_factory = glance.quota.ImageMemberFactoryProxy(
             self.base_image_member_factory, context,
-            db_api)
+            db_api, store)
 
     def test_new_image_member(self):
         self.config(image_member_quota=1)
@@ -475,7 +617,9 @@ class TestImageLocationQuotas(test_utils.BaseTestCase):
         self.base_image = mock.Mock()
         self.base_image.locations = []
         self.base_image.size = 1
+        self.base_image.extra_properties = {}
         self.image = glance.quota.ImageProxy(self.base_image,
+                                             mock.Mock(),
                                              mock.Mock(),
                                              mock.Mock())
 
@@ -483,6 +627,7 @@ class TestImageLocationQuotas(test_utils.BaseTestCase):
         self.image_repo_proxy = glance.quota.ImageRepoProxy(
             self.image_repo_mock,
             mock.Mock(),
+            mock.Mock(),
             mock.Mock())
 
     def test_replace_image_location(self):
@@ -504,7 +649,7 @@ class TestImageLocationQuotas(test_utils.BaseTestCase):
         ]
         exc = self.assertRaises(exception.ImageLocationLimitExceeded,
                                 setattr, self.image, 'locations', locations)
-        self.assertTrue('Attempted: 3, Maximum: 1' in six.text_type(exc))
+        self.assertIn('Attempted: 3, Maximum: 1', six.text_type(exc))
         self.assertEqual(len(self.image.locations), 1)
 
     def test_replace_unlimited_image_locations(self):
@@ -527,7 +672,7 @@ class TestImageLocationQuotas(test_utils.BaseTestCase):
         location2 = {"url": "file:///fake2.img.tar.gz", "metadata": {}}
         exc = self.assertRaises(exception.ImageLocationLimitExceeded,
                                 self.image.locations.append, location2)
-        self.assertTrue('Attempted: 2, Maximum: 1' in six.text_type(exc))
+        self.assertIn('Attempted: 2, Maximum: 1', six.text_type(exc))
 
     def test_add_unlimited_image_locations(self):
         self.config(image_location_quota=-1)
