@@ -14,9 +14,10 @@
 #    under the License.
 import mox
 
+import glance_store
+
 from glance.common import exception
-import glance.store
-from glance.tests.unit import base as unit_test_base
+import glance.location
 from glance.tests.unit import utils as unit_test_utils
 from glance.tests import utils
 
@@ -34,7 +35,7 @@ class ImageRepoStub(object):
     def add(self, image):
         return image
 
-    def save(self, image, from_state=None):
+    def save(self, image):
         return image
 
 
@@ -82,33 +83,38 @@ class FakeMemberRepo(object):
 class TestStoreImage(utils.BaseTestCase):
     def setUp(self):
         locations = [{'url': '%s/%s' % (BASE_URI, UUID1),
-                      'metadata': {}}]
+                      'metadata': {}, 'status': 'active'}]
         self.image_stub = ImageStub(UUID1, 'active', locations)
         self.store_api = unit_test_utils.FakeStoreAPI()
+        self.store_utils = unit_test_utils.FakeStoreUtils(self.store_api)
         super(TestStoreImage, self).setUp()
 
     def test_image_delete(self):
-        image = glance.store.ImageProxy(self.image_stub, {}, self.store_api)
+        image = glance.location.ImageProxy(self.image_stub, {},
+                                           self.store_api, self.store_utils)
         location = image.locations[0]
         self.assertEqual(image.status, 'active')
-        self.store_api.get_from_backend({}, location['url'])
+        self.store_api.get_from_backend(location['url'], context={})
         image.delete()
         self.assertEqual(image.status, 'deleted')
-        self.assertRaises(exception.NotFound,
-                          self.store_api.get_from_backend, {}, location['url'])
+        self.assertRaises(glance_store.NotFound,
+                          self.store_api.get_from_backend, location['url'], {})
 
     def test_image_get_data(self):
-        image = glance.store.ImageProxy(self.image_stub, {}, self.store_api)
+        image = glance.location.ImageProxy(self.image_stub, {},
+                                           self.store_api, self.store_utils)
         self.assertEqual(image.get_data(), 'XXX')
 
     def test_image_get_data_from_second_location(self):
-        def fake_get_from_backend(self, context, location):
+        def fake_get_from_backend(self, location, offset=0,
+                                  chunk_size=None, context=None):
             if UUID1 in location:
                 raise Exception('not allow download from %s' % location)
             else:
                 return self.data[location]
 
-        image1 = glance.store.ImageProxy(self.image_stub, {}, self.store_api)
+        image1 = glance.location.ImageProxy(self.image_stub, {},
+                                            self.store_api, self.store_utils)
         self.assertEqual(image1.get_data(), 'XXX')
         # Multiple location support
         context = glance.context.RequestContext(user=USER1)
@@ -131,7 +137,8 @@ class TestStoreImage(utils.BaseTestCase):
     def test_image_set_data(self):
         context = glance.context.RequestContext(user=USER1)
         image_stub = ImageStub(UUID2, status='queued', locations=[])
-        image = glance.store.ImageProxy(image_stub, context, self.store_api)
+        image = glance.location.ImageProxy(image_stub, context,
+                                           self.store_api, self.store_utils)
         image.set_data('YYYY', 4)
         self.assertEqual(image.size, 4)
         #NOTE(markwash): FakeStore returns image_id for location
@@ -144,7 +151,9 @@ class TestStoreImage(utils.BaseTestCase):
         image_stub = ImageStub(UUID2, status='queued', locations=[])
         loc_meta = {'key': 'value5032'}
         store_api = unit_test_utils.FakeStoreAPI(store_metadata=loc_meta)
-        image = glance.store.ImageProxy(image_stub, context, store_api)
+        store_utils = unit_test_utils.FakeStoreUtils(store_api)
+        image = glance.location.ImageProxy(image_stub, context,
+                                           store_api, store_utils)
         image.set_data('YYYY', 4)
         self.assertEqual(image.size, 4)
         location_data = image.locations[0]
@@ -154,14 +163,15 @@ class TestStoreImage(utils.BaseTestCase):
         self.assertEqual(image.status, 'active')
         image.delete()
         self.assertEqual(image.status, 'deleted')
-        self.assertRaises(exception.NotFound,
-                          self.store_api.get_from_backend, {},
-                          image.locations[0]['url'])
+        self.assertRaises(glance_store.NotFound,
+                          self.store_api.get_from_backend,
+                          image.locations[0]['url'], {})
 
     def test_image_set_data_unknown_size(self):
         context = glance.context.RequestContext(user=USER1)
         image_stub = ImageStub(UUID2, status='queued', locations=[])
-        image = glance.store.ImageProxy(image_stub, context, self.store_api)
+        image = glance.location.ImageProxy(image_stub, context,
+                                           self.store_api, self.store_utils)
         image.set_data('YYYY', None)
         self.assertEqual(image.size, 4)
         #NOTE(markwash): FakeStore returns image_id for location
@@ -170,18 +180,18 @@ class TestStoreImage(utils.BaseTestCase):
         self.assertEqual(image.status, 'active')
         image.delete()
         self.assertEqual(image.status, 'deleted')
-        self.assertRaises(exception.NotFound,
-                          self.store_api.get_from_backend, {},
-                          image.locations[0]['url'])
+        self.assertRaises(glance_store.NotFound,
+                          self.store_api.get_from_backend,
+                          image.locations[0]['url'], context={})
 
     def _add_image(self, context, image_id, data, len):
         image_stub = ImageStub(image_id, status='queued', locations=[])
-        image = glance.store.ImageProxy(image_stub,
-                                        context, self.store_api)
+        image = glance.location.ImageProxy(image_stub, context,
+                                           self.store_api, self.store_utils)
         image.set_data(data, len)
         self.assertEqual(image.size, len)
         #NOTE(markwash): FakeStore returns image_id for location
-        location = {'url': image_id, 'metadata': {}}
+        location = {'url': image_id, 'metadata': {}, 'status': 'active'}
         self.assertEqual(image.locations, [location])
         self.assertEqual(image_stub.locations, [location])
         self.assertEqual(image.status, 'active')
@@ -200,7 +210,7 @@ class TestStoreImage(utils.BaseTestCase):
         image1.delete()
 
         self.assertEqual(len(self.store_api.data.keys()), 2)
-        self.assertFalse(UUID2 in self.store_api.data.keys())
+        self.assertNotIn(UUID2, self.store_api.data.keys())
 
     def test_image_change_append_invalid_location_metatdata(self):
         UUID3 = 'a8a61ec4-d7a3-11e2-8c28-000c29c27581'
@@ -217,15 +227,15 @@ class TestStoreImage(utils.BaseTestCase):
         # check below cases within 'TestStoreMetaDataChecker'.
         location_bad = {'url': UUID3, 'metadata': "a invalid metadata"}
 
-        self.assertRaises(glance.store.BackendException,
+        self.assertRaises(glance_store.BackendException,
                           image1.locations.append, location_bad)
 
         image1.delete()
         image2.delete()
 
         self.assertEqual(len(self.store_api.data.keys()), 2)
-        self.assertFalse(UUID2 in self.store_api.data.keys())
-        self.assertFalse(UUID3 in self.store_api.data.keys())
+        self.assertNotIn(UUID2, self.store_api.data.keys())
+        self.assertNotIn(UUID3, self.store_api.data.keys())
 
     def test_image_change_append_locations(self):
         UUID3 = 'a8a61ec4-d7a3-11e2-8c28-000c29c27581'
@@ -235,8 +245,8 @@ class TestStoreImage(utils.BaseTestCase):
         (image1, image_stub1) = self._add_image(context, UUID2, 'XXXX', 4)
         (image2, image_stub2) = self._add_image(context, UUID3, 'YYYY', 4)
 
-        location2 = {'url': UUID2, 'metadata': {}}
-        location3 = {'url': UUID3, 'metadata': {}}
+        location2 = {'url': UUID2, 'metadata': {}, 'status': 'active'}
+        location3 = {'url': UUID3, 'metadata': {}, 'status': 'active'}
 
         image1.locations.append(location3)
 
@@ -246,8 +256,8 @@ class TestStoreImage(utils.BaseTestCase):
         image1.delete()
 
         self.assertEqual(len(self.store_api.data.keys()), 2)
-        self.assertFalse(UUID2 in self.store_api.data.keys())
-        self.assertFalse(UUID3 in self.store_api.data.keys())
+        self.assertNotIn(UUID2, self.store_api.data.keys())
+        self.assertNotIn(UUID3, self.store_api.data.keys())
 
         image2.delete()
 
@@ -259,8 +269,8 @@ class TestStoreImage(utils.BaseTestCase):
         (image1, image_stub1) = self._add_image(context, UUID2, 'XXXX', 4)
         (image2, image_stub2) = self._add_image(context, UUID3, 'YYYY', 4)
 
-        location2 = {'url': UUID2, 'metadata': {}}
-        location3 = {'url': UUID3, 'metadata': {}}
+        location2 = {'url': UUID2, 'metadata': {}, 'status': 'active'}
+        location3 = {'url': UUID3, 'metadata': {}, 'status': 'active'}
 
         image1.locations.append(location3)
 
@@ -275,8 +285,8 @@ class TestStoreImage(utils.BaseTestCase):
         image1.delete()
 
         self.assertEqual(len(self.store_api.data.keys()), 2)
-        self.assertFalse(UUID2 in self.store_api.data.keys())
-        self.assertFalse(UUID3 in self.store_api.data.keys())
+        self.assertNotIn(UUID2, self.store_api.data.keys())
+        self.assertNotIn(UUID3, self.store_api.data.keys())
 
         image2.delete()
 
@@ -294,7 +304,7 @@ class TestStoreImage(utils.BaseTestCase):
         image1.delete()
 
         self.assertEqual(len(self.store_api.data.keys()), 2)
-        self.assertFalse(UUID2 in self.store_api.data.keys())
+        self.assertNotIn(UUID2, self.store_api.data.keys())
 
     def test_image_change_extend_invalid_locations_metadata(self):
         UUID3 = 'a8a61ec4-d7a3-11e2-8c28-000c29c27581'
@@ -306,15 +316,15 @@ class TestStoreImage(utils.BaseTestCase):
 
         location_bad = {'url': UUID3, 'metadata': "a invalid metadata"}
 
-        self.assertRaises(glance.store.BackendException,
+        self.assertRaises(glance_store.BackendException,
                           image1.locations.extend, [location_bad])
 
         image1.delete()
         image2.delete()
 
         self.assertEqual(len(self.store_api.data.keys()), 2)
-        self.assertFalse(UUID2 in self.store_api.data.keys())
-        self.assertFalse(UUID3 in self.store_api.data.keys())
+        self.assertNotIn(UUID2, self.store_api.data.keys())
+        self.assertNotIn(UUID3, self.store_api.data.keys())
 
     def test_image_change_extend_locations(self):
         UUID3 = 'a8a61ec4-d7a3-11e2-8c28-000c29c27581'
@@ -324,8 +334,8 @@ class TestStoreImage(utils.BaseTestCase):
         (image1, image_stub1) = self._add_image(context, UUID2, 'XXXX', 4)
         (image2, image_stub2) = self._add_image(context, UUID3, 'YYYY', 4)
 
-        location2 = {'url': UUID2, 'metadata': {}}
-        location3 = {'url': UUID3, 'metadata': {}}
+        location2 = {'url': UUID2, 'metadata': {}, 'status': 'active'}
+        location3 = {'url': UUID3, 'metadata': {}, 'status': 'active'}
 
         image1.locations.extend([location3])
 
@@ -335,8 +345,8 @@ class TestStoreImage(utils.BaseTestCase):
         image1.delete()
 
         self.assertEqual(len(self.store_api.data.keys()), 2)
-        self.assertFalse(UUID2 in self.store_api.data.keys())
-        self.assertFalse(UUID3 in self.store_api.data.keys())
+        self.assertNotIn(UUID2, self.store_api.data.keys())
+        self.assertNotIn(UUID3, self.store_api.data.keys())
 
         image2.delete()
 
@@ -348,8 +358,8 @@ class TestStoreImage(utils.BaseTestCase):
         (image1, image_stub1) = self._add_image(context, UUID2, 'XXXX', 4)
         (image2, image_stub2) = self._add_image(context, UUID3, 'YYYY', 4)
 
-        location2 = {'url': UUID2, 'metadata': {}}
-        location3 = {'url': UUID3, 'metadata': {}}
+        location2 = {'url': UUID2, 'metadata': {}, 'status': 'active'}
+        location3 = {'url': UUID3, 'metadata': {}, 'status': 'active'}
         location_bad = {'url': 'unknown://location', 'metadata': {}}
 
         image1.locations.extend([location3])
@@ -364,8 +374,8 @@ class TestStoreImage(utils.BaseTestCase):
         image2.delete()
 
         self.assertEqual(len(self.store_api.data.keys()), 2)
-        self.assertFalse(UUID2 in self.store_api.data.keys())
-        self.assertFalse(UUID3 in self.store_api.data.keys())
+        self.assertNotIn(UUID2, self.store_api.data.keys())
+        self.assertNotIn(UUID3, self.store_api.data.keys())
 
     def test_image_change_delete_location(self):
         self.assertEqual(len(self.store_api.data.keys()), 2)
@@ -379,7 +389,7 @@ class TestStoreImage(utils.BaseTestCase):
         self.assertEqual(len(image1.locations), 0)
 
         self.assertEqual(len(self.store_api.data.keys()), 2)
-        self.assertFalse(UUID2 in self.store_api.data.keys())
+        self.assertNotIn(UUID2, self.store_api.data.keys())
 
         image1.delete()
 
@@ -396,7 +406,7 @@ class TestStoreImage(utils.BaseTestCase):
         image1.delete()
 
         self.assertEqual(len(self.store_api.data.keys()), 2)
-        self.assertFalse(UUID2 in self.store_api.data.keys())
+        self.assertNotIn(UUID2, self.store_api.data.keys())
 
     def test_image_change_insert_invalid_location_metadata(self):
         UUID3 = 'a8a61ec4-d7a3-11e2-8c28-000c29c27581'
@@ -408,15 +418,15 @@ class TestStoreImage(utils.BaseTestCase):
 
         location_bad = {'url': UUID3, 'metadata': "a invalid metadata"}
 
-        self.assertRaises(glance.store.BackendException,
+        self.assertRaises(glance_store.BackendException,
                           image1.locations.insert, 0, location_bad)
 
         image1.delete()
         image2.delete()
 
         self.assertEqual(len(self.store_api.data.keys()), 2)
-        self.assertFalse(UUID2 in self.store_api.data.keys())
-        self.assertFalse(UUID3 in self.store_api.data.keys())
+        self.assertNotIn(UUID2, self.store_api.data.keys())
+        self.assertNotIn(UUID3, self.store_api.data.keys())
 
     def test_image_change_insert_location(self):
         UUID3 = 'a8a61ec4-d7a3-11e2-8c28-000c29c27581'
@@ -426,8 +436,8 @@ class TestStoreImage(utils.BaseTestCase):
         (image1, image_stub1) = self._add_image(context, UUID2, 'XXXX', 4)
         (image2, image_stub2) = self._add_image(context, UUID3, 'YYYY', 4)
 
-        location2 = {'url': UUID2, 'metadata': {}}
-        location3 = {'url': UUID3, 'metadata': {}}
+        location2 = {'url': UUID2, 'metadata': {}, 'status': 'active'}
+        location3 = {'url': UUID3, 'metadata': {}, 'status': 'active'}
 
         image1.locations.insert(0, location3)
 
@@ -437,8 +447,8 @@ class TestStoreImage(utils.BaseTestCase):
         image1.delete()
 
         self.assertEqual(len(self.store_api.data.keys()), 2)
-        self.assertFalse(UUID2 in self.store_api.data.keys())
-        self.assertFalse(UUID3 in self.store_api.data.keys())
+        self.assertNotIn(UUID2, self.store_api.data.keys())
+        self.assertNotIn(UUID3, self.store_api.data.keys())
 
         image2.delete()
 
@@ -467,15 +477,16 @@ class TestStoreImage(utils.BaseTestCase):
         image2.delete()
 
         self.assertEqual(len(self.store_api.data.keys()), 2)
-        self.assertFalse(UUID2 in self.store_api.data.keys())
-        self.assertFalse(UUID3 in self.store_api.data.keys())
+        self.assertNotIn(UUID2, self.store_api.data.keys())
+        self.assertNotIn(UUID3, self.store_api.data.keys())
 
     def test_image_change_adding_invalid_location_uri(self):
         self.assertEqual(len(self.store_api.data.keys()), 2)
 
         context = glance.context.RequestContext(user=USER1)
         image_stub1 = ImageStub('fake_image_id', status='queued', locations=[])
-        image1 = glance.store.ImageProxy(image_stub1, context, self.store_api)
+        image1 = glance.location.ImageProxy(image_stub1, context,
+                                            self.store_api, self.store_utils)
 
         location_bad = {'url': 'unknown://location', 'metadata': {}}
 
@@ -487,7 +498,7 @@ class TestStoreImage(utils.BaseTestCase):
         image1.delete()
 
         self.assertEqual(len(self.store_api.data.keys()), 2)
-        self.assertFalse(UUID2 in self.store_api.data.keys())
+        self.assertNotIn(UUID2, self.store_api.data.keys())
 
     def test_image_change_adding_invalid_location_metadata(self):
         self.assertEqual(len(self.store_api.data.keys()), 2)
@@ -496,11 +507,12 @@ class TestStoreImage(utils.BaseTestCase):
         (image1, image_stub1) = self._add_image(context, UUID2, 'XXXX', 4)
 
         image_stub2 = ImageStub('fake_image_id', status='queued', locations=[])
-        image2 = glance.store.ImageProxy(image_stub2, context, self.store_api)
+        image2 = glance.location.ImageProxy(image_stub2, context,
+                                            self.store_api, self.store_utils)
 
         location_bad = {'url': UUID2, 'metadata': "a invalid metadata"}
 
-        self.assertRaises(glance.store.BackendException,
+        self.assertRaises(glance_store.BackendException,
                           image2.locations.__iadd__, [location_bad])
         self.assertEqual(image_stub2.locations, [])
         self.assertEqual(image2.locations, [])
@@ -509,7 +521,7 @@ class TestStoreImage(utils.BaseTestCase):
         image2.delete()
 
         self.assertEqual(len(self.store_api.data.keys()), 2)
-        self.assertFalse(UUID2 in self.store_api.data.keys())
+        self.assertNotIn(UUID2, self.store_api.data.keys())
 
     def test_image_change_adding_locations(self):
         UUID3 = 'a8a61ec4-d7a3-11e2-8c28-000c29c27581'
@@ -520,7 +532,8 @@ class TestStoreImage(utils.BaseTestCase):
         (image2, image_stub2) = self._add_image(context, UUID3, 'YYYY', 4)
 
         image_stub3 = ImageStub('fake_image_id', status='queued', locations=[])
-        image3 = glance.store.ImageProxy(image_stub3, context, self.store_api)
+        image3 = glance.location.ImageProxy(image_stub3, context,
+                                            self.store_api, self.store_utils)
 
         location2 = {'url': UUID2, 'metadata': {}}
         location3 = {'url': UUID3, 'metadata': {}}
@@ -533,8 +546,8 @@ class TestStoreImage(utils.BaseTestCase):
         image3.delete()
 
         self.assertEqual(len(self.store_api.data.keys()), 2)
-        self.assertFalse(UUID2 in self.store_api.data.keys())
-        self.assertFalse(UUID3 in self.store_api.data.keys())
+        self.assertNotIn(UUID2, self.store_api.data.keys())
+        self.assertNotIn(UUID3, self.store_api.data.keys())
 
         image1.delete()
         image2.delete()
@@ -547,7 +560,9 @@ class TestStoreImage(utils.BaseTestCase):
         (image1, image_stub1) = self._add_image(context, UUID2, 'XXXX', 4)
         (image2, image_stub2) = self._add_image(context, UUID3, 'YYYY', 4)
         image_stub3 = ImageStub('fake_image_id', status='queued', locations=[])
-        image3 = glance.store.ImageProxy(image_stub3, context, self.store_api)
+
+        image3 = glance.location.ImageProxy(image_stub3, context,
+                                            self.store_api, self.store_utils)
 
         location2 = {'url': UUID2, 'metadata': {}}
         location3 = {'url': UUID3, 'metadata': {}}
@@ -559,8 +574,8 @@ class TestStoreImage(utils.BaseTestCase):
         image3.delete()
 
         self.assertEqual(len(self.store_api.data.keys()), 2)
-        self.assertFalse(UUID2 in self.store_api.data.keys())
-        self.assertFalse(UUID3 in self.store_api.data.keys())
+        self.assertNotIn(UUID2, self.store_api.data.keys())
+        self.assertNotIn(UUID3, self.store_api.data.keys())
 
         image1.delete()
         image2.delete()
@@ -573,7 +588,8 @@ class TestStoreImage(utils.BaseTestCase):
         (image1, image_stub1) = self._add_image(context, UUID2, 'XXXX', 4)
         (image2, image_stub2) = self._add_image(context, UUID3, 'YYYY', 4)
         image_stub3 = ImageStub('fake_image_id', status='queued', locations=[])
-        image3 = glance.store.ImageProxy(image_stub3, context, self.store_api)
+        image3 = glance.location.ImageProxy(image_stub3, context,
+                                            self.store_api, self.store_utils)
 
         location2 = {'url': UUID2, 'metadata': {}}
         location3 = {'url': UUID3, 'metadata': {}}
@@ -586,8 +602,8 @@ class TestStoreImage(utils.BaseTestCase):
         image3.delete()
 
         self.assertEqual(len(self.store_api.data.keys()), 2)
-        self.assertFalse(UUID2 in self.store_api.data.keys())
-        self.assertFalse(UUID3 in self.store_api.data.keys())
+        self.assertNotIn(UUID2, self.store_api.data.keys())
+        self.assertNotIn(UUID3, self.store_api.data.keys())
 
         image1.delete()
         image2.delete()
@@ -601,7 +617,8 @@ class TestStoreImage(utils.BaseTestCase):
         (image2, image_stub2) = self._add_image(context, UUID3, 'YYYY', 4)
 
         image_stub3 = ImageStub('fake_image_id', status='queued', locations=[])
-        image3 = glance.store.ImageProxy(image_stub3, context, self.store_api)
+        image3 = glance.location.ImageProxy(image_stub3, context,
+                                            self.store_api, self.store_utils)
 
         location2 = {'url': UUID2, 'metadata': {}}
         location3 = {'url': UUID3, 'metadata': {}}
@@ -609,14 +626,14 @@ class TestStoreImage(utils.BaseTestCase):
 
         image3.locations += [location2, location3]
 
-        self.assertTrue(location3 in image_stub3.locations)
-        self.assertFalse(location_bad in image_stub3.locations)
+        self.assertIn(location3, image_stub3.locations)
+        self.assertNotIn(location_bad, image_stub3.locations)
 
         image3.delete()
 
         self.assertEqual(len(self.store_api.data.keys()), 2)
-        self.assertFalse(UUID2 in self.store_api.data.keys())
-        self.assertFalse(UUID3 in self.store_api.data.keys())
+        self.assertNotIn(UUID2, self.store_api.data.keys())
+        self.assertNotIn(UUID3, self.store_api.data.keys())
 
         image1.delete()
         image2.delete()
@@ -633,7 +650,8 @@ class TestStoreImage(utils.BaseTestCase):
         location3 = {'url': UUID3, 'metadata': {}}
 
         image_stub3 = ImageStub('fake_image_id', status='queued', locations=[])
-        image3 = glance.store.ImageProxy(image_stub3, context, self.store_api)
+        image3 = glance.location.ImageProxy(image_stub3, context,
+                                            self.store_api, self.store_utils)
         image3.locations += [location2, location3]
 
         image_stub3.locations.reverse()
@@ -644,8 +662,8 @@ class TestStoreImage(utils.BaseTestCase):
         image3.delete()
 
         self.assertEqual(len(self.store_api.data.keys()), 2)
-        self.assertFalse(UUID2 in self.store_api.data.keys())
-        self.assertFalse(UUID3 in self.store_api.data.keys())
+        self.assertNotIn(UUID2, self.store_api.data.keys())
+        self.assertNotIn(UUID3, self.store_api.data.keys())
 
         image1.delete()
         image2.delete()
@@ -655,16 +673,20 @@ class TestStoreImageRepo(utils.BaseTestCase):
     def setUp(self):
         super(TestStoreImageRepo, self).setUp()
         self.store_api = unit_test_utils.FakeStoreAPI()
+        store_utils = unit_test_utils.FakeStoreUtils(self.store_api)
         self.image_stub = ImageStub(UUID1)
-        self.image = glance.store.ImageProxy(self.image_stub,
-                                             {}, self.store_api)
+        self.image = glance.location.ImageProxy(self.image_stub, {},
+                                                self.store_api, store_utils)
         self.image_repo_stub = ImageRepoStub()
-        self.image_repo = glance.store.ImageRepoProxy(self.image_repo_stub,
-                                                      {}, self.store_api)
+        self.image_repo = glance.location.ImageRepoProxy(self.image_repo_stub,
+                                                         {}, self.store_api,
+                                                         store_utils)
 
     def test_add_updates_acls(self):
-        self.image_stub.locations = [{'url': 'foo', 'metadata': {}},
-                                     {'url': 'bar', 'metadata': {}}]
+        self.image_stub.locations = [{'url': 'foo', 'metadata': {},
+                                      'status': 'active'},
+                                     {'url': 'bar', 'metadata': {},
+                                      'status': 'active'}]
         self.image_stub.visibility = 'public'
         self.image_repo.add(self.image)
         self.assertTrue(self.store_api.acls['foo']['public'])
@@ -681,12 +703,14 @@ class TestStoreImageRepo(utils.BaseTestCase):
         self.assertEqual(len(self.store_api.acls), 0)
 
     def test_save_updates_acls(self):
-        self.image_stub.locations = [{'url': 'foo', 'metadata': {}}]
+        self.image_stub.locations = [{'url': 'foo', 'metadata': {},
+                                      'status': 'active'}]
         self.image_repo.save(self.image)
         self.assertIn('foo', self.store_api.acls)
 
     def test_add_fetches_members_if_private(self):
-        self.image_stub.locations = [{'url': 'glue', 'metadata': {}}]
+        self.image_stub.locations = [{'url': 'glue', 'metadata': {},
+                                      'status': 'active'}]
         self.image_stub.visibility = 'private'
         self.image_repo.add(self.image)
         self.assertIn('glue', self.store_api.acls)
@@ -696,7 +720,8 @@ class TestStoreImageRepo(utils.BaseTestCase):
         self.assertEqual(acls['read'], [TENANT1, TENANT2])
 
     def test_save_fetches_members_if_private(self):
-        self.image_stub.locations = [{'url': 'glue', 'metadata': {}}]
+        self.image_stub.locations = [{'url': 'glue', 'metadata': {},
+                                      'status': 'active'}]
         self.image_stub.visibility = 'private'
         self.image_repo.save(self.image)
         self.assertIn('glue', self.store_api.acls)
@@ -706,7 +731,8 @@ class TestStoreImageRepo(utils.BaseTestCase):
         self.assertEqual(acls['read'], [TENANT1, TENANT2])
 
     def test_member_addition_updates_acls(self):
-        self.image_stub.locations = [{'url': 'glug', 'metadata': {}}]
+        self.image_stub.locations = [{'url': 'glug', 'metadata': {},
+                                      'status': 'active'}]
         self.image_stub.visibility = 'private'
         member_repo = self.image.get_member_repo()
         membership = glance.domain.ImageMembership(
@@ -719,7 +745,8 @@ class TestStoreImageRepo(utils.BaseTestCase):
         self.assertEqual(acls['read'], [TENANT1, TENANT2, TENANT3])
 
     def test_member_removal_updates_acls(self):
-        self.image_stub.locations = [{'url': 'glug', 'metadata': {}}]
+        self.image_stub.locations = [{'url': 'glug', 'metadata': {},
+                                      'status': 'active'}]
         self.image_stub.visibility = 'private'
         member_repo = self.image.get_member_repo()
         membership = glance.domain.ImageMembership(
@@ -732,19 +759,22 @@ class TestStoreImageRepo(utils.BaseTestCase):
         self.assertEqual(acls['read'], [TENANT2])
 
 
-class TestImageFactory(unit_test_base.StoreClearingUnitTest):
+class TestImageFactory(utils.BaseTestCase):
 
     def setUp(self):
         super(TestImageFactory, self).setUp()
-        self.image_factory = glance.store.ImageFactoryProxy(
+        store_api = unit_test_utils.FakeStoreAPI()
+        store_utils = unit_test_utils.FakeStoreUtils(store_api)
+        self.image_factory = glance.location.ImageFactoryProxy(
             ImageFactoryStub(),
             glance.context.RequestContext(user=USER1),
-            unit_test_utils.FakeStoreAPI())
+            store_api,
+            store_utils)
 
     def test_new_image(self):
         image = self.image_factory.new_image()
-        self.assertTrue(image.image_id is None)
-        self.assertTrue(image.status is None)
+        self.assertIsNone(image.image_id)
+        self.assertIsNone(image.status)
         self.assertEqual(image.visibility, 'private')
         self.assertEqual(image.locations, [])
 
@@ -762,43 +792,43 @@ class TestImageFactory(unit_test_base.StoreClearingUnitTest):
 class TestStoreMetaDataChecker(utils.BaseTestCase):
 
     def test_empty(self):
-        glance.store.check_location_metadata({})
+        glance_store.check_location_metadata({})
 
     def test_unicode(self):
         m = {'key': u'somevalue'}
-        glance.store.check_location_metadata(m)
+        glance_store.check_location_metadata(m)
 
     def test_unicode_list(self):
         m = {'key': [u'somevalue', u'2']}
-        glance.store.check_location_metadata(m)
+        glance_store.check_location_metadata(m)
 
     def test_unicode_dict(self):
         inner = {'key1': u'somevalue', 'key2': u'somevalue'}
         m = {'topkey': inner}
-        glance.store.check_location_metadata(m)
+        glance_store.check_location_metadata(m)
 
     def test_unicode_dict_list(self):
         inner = {'key1': u'somevalue', 'key2': u'somevalue'}
         m = {'topkey': inner, 'list': [u'somevalue', u'2'], 'u': u'2'}
-        glance.store.check_location_metadata(m)
+        glance_store.check_location_metadata(m)
 
     def test_nested_dict(self):
         inner = {'key1': u'somevalue', 'key2': u'somevalue'}
         inner = {'newkey': inner}
         inner = {'anotherkey': inner}
         m = {'topkey': inner}
-        glance.store.check_location_metadata(m)
+        glance_store.check_location_metadata(m)
 
     def test_simple_bad(self):
         m = {'key1': object()}
-        self.assertRaises(glance.store.BackendException,
-                          glance.store.check_location_metadata,
+        self.assertRaises(glance_store.BackendException,
+                          glance_store.check_location_metadata,
                           m)
 
     def test_list_bad(self):
         m = {'key1': [u'somevalue', object()]}
-        self.assertRaises(glance.store.BackendException,
-                          glance.store.check_location_metadata,
+        self.assertRaises(glance_store.BackendException,
+                          glance_store.check_location_metadata,
                           m)
 
     def test_nested_dict_bad(self):
@@ -807,8 +837,8 @@ class TestStoreMetaDataChecker(utils.BaseTestCase):
         inner = {'anotherkey': inner}
         m = {'topkey': inner}
 
-        self.assertRaises(glance.store.BackendException,
-                          glance.store.check_location_metadata,
+        self.assertRaises(glance_store.BackendException,
+                          glance_store.check_location_metadata,
                           m)
 
 
@@ -828,36 +858,36 @@ class TestStoreAddToBackend(utils.BaseTestCase):
         self.mox.UnsetStubs()
 
     def _bad_metadata(self, in_metadata):
-        store = self.mox.CreateMockAnything()
-        store.add(self.image_id, mox.IgnoreArg(), self.size).AndReturn(
+        mstore = self.mox.CreateMockAnything()
+        mstore.add(self.image_id, mox.IgnoreArg(), self.size).AndReturn(
             (self.location, self.size, self.checksum, in_metadata))
-        store.__str__ = lambda: "hello"
-        store.__unicode__ = lambda: "hello"
+        mstore.__str__ = lambda: "hello"
+        mstore.__unicode__ = lambda: "hello"
 
         self.mox.ReplayAll()
 
-        self.assertRaises(glance.store.BackendException,
-                          glance.store.store_add_to_backend,
+        self.assertRaises(glance_store.BackendException,
+                          glance_store.store_add_to_backend,
                           self.image_id,
                           self.data,
                           self.size,
-                          store)
+                          mstore)
         self.mox.VerifyAll()
 
     def _good_metadata(self, in_metadata):
 
-        store = self.mox.CreateMockAnything()
-        store.add(self.image_id, mox.IgnoreArg(), self.size).AndReturn(
+        mstore = self.mox.CreateMockAnything()
+        mstore.add(self.image_id, mox.IgnoreArg(), self.size).AndReturn(
             (self.location, self.size, self.checksum, in_metadata))
 
         self.mox.ReplayAll()
         (location,
          size,
          checksum,
-         metadata) = glance.store.store_add_to_backend(self.image_id,
+         metadata) = glance_store.store_add_to_backend(self.image_id,
                                                        self.data,
                                                        self.size,
-                                                       store)
+                                                       mstore)
         self.mox.VerifyAll()
         self.assertEqual(self.location, location)
         self.assertEqual(self.size, size)
@@ -912,8 +942,8 @@ class TestStoreAddToBackend(utils.BaseTestCase):
 
         self.mox.ReplayAll()
 
-        self.assertRaises(glance.store.BackendException,
-                          glance.store.store_add_to_backend,
+        self.assertRaises(glance_store.BackendException,
+                          glance_store.store_add_to_backend,
                           self.image_id,
                           self.data,
                           self.size,
