@@ -16,11 +16,13 @@
 import mock
 import uuid
 
+import glance_store
 import six
 import webob
 
 import glance.api.v2.image_data
 from glance.common import exception
+from glance.common import wsgi
 
 from glance.tests.unit import base
 import glance.tests.unit.utils as unit_test_utils
@@ -60,7 +62,7 @@ class FakeImage(object):
         else:
             self._status = value
 
-    def get_data(self):
+    def get_data(self, *args, **kwargs):
         return self.data
 
     def set_data(self, data, size=None):
@@ -79,7 +81,7 @@ class FakeImageRepo(object):
         else:
             return self.result
 
-    def save(self, image, from_state=None):
+    def save(self, image):
         self.saved_image = image
 
 
@@ -103,7 +105,9 @@ class TestImagesController(base.StoreClearingUnitTest):
 
     def test_download(self):
         request = unit_test_utils.get_fake_request()
-        image = FakeImage('abcd', locations=['http://example.com/image'])
+        image = FakeImage('abcd',
+                          locations=[{'url': 'http://example.com/image',
+                                      'metadata': {}, 'status': 'active'}])
         self.image_repo.result = image
         image = self.controller.download(request, unit_test_utils.UUID1)
         self.assertEqual(image.image_id, 'abcd')
@@ -180,21 +184,32 @@ class TestImagesController(base.StoreClearingUnitTest):
                           request, unit_test_utils.UUID1, 'YYYY', 4)
 
     def test_upload_non_existent_image_during_save_initiates_deletion(self):
-        def fake_save_not_found(self):
+        def fake_save(self):
             raise exception.NotFound()
 
-        def fake_save_conflict(self):
-            raise exception.Conflict()
+        request = unit_test_utils.get_fake_request()
+        image = FakeImage('abcd', locations=['http://example.com/image'])
+        self.image_repo.result = image
+        self.image_repo.save = fake_save
+        image.delete = mock.Mock()
+        self.assertRaises(webob.exc.HTTPGone, self.controller.upload,
+                          request, str(uuid.uuid4()), 'ABC', 3)
+        self.assertTrue(image.delete.called)
 
-        for fun in [fake_save_not_found, fake_save_conflict]:
-            request = unit_test_utils.get_fake_request()
-            image = FakeImage('abcd', locations=['http://example.com/image'])
-            self.image_repo.result = image
-            self.image_repo.save = fun
-            image.delete = mock.Mock()
-            self.assertRaises(webob.exc.HTTPGone, self.controller.upload,
-                              request, str(uuid.uuid4()), 'ABC', 3)
-            self.assertTrue(image.delete.called)
+    def test_upload_non_existent_image_raises_not_found_exception(self):
+        def fake_save(self):
+            raise exception.NotFound()
+
+        def fake_delete():
+            raise exception.NotFound()
+
+        request = unit_test_utils.get_fake_request()
+        image = FakeImage('abcd', locations=['http://example.com/image'])
+        self.image_repo.result = image
+        self.image_repo.save = fake_save
+        image.delete = fake_delete
+        self.assertRaises(webob.exc.HTTPGone, self.controller.upload,
+                          request, str(uuid.uuid4()), 'ABC', 3)
 
     def test_upload_non_existent_image_before_save(self):
         request = unit_test_utils.get_fake_request()
@@ -215,7 +230,7 @@ class TestImagesController(base.StoreClearingUnitTest):
     def test_upload_storage_full(self):
         request = unit_test_utils.get_fake_request()
         image = FakeImage()
-        image.set_data = Raise(exception.StorageFull)
+        image.set_data = Raise(glance_store.StorageFull)
         self.image_repo.result = image
         self.assertRaises(webob.exc.HTTPRequestEntityTooLarge,
                           self.controller.upload,
@@ -230,6 +245,13 @@ class TestImagesController(base.StoreClearingUnitTest):
                           self.controller.upload,
                           request, unit_test_utils.UUID1, 'YYYYYYY', 7)
 
+    def test_upload_storage_quota_full(self):
+        request = unit_test_utils.get_fake_request()
+        self.image_repo.result = exception.StorageQuotaFull("message")
+        self.assertRaises(webob.exc.HTTPRequestEntityTooLarge,
+                          self.controller.upload,
+                          request, unit_test_utils.UUID1, 'YYYYYYY', 7)
+
     def test_upload_storage_forbidden(self):
         request = unit_test_utils.get_fake_request(user=unit_test_utils.USER2)
         image = FakeImage()
@@ -238,15 +260,32 @@ class TestImagesController(base.StoreClearingUnitTest):
         self.assertRaises(webob.exc.HTTPForbidden, self.controller.upload,
                           request, unit_test_utils.UUID2, 'YY', 2)
 
+    def test_upload_storage_internal_error(self):
+        request = unit_test_utils.get_fake_request()
+        self.image_repo.result = exception.ServerError()
+        self.assertRaises(exception.ServerError,
+                          self.controller.upload,
+                          request, unit_test_utils.UUID1, 'ABC', 3)
+
     def test_upload_storage_write_denied(self):
         request = unit_test_utils.get_fake_request(user=unit_test_utils.USER3)
         image = FakeImage()
-        image.set_data = Raise(exception.StorageWriteDenied)
+        image.set_data = Raise(glance_store.StorageWriteDenied)
         self.image_repo.result = image
         self.assertRaises(webob.exc.HTTPServiceUnavailable,
                           self.controller.upload,
                           request, unit_test_utils.UUID2, 'YY', 2)
 
+    def test_upload_storage_store_disabled(self):
+        """Test that uploading an image file raises StoreDisabled exception"""
+        request = unit_test_utils.get_fake_request(user=unit_test_utils.USER3)
+        image = FakeImage()
+        image.set_data = Raise(glance_store.StoreAddDisabled)
+        self.image_repo.result = image
+        self.assertRaises(webob.exc.HTTPGone,
+                          self.controller.upload,
+                          request, unit_test_utils.UUID2, 'YY', 2)
+
     def _test_upload_download_prepare_notification(self):
         request = unit_test_utils.get_fake_request()
         self.controller.upload(request, unit_test_utils.UUID2, 'YYYY', 4)
@@ -301,7 +340,7 @@ class TestImagesController(base.StoreClearingUnitTest):
     def test_restore_image_when_upload_failed(self):
         request = unit_test_utils.get_fake_request()
         image = FakeImage('fake')
-        image.set_data = Raise(exception.StorageWriteDenied)
+        image.set_data = Raise(glance_store.StorageWriteDenied)
         self.image_repo.result = image
         self.assertRaises(webob.exc.HTTPServiceUnavailable,
                           self.controller.upload,
@@ -386,7 +425,7 @@ class TestImageDataSerializer(test_utils.BaseTestCase):
         self.serializer = glance.api.v2.image_data.ResponseSerializer()
 
     def test_download(self):
-        request = webob.Request.blank('/')
+        request = wsgi.Request.blank('/')
         request.environ = {}
         response = webob.Response()
         response.request = request
@@ -394,12 +433,12 @@ class TestImageDataSerializer(test_utils.BaseTestCase):
         self.serializer.download(response, image)
         self.assertEqual('ZZZ', response.body)
         self.assertEqual('3', response.headers['Content-Length'])
-        self.assertFalse('Content-MD5' in response.headers)
+        self.assertNotIn('Content-MD5', response.headers)
         self.assertEqual('application/octet-stream',
                          response.headers['Content-Type'])
 
     def test_download_with_checksum(self):
-        request = webob.Request.blank('/')
+        request = wsgi.Request.blank('/')
         request.environ = {}
         response = webob.Response()
         response.request = request
@@ -416,13 +455,13 @@ class TestImageDataSerializer(test_utils.BaseTestCase):
         """Make sure the serializer can return 403 forbidden error instead of
         500 internal server error.
         """
-        def get_data():
+        def get_data(*args, **kwargs):
             raise exception.Forbidden()
 
         self.stubs.Set(glance.api.policy.ImageProxy,
                        'get_data',
                        get_data)
-        request = webob.Request.blank('/')
+        request = wsgi.Request.blank('/')
         request.environ = {}
         response = webob.Response()
         response.request = request
