@@ -82,13 +82,12 @@ class BaseTestCase(testtools.TestCase):
         return dst_file_name
 
     def set_property_protection_rules(self, rules):
-        f = open(self.property_file, 'w')
-        for rule_key in rules.keys():
-            f.write('[%s]\n' % rule_key)
-            for operation in rules[rule_key].keys():
-                roles_str = ','.join(rules[rule_key][operation])
-                f.write('%s = %s\n' % (operation, roles_str))
-        f.close()
+        with open(self.property_file, 'w') as f:
+            for rule_key in rules.keys():
+                f.write('[%s]\n' % rule_key)
+                for operation in rules[rule_key].keys():
+                    roles_str = ','.join(rules[rule_key][operation])
+                    f.write('%s = %s\n' % (operation, roles_str))
 
     def config(self, **kw):
         """
@@ -104,7 +103,7 @@ class BaseTestCase(testtools.TestCase):
         test by the fixtures cleanup process.
         """
         group = kw.pop('group', None)
-        for k, v in kw.iteritems():
+        for k, v in six.iteritems(kw):
             CONF.set_override(k, v, group)
 
 
@@ -445,13 +444,15 @@ class RegistryAPIMixIn(object):
             created_at=created_at, updated_at=updated_at,
             **kwargs)
 
-    def get_api_response_ext(self, http_resp, url='/images', headers={},
+    def get_api_response_ext(self, http_resp, url='/images', headers=None,
                              body=None, method=None, api=None,
                              content_type=None):
         if api is None:
             api = self.api
+        if headers is None:
+            headers = {}
         req = webob.Request.blank(url)
-        for k, v in headers.iteritems():
+        for k, v in six.iteritems(headers):
             req.headers[k] = v
         if method:
             req.method = method
@@ -563,7 +564,9 @@ class HttplibWsgiAdapter(object):
         self.app = app
         self.req = None
 
-    def request(self, method, url, body=None, headers={}):
+    def request(self, method, url, body=None, headers=None):
+        if headers is None:
+            headers = {}
         self.req = webob.Request.blank(url, method=method, headers=headers)
         self.req.body = body
 
