@@ -1,15 +1,27 @@
-#!/usr/bin/python
+#!/usr/bin/env python
+# All Rights Reserved.
+#
+#    Licensed under the Apache License, Version 2.0 (the "License"); you may
+#    not use this file except in compliance with the License. You may obtain
+#    a copy of the License at
+#
+#         http://www.apache.org/licenses/LICENSE-2.0
+#
+#    Unless required by applicable law or agreed to in writing, software
+#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+#    License for the specific language governing permissions and limitations
+#    under the License.
 
 import sys
 
 import keystoneclient.v2_0.client
+from oslo.config import cfg
 
 import glance.context
-from glance.openstack.common import cfg
+import glance.db.sqlalchemy.api as db_api
 import glance.openstack.common.log as logging
 import glance.registry.context
-import glance.db.sqlalchemy.api as db_api
-
 
 LOG = logging.getLogger(__name__)
 LOG.addHandler(logging.StreamHandler())
@@ -68,7 +80,8 @@ if __name__ == "__main__":
         cfg.StrOpt('keystone-admin-user',
                    help='Administrative user\'s id'),
         cfg.StrOpt('keystone-admin-password',
-                   help='Administrative user\'s password'),
+                   help='Administrative user\'s password',
+                   secret=True),
     ]
     config.register_cli_opts(extra_cli_opts)
     config(project='glance', prog='glance-registry')
